@@ -97,6 +97,8 @@ static struct iax_session *session[2];	/* one for calls, the other for registrat
 static int dspaudiofd = -1, iaxaudiofd = -1;
 static char dspaudiofile[50];
 static char iaxaudiofile[50];
+static char dspnowaudiofile[50];
+static char iaxnowaudiofile[50];
 static struct timeval now, lasthangup, nextaudio;	/* target time for next audio */
 static int window = 1000;		/* u-sec prior to target that we will let audio go */
 static int refresh = 60;		/* negotiated refresh */
@@ -154,7 +156,7 @@ printlog(FILE *fp, char *fmt, ...)
 
 
 void
-cleanup()
+cleanup(int sig)
 {
     if (refreshreq && regstate == REGISTERED) {
 	struct iax_event *iaxevent = 0;
@@ -173,11 +175,20 @@ cleanup()
     unlink(devlink);
     close(amaster);
     close(aslave);
-    if (record || replay) {
-	if (dspaudiofd > 0) close(dspaudiofd);
-	if (iaxaudiofd > 0) close(iaxaudiofd);
+    if (record) {
+	if (dspaudiofd > 0) {
+	    close(dspaudiofd);
+	    rename(dspnowaudiofile, dspaudiofile);
+	}
+	if (iaxaudiofd > 0) {
+	    close(iaxaudiofd);
+	    rename(iaxnowaudiofile, iaxaudiofile);
+	}
+    } else if (replay) {
+	close(dspaudiofd);
+	close(iaxaudiofd);
     }
-    _exit(0);
+    _exit(sig);
 }
 
 void
@@ -186,9 +197,9 @@ sighandler(int sig)
     signal(SIGINT, NULL);
     signal(SIGTERM, NULL);
 
-    printlog(LOG_ERROR, "Terminating...\n");
+    printlog(LOG_ERROR, "Terminating on signal %d...\n", sig);
 
-    cleanup();
+    cleanup(sig);
 }
 
 void
@@ -205,7 +216,11 @@ timediff(struct timeval a, struct timeval b)
 {
     struct timeval tv;
     timersub(&a, &b, &tv);
-    return (tv.tv_sec * 1000000 + tv.tv_usec);
+    /* The return value, in microseconds, will overflow
+       a signed 32-bit integer every 36 minutes.  As we don't 
+       need to account for timings that long anyway we cap the
+       timer at 5-minutes. */
+    return (tv.tv_sec > 300 ? 300000000 : tv.tv_sec * 1000000 + tv.tv_usec);
 }
 
 void
@@ -224,6 +239,10 @@ void
 setconfigline(char *line)
 {
     if (strchr(line, '%')) return;	/* don't allow troublesome "%" in configs */
+    /* trim comments */
+    if (strchr(line, ';')) line[strchr(line, ';') - line] = '\0';
+    /* trim trailing whitespace */
+    while (line[strlen(line) - 1] == ' ' || line[strlen(line) - 1] == '\t') line[strlen(line) - 1] = '\0';
     if (strncasecmp(line, "device", 6) == 0) {
 	line += 6;
 	while (*line == '\t' || *line == ' ') line++;
@@ -251,8 +270,8 @@ setconfigline(char *line)
     if (strncasecmp(line, "refresh", 7) == 0) {
 	line += 7;
 	while (*line == '\t' || *line == ' ') line++;
-	printlog(LOG_INFO, "Setting refresh = %d\n", refreshreq);
 	refreshreq = atoi(line);
+	printlog(LOG_INFO, "Setting refresh = %d\n", refreshreq);
     }
     if (strncasecmp(line, "server", 6) == 0) {
 	line += 6;
@@ -317,7 +336,7 @@ readconfig(const char *name)
 	int fd = open(filename, O_RDONLY);
 	if (fd < 0) {
 	    printlog(LOG_ERROR, "Cannot open %s.\n", filename);
-	    cleanup();
+	    cleanup(-1);
 	}
 	int cc, pos = 0;
 	char line[1024];
@@ -346,6 +365,18 @@ at_tx_handler(at_state_t *s, void *user_data, const uint8_t *buf, size_t len)
 	printlog(LOG_ERROR, "Unable to pass the full buffer onto the device file. %zd bytes of %d written: %s\n",
 		 wrote, len, strerror(errno));
 	if (wrote == -1) wrote = 0;	/* nothing was written */
+	/*
+	  The pty has a limited buffer size. (On my system it's 4095 bytes.)
+	  Being here tells us that the pty buffer is full, and that generally
+	  indicates that nothing is reading the tty on the other end.  In
+	  order to make room for future writes we need to flush the pty
+	  buffer.
+	*/
+	if (tcflush(amaster, TCOFLUSH)) {
+	    printlog(LOG_ERROR, "Unable to flush pty buffer: %s\n", strerror(errno));
+	} else {
+	    printlog(LOG_INFO, "Successfully flushed pty buffer\n");
+	}
     }
     return wrote;
 }
@@ -397,11 +428,19 @@ t31_call_control_handler(t31_state_t *s, void *user_data, int op, const char *nu
 		nextaudio.tv_usec -= 1000000;
 	    }
 	    if (record) {
-		unlink(dspaudiofile);
-		unlink(iaxaudiofile);
-		dspaudiofd = open(dspaudiofile, O_WRONLY|O_CREAT, 00660);
-		iaxaudiofd = open(iaxaudiofile, O_WRONLY|O_CREAT, 00660);
+		if (dspaudiofd > 0) {
+		    close(dspaudiofd);
+		    rename(dspnowaudiofile, dspaudiofile);
+		}
+		if (iaxaudiofd > 0) {
+		    close(iaxaudiofd);
+		    rename(iaxnowaudiofile, iaxaudiofile);
+		}
+		dspaudiofd = open(dspnowaudiofile, O_WRONLY|O_CREAT, 00660);
+		iaxaudiofd = open(iaxnowaudiofile, O_WRONLY|O_CREAT, 00660);
 	    } else if (replay) {
+		if (dspaudiofd > 0) close(dspaudiofd);
+		if (iaxaudiofd > 0) close(iaxaudiofd);
 		dspaudiofd = open(dspaudiofile, O_RDONLY);
 		iaxaudiofd = open(iaxaudiofile, O_RDONLY);
 	    }
@@ -410,6 +449,21 @@ t31_call_control_handler(t31_state_t *s, void *user_data, int op, const char *nu
 	    /* Hang up */
 	    printlog(LOG_INFO, "Hanging Up\n");
 	    modemstate = MODEM_ONHOOK;
+	    if (record) {
+		if (dspaudiofd > 0) {
+		    close(dspaudiofd);
+		    rename(dspnowaudiofile, dspaudiofile);
+		}
+		if (iaxaudiofd > 0) {
+		    close(iaxaudiofd);
+		    rename(iaxnowaudiofile, iaxaudiofile);
+		}
+	    } else if (replay) {
+		if (dspaudiofd > 0) close(dspaudiofd);
+		if (iaxaudiofd > 0) close(iaxaudiofd);
+	    }
+	    dspaudiofd = -1;
+	    iaxaudiofd = -1;
 	    gettimeofday(&lasthangup, NULL);
 	    if (phonestate != PHONE_FREED) {
 		iax_hangup(session[0], "Normal disconnect");
@@ -460,6 +514,9 @@ t31_call_control_handler(t31_state_t *s, void *user_data, int op, const char *nu
 	case AT_MODEM_CONTROL_RNG:
 	    /* Ring indicator, ignore it */
 	    break;
+	case AT_MODEM_CONTROL_ONHOOK:
+	    /* Hangup indicator, ignore it */
+	    break;
 	default:
 	    printlog(LOG_ERROR, "Unknown DSP control handler: %d\n", op);
 	    break;
@@ -543,6 +600,7 @@ iaxmodem(const char *config, int nolog)
     lasthangup.tv_sec -= 5;
     char modembuf[DSP_BUFSIZE];
     uint8_t dspbuf[sizeof(int16_t)*VOIP_PACKET_SIZE];
+    int16_t iaxbuf[VOIP_PACKET_SIZE];
     static t31_state_t t31_state;
     int t31buflen;
     int tioflags;
@@ -555,6 +613,8 @@ iaxmodem(const char *config, int nolog)
     int i;
     unsigned int last_ts = 0;
 
+    snprintf(dspnowaudiofile, 49, "/tmp/%s-dsp.raw.recording", regpeer);
+    snprintf(iaxnowaudiofile, 49, "/tmp/%s-iax.raw.recording", regpeer);
     snprintf(dspaudiofile, 49, "/tmp/%s-dsp.raw", regpeer);
     snprintf(iaxaudiofile, 49, "/tmp/%s-iax.raw", regpeer);
 
@@ -607,7 +667,7 @@ iaxmodem(const char *config, int nolog)
 
     if (openpty(&amaster, &aslave, NULL, NULL, NULL)) {
         printlog(LOG_ERROR, "Fatal error: failed to initialize pty\n");
-        cleanup();
+        cleanup(-1);
     }
     char *stty = ttyname(aslave);
     printlog(LOG_INFO, "Opened pty, slave device: %s\n", stty);
@@ -616,7 +676,7 @@ iaxmodem(const char *config, int nolog)
     }
     if (symlink(stty, devlink)) {
 	printlog(LOG_ERROR, "Fatal error: failed to create %s symbolic link\n", devlink);
-	cleanup();
+	cleanup(-1);
     }
     printlog(LOG_INFO, "Created %s symbolic link\n", devlink);
 
@@ -634,7 +694,7 @@ iaxmodem(const char *config, int nolog)
 
     if (fcntl(amaster, F_SETFL, fcntl(amaster, F_GETFL, 0) | O_NONBLOCK)) {
 	printlog(LOG_ERROR, "Cannot set up non-blocking read on %s\n", ttyname(amaster));
-	cleanup();
+	cleanup(-1);
     }
 
     /* Root privileges not needed anymore, drop privs. */
@@ -643,7 +703,7 @@ iaxmodem(const char *config, int nolog)
     
     if ((port = iax_init(port) < 0)) {
 	printlog(LOG_ERROR, "Fatal error: failed to initialize iax with port %d\n", port);
-	cleanup();
+	cleanup(-1);
     }
     iaxnetfd = iax_get_fd();
 
@@ -660,9 +720,9 @@ iaxmodem(const char *config, int nolog)
     iax_disable_debug();
 //    iax_enable_debug();
 
-    if (t31_init(&t31_state, at_tx_handler, NULL, t31_call_control_handler, NULL) < 0) {
+    if (t31_init(&t31_state, at_tx_handler, NULL, t31_call_control_handler, NULL, NULL, NULL) < 0) {
 	printlog(LOG_ERROR, "Cannot initialize the T.31 modem\n");
-	cleanup();
+	cleanup(-1);
     }
 
     /*
@@ -840,8 +900,8 @@ iaxmodem(const char *config, int nolog)
 	    avail = DSP_BUFSIZE;
 	    printlog(LOG_INFO, "strange... tx_in_bytes: %d, tx_out_bytes: %d, DSP_BUFSIZE: %d\n", t31_state.tx_in_bytes, t31_state.tx_out_bytes, DSP_BUFSIZE);
 	}
-	if (((modemstate != MODEM_CONNECTED && selectretval && FD_ISSET(amaster, &select_rfds)) ||
-	     (modemstate == MODEM_CONNECTED && !t31_state.tx_holding)) && avail) {
+	if ((modemstate != MODEM_CONNECTED && selectretval && FD_ISSET(amaster, &select_rfds)) ||
+	    (modemstate == MODEM_CONNECTED && !t31_state.tx_holding && avail)) {
 	    ssize_t len;
 	    do {
 		len = read(amaster, modembuf, avail);
@@ -859,21 +919,6 @@ iaxmodem(const char *config, int nolog)
 		    lastdtedata = now;
 		}
 	    } while (len > 0 && avail > 0);
-	} else if (selectretval && FD_ISSET(amaster, &select_rfds) && avail != 0 ) {
-	    /*
-	     * The DTE is getting ahead of the DSP/DCE.  This shouldn't happen.
-	     *
-	     * Maybe we need to do something to "reset" the DSP.  We could do that
-	     * by t31_reset(&t31_state) followed by a t31_init.
-	     *
-	     * For now we'll just flush the data...
-	     */
-	    ssize_t len;
-	    char flush[1024];
-	    do {
-		len = read(amaster, flush, sizeof(flush));
-		printlog(LOG_ERROR, "The DSP has no room for more data.  Flushing %d bytes.\n", len);
-	    } while (len > 0);
 	}
 	/*
 	 * Is it time to send another RING message to the DTE?
@@ -900,7 +945,7 @@ iaxmodem(const char *config, int nolog)
 	    while ((iaxevent = iax_get_event(0))) {
 		switch (iaxevent->etype) {
 		    case IAX_EVENT_REGACK:
-			printlog(LOG_ERROR, "Registration completed successfully.\n");
+			printlog(LOG_INFO, "Registration completed successfully.\n");
 			if (iaxevent->ies.refresh) refresh = iaxevent->ies.refresh;
 			regstate = REGISTERED;
 			break;
@@ -941,11 +986,19 @@ iaxmodem(const char *config, int nolog)
 			audiobalance = 0;
 			skew = 0;
 			if (record) {
-			    unlink(dspaudiofile);
-			    unlink(iaxaudiofile);
-			    dspaudiofd = open(dspaudiofile, O_WRONLY|O_CREAT, 00660);
-			    iaxaudiofd = open(iaxaudiofile, O_WRONLY|O_CREAT, 00660);
+			    if (dspaudiofd > 0) {
+				close(dspaudiofd);
+				rename(dspnowaudiofile, dspaudiofile);
+			    }
+			    if (iaxaudiofd > 0) {
+				close(iaxaudiofd);
+				rename(iaxnowaudiofile, iaxaudiofile);
+			    }
+			    dspaudiofd = open(dspnowaudiofile, O_WRONLY|O_CREAT, 00660);
+			    iaxaudiofd = open(iaxnowaudiofile, O_WRONLY|O_CREAT, 00660);
 			} else if (replay) {
+			    if (dspaudiofd > 0) close(dspaudiofd);
+			    if (iaxaudiofd > 0) close(iaxaudiofd);
 			    dspaudiofd = open(dspaudiofile, O_RDONLY);
 			    iaxaudiofd = open(iaxaudiofile, O_RDONLY);
 			}
@@ -1017,9 +1070,18 @@ iaxmodem(const char *config, int nolog)
 			    modemstate = MODEM_ONHOOK;
 			}
 			phonestate = PHONE_FREED;
-			if (record || replay) {
-			    close(dspaudiofd);
-			    close(iaxaudiofd);
+			if (record) {
+			    if (dspaudiofd > 0) {
+				close(dspaudiofd);
+				rename(dspnowaudiofile, dspaudiofile);
+			    }
+			    if (iaxaudiofd > 0) {
+				close(iaxaudiofd);
+				rename(iaxnowaudiofile, iaxaudiofile);
+			    }
+			} else if (replay) {
+			    if (dspaudiofd > 0) close(dspaudiofd);
+			    if (iaxaudiofd > 0) close(iaxaudiofd);
 			}
 			dspaudiofd = -1;
 			iaxaudiofd = -1;
@@ -1035,10 +1097,38 @@ iaxmodem(const char *config, int nolog)
 			break;
 		    case IAX_EVENT_VOICE:
 			/*
-			 * We watch for IAX2 jitter... for now just for fun.
+			 * Watch for IAX2 jitter... the 32-bit timestamp shouldn't ever "wrap" 
+			 * unless we expect a single call to last more than a month.
 			 */
-			if (last_ts && iaxevent->ts != last_ts + 20)
-			    printlog(LOG_INFO, "IAX2 jitter - last_ts: %d, ts: %d\n", last_ts, iaxevent->ts);
+			if (last_ts && iaxevent->ts <= last_ts) {
+			    /*
+			     * We've already sent audio corresponding to this time.
+			     */
+			    break;
+			}
+			if (last_ts && iaxevent->ts != last_ts + 20) {
+			    printlog(LOG_ERROR, "IAX2 jitter - last_ts: %d, ts: %d\n", last_ts, iaxevent->ts);
+			    int16_t fillbuf[VOIP_PACKET_SIZE];
+			    while (last_ts + 20 < iaxevent->ts) {
+				/*
+				 * This audio packet skips ahead.  Oops, audio was lost.
+				 * Compensate by repeating the last audio packet enough
+				 * to fill in the missing time.
+				 *
+				 * It is important to make sure that we do not replace a
+				 * period of non-silence with silence and that silence is
+				 * not replaced with non-silence... in order to keep as
+				 * syncrhonous with the remote as possible.
+				 */
+				if (replay) read(iaxaudiofd, fillbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
+				if (modemstate == MODEM_CONNECTED && t31_rx(&t31_state, fillbuf, VOIP_PACKET_SIZE)) {
+				    printlog(LOG_ERROR, "Error sending %d units of IAX make-up audio to DSP.\n", VOIP_PACKET_SIZE);
+				}
+				if (record) write(iaxaudiofd, fillbuf, VOIP_PACKET_SIZE*sizeof(int16_t));
+				last_ts += 20;
+				audiobalance--;
+			    }
+			}
 			last_ts = iaxevent->ts;
 
 			if (modemstate == MODEM_CONNECTED) {
@@ -1055,26 +1145,35 @@ iaxmodem(const char *config, int nolog)
 			    }
 			}
 			/* Here's an audio frame from the IAX server, send it to the DSP. */
-			if (codec == AST_FORMAT_SLINEAR) {
-			    int units = iaxevent->datalen/sizeof(int16_t);
-			    orderbytes((int16_t *) iaxevent->data, units);
-			    if (replay) read(iaxaudiofd, (uint8_t *) iaxevent->data, iaxevent->datalen);
-			    if (modemstate == MODEM_CONNECTED && t31_rx(&t31_state, (int16_t *) iaxevent->data, units)) {
+			{
+			    int units = iaxevent->datalen;
+			    int16_t convertedbuf[iaxevent->datalen];
+			    int16_t* audiodata;
+			    int i = 0;
+			    switch (codec) {
+				case AST_FORMAT_SLINEAR:
+				    units = iaxevent->datalen/sizeof(int16_t);
+				    orderbytes((int16_t *) iaxevent->data, units);
+				    audiodata = (int16_t*) iaxevent->data;
+				    break;
+				case AST_FORMAT_ALAW:
+				    for (; i < iaxevent->datalen; i++) convertedbuf[i] = alaw_to_linear((iaxevent->data)[i]);
+				    audiodata = convertedbuf;
+				    break;
+				case AST_FORMAT_ULAW:
+				    for (; i < iaxevent->datalen; i++) convertedbuf[i] = ulaw_to_linear((iaxevent->data)[i]);
+				    audiodata = convertedbuf;
+				    break;
+				default:
+				    printlog(LOG_ERROR, "Unknown codec!\n");
+				    break;
+			    }
+			    memcpy(iaxbuf, audiodata, units >= VOIP_PACKET_SIZE ? VOIP_PACKET_SIZE*sizeof(int16_t) : units*sizeof(int16_t));
+			    if (replay) read(iaxaudiofd, audiodata, units*sizeof(int16_t));
+			    if (modemstate == MODEM_CONNECTED && t31_rx(&t31_state, audiodata, units)) {
 				printlog(LOG_ERROR, "Error sending %d units of IAX audio to DSP.\n", units);
 			    }
-			    if (record) write(iaxaudiofd, (uint8_t *) iaxevent->data, iaxevent->datalen);
-			} else {
-			    int16_t convertedbuf[iaxevent->datalen];
-			    int i = 0;
-			    if (codec == AST_FORMAT_ULAW)
-				for (; i < iaxevent->datalen; i++) convertedbuf[i] = ulaw_to_linear((iaxevent->data)[i]);
-			    else
-				for (; i < iaxevent->datalen; i++) convertedbuf[i] = alaw_to_linear((iaxevent->data)[i]);
-			    if (replay) read(iaxaudiofd, (uint8_t *) convertedbuf, iaxevent->datalen);
-			    if (modemstate == MODEM_CONNECTED && t31_rx(&t31_state, convertedbuf, iaxevent->datalen)) {
-				printlog(LOG_ERROR, "Error sending %d units of IAX audio to DSP.\n", iaxevent->datalen);
-			    }
-			    if (record) write(iaxaudiofd, (uint8_t *) convertedbuf, iaxevent->datalen*sizeof(int16_t));
+			    if (record) write(iaxaudiofd, audiodata, units*sizeof(int16_t));
 			}
 			break;
 		    case IAX_EVENT_TRANSFER:
@@ -1092,6 +1191,16 @@ iaxmodem(const char *config, int nolog)
 			}
 			phonestate = PHONE_FREED;
 			break;
+		    case IAX_EVENT_DTMF:
+			/* only report DTMF if the modem is in command-mode */
+			if (t31_state.at_state.at_rx_mode == AT_MODE_OFFHOOK_COMMAND ||
+			    t31_state.at_state.at_rx_mode == AT_MODE_ONHOOK_COMMAND) {
+			    char dtmf[16];
+			    snprintf(dtmf, 15, "\r\nDTMF=%d\r\n", iaxevent->subclass - 48);
+			    at_tx_handler(&t31_state.at_state, NULL, (u_char*) dtmf, strlen(dtmf));
+			}
+			printlog(LOG_INFO, "Received DTMF '%d'\n", iaxevent->subclass - 48);
+			break;
 		    default:
 			printlog(LOG_ERROR, "Don't know what to do with IAX event %d.\n", iaxevent->etype);
 			break;
@@ -1122,8 +1231,7 @@ add_modem(int pid, char *config)
 
     nm = malloc(sizeof(struct modem));
 
-    for (m = modems; (m != NULL) && (m->next != NULL); m = m->next)
-      ;
+    for (m = modems; (m != NULL) && (m->next != NULL); m = m->next);
 
     nm->pid = pid;
     nm->config = config;
@@ -1137,6 +1245,30 @@ add_modem(int pid, char *config)
     numchild++;
 }
 
+void
+restart_modem(pid_t pid)
+{
+    struct modem *m;
+
+    for (m = modems; m != NULL; m = m->next) {
+	if (m->pid == pid) {
+	    pid_t newpid = fork();
+	    if (newpid == 0) {
+		/* child */
+		iaxmodem(m->config, 0);
+		_exit(255);		/* shouldn't ever get here */
+	    } else if (newpid > 0) {
+		/* parent */
+		m->pid = newpid;
+	    } else {
+		/* failed */
+		printlog(LOG_ERROR, "Error: fork failed: %s\n", strerror(errno));
+	    }
+	    break;
+	}
+    }
+}
+
 void
 remove_modem(int pid)
 {
@@ -1154,6 +1286,7 @@ remove_modem(int pid)
 
 	  free(m->config);
 	  free(m);
+	  numchild--;
 
 	  break;
 	}
@@ -1178,20 +1311,26 @@ remove_all_modems()
 void
 wait_for_modems()
 {
-    int pid;
+    pid_t pid;
+    int status;
 
-    while (numchild > 0)
-      {
+    while (numchild > 0) {
 	/* Wait for any child to exit */
-	pid = waitpid(-1, NULL, 0);
+	pid = waitpid(-1, &status, 0);
 
 	if (pid > 0) {
-	  remove_modem(pid);
-	  numchild--;
+	    status >>= 8;
+	    if (status == SIGHUP) {
+		printlog(LOG_ERROR, "iaxmodem process %d ended, status %d, attempting to restart\n", pid, status);
+		restart_modem(pid);
+	    } else {
+		printlog(LOG_ERROR, "iaxmodem process %d ended, status %d\n", pid, status);
+		remove_modem(pid);
+	    }
 	} else {
-	  break;
+	    break;
 	}
-      }
+    }
 }
 
 void
@@ -1205,7 +1344,6 @@ terminate_modems()
       }
 }
 
-
 int
 spawn_modems(void)
 {
