@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: complex_dds.c,v 1.13 2006/03/27 13:35:13 steveu Exp $
+ * $Id: complex_dds.c,v 1.17 2006/10/24 13:45:25 steveu Exp $
  */
 
 /*! \file */
@@ -2143,12 +2142,12 @@ float dds_modf(uint32_t *phase_acc, int32_t phase_rate, float scale, int32_t pha
 
 /* This is a simple direct digital synthesis (DDS) function to generate complex
    sine waves. */
-complex_t dds_complexf(uint32_t *phase_acc, int32_t phase_rate)
+complexf_t dds_complexf(uint32_t *phase_acc, int32_t phase_rate)
 {
-    complex_t amp;
+    complexf_t amp;
 
-    amp = complex_set(sine_table[(*phase_acc + (1 << 30)) >> (32 - SLENK)],
-                      sine_table[*phase_acc >> (32 - SLENK)]);
+    amp = complex_setf(sine_table[(*phase_acc + (1 << 30)) >> (32 - SLENK)],
+                       sine_table[*phase_acc >> (32 - SLENK)]);
     *phase_acc += phase_rate;
     return amp;
 }
@@ -2156,12 +2155,12 @@ complex_t dds_complexf(uint32_t *phase_acc, int32_t phase_rate)
 
 /* This is a simple direct digital synthesis (DDS) function to generate complex
    sine waves. */
-complex_t dds_complex_modf(uint32_t *phase_acc, int32_t phase_rate, float scale, int32_t phase)
+complexf_t dds_complex_modf(uint32_t *phase_acc, int32_t phase_rate, float scale, int32_t phase)
 {
-    complex_t amp;
+    complexf_t amp;
 
-    amp = complex_set(sine_table[(*phase_acc + phase + (1 << 30)) >> (32 - SLENK)]*scale,
-                      sine_table[(*phase_acc + phase) >> (32 - SLENK)]*scale);
+    amp = complex_setf(sine_table[(*phase_acc + phase + (1 << 30)) >> (32 - SLENK)]*scale,
+                       sine_table[(*phase_acc + phase) >> (32 - SLENK)]*scale);
     *phase_acc += phase_rate;
     return amp;
 }
