@@ -15,9 +15,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,7 +27,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: echo.c,v 1.15 2006/05/08 13:22:56 steveu Exp $
+ * $Id: echo.c,v 1.19 2006/10/24 13:45:25 steveu Exp $
  */
 
 /*! \file */
@@ -92,7 +91,6 @@
 #include "spandsp/logging.h"
 #include "spandsp/bit_operations.h"
 #include "spandsp/echo.h"
-#include "spandsp/mmx.h"
 
 #if !defined(NULL)
 #define NULL (void *) 0
@@ -155,7 +153,7 @@ static int narrowband_detect(echo_can_state_t *ec)
     return score;
 }
 
-static inline void lms_adapt(echo_can_state_t *ec, int factor)
+static __inline__ void lms_adapt(echo_can_state_t *ec, int factor)
 {
     int i;
 
