@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: fsk.c,v 1.23 2006/04/24 14:48:56 steveu Exp $
+ * $Id: fsk.c,v 1.26 2006/10/24 16:38:32 steveu Exp $
  */
 
 /*! \file */
@@ -41,7 +40,6 @@
 
 #include "spandsp/telephony.h"
 #include "spandsp/complex.h"
-#include "spandsp/vector.h"
 #include "spandsp/dds.h"
 #include "spandsp/power_meter.h"
 #include "spandsp/async.h"
@@ -154,8 +152,7 @@ int fsk_tx(fsk_tx_state_t *s, int16_t *amp, int len)
         if ((s->baud_frac += s->baud_inc) >= 0x10000)
         {
             s->baud_frac -= 0x10000;
-            bit = s->get_bit(s->user_data);
-            if (bit == 2)
+            if ((bit = s->get_bit(s->user_data)) == PUTBIT_END_OF_DATA)
             {
                 s->shutdown = TRUE;
                 break;
@@ -203,6 +200,7 @@ void fsk_rx_set_put_bit(fsk_rx_state_t *s, put_bit_func_t put_bit, void *user_da
 fsk_rx_state_t *fsk_rx_init(fsk_rx_state_t *s,
                             fsk_spec_t *spec,
                             int sync_mode,
+                            int32_t min_power,
                             put_bit_func_t put_bit,
                             void *user_data)
 {
@@ -211,7 +209,10 @@ fsk_rx_state_t *fsk_rx_init(fsk_rx_state_t *s,
     memset(s, 0, sizeof(*s));
     s->baud_rate = spec->baud_rate;
     s->sync_mode = sync_mode;
-    s->min_power = power_meter_level_dbm0((float) spec->min_level);
+    if (min_power)
+        s->min_power = min_power;
+    else
+        s->min_power = power_meter_level_dbm0((float) spec->min_level);
     s->put_bit = put_bit;
     s->user_data = user_data;
 
