@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v29rx.h,v 1.34 2006/05/02 14:04:44 steveu Exp $
+ * $Id: v29rx.h,v 1.41 2006/10/24 13:45:28 steveu Exp $
  */
 
 /*! \file */
@@ -65,29 +64,26 @@ coefficients, the properly tuned symbol tracker can select data samples at 4800
 samples/second from points within 1.125 degrees of the centre and mid-points of
 each symbol. The output of the filter is multiplied by a complex carrier, generated
 by a DDS. The result is a baseband signal, requiring no further filtering, apart from
-an adaptive equalizer. The baseband signal is fed to a T/2 adaptive equalizer. The Gardner
-algorithm is used to track the symbols, and tune the sampling. Initially this algorithm is very
-lightly damped, to ensure the symbol alignment pulls in quickly. Because the sampling
-rate will not be precisely the same as the transmitter's (the spec. says the symbol
-timing should be within 0.01%), the receiver constantly evaluates and corrects
-this sampling throughout its operation. During the symbol timing maintainence phase,
-the algorithm uses heavier damping to stabilise the sampling points, This heavy damping
-achieves several things - the Gardner algorithm is statistically based when applied to QAM,
-so the statistics must be smoothed; a number of samples must be fed to the equalizer buffer
-before the equalizer output actually responds to a step change in the sampling; we need
-to prevent rapid fluctuations in the sampling position, due to the optimal position being
-close to a boundary.
+an adaptive equalizer. The baseband signal is fed to a T/2 adaptive equalizer.
+A band edge component maximisation algorithm is used to tune the sampling, so the samples
+fed to the equalizer are close to the mid point and edges of each symbol. Initially
+the algorithm is very lightly damped, to ensure the symbol alignment pulls in
+quickly. Because the sampling rate will not be precisely the same as the
+transmitter's (the spec. says the symbol timing should be within 0.01%), the
+receiver constantly evaluates and corrects this sampling throughout its
+operation. During the symbol timing maintainence phase, the algorithm uses
+a heavier damping.
 
 The carrier is specified as 1700Hz +-1Hz at the transmitter, and 1700 +-7Hz at
 the receiver. The receive carrier would only be this inaccurate if the link
 includes FDM sections. These are being phased out, but the design must still
 allow for the worst case. Using an initial 1700Hz signal for demodulation gives
 a worst case rotation rate for the constellation of about one degree per symbol.
-Once the Gardner algorithm has been given time to lock to the symbol timing of
-the initial alternating pattern, the phase of the demodulated signal is recorded
-on two successive symbols - once for each of the constellation positions. The
-receiver then tracks the symbol alternations, until a large phase jump occurs.
-This signifies the start of the next phase of the training sequence. At this
+Once the symbol timing synchronisation algorithm has been given time to lock to
+the symbol timing of the initial alternating pattern, the phase of the demodulated
+signal is recorded on two successive symbols - once for each of the constellation
+positions. The receiver then tracks the symbol alternations, until a large phase jump
+occurs. This signifies the start of the next phase of the training sequence. At this
 point the total phase shift between the original recorded symbol phase, and the
 symbol phase just before the phase jump occurred is used to provide a coarse
 estimation of the rotation rate of the constellation, and it current absolute
@@ -130,9 +126,9 @@ therefore, only tests that bits starting at bit 24 are really ones.
 #define V29_EQUALIZER_POST_LEN  15  /* this much after the real event */
 #define V29_EQUALIZER_MASK      63  /* one less than a power of 2 >= (2*V29_EQUALIZER_LEN + 1) */
 
-#define V29RX_FILTER_STEPS      27
+#define V29_RX_FILTER_STEPS     27
 
-typedef void (qam_report_handler_t)(void *user_data, const complex_t *constel, const complex_t *target, int symbol);
+typedef void (qam_report_handler_t)(void *user_data, const complexf_t *constel, const complexf_t *target, int symbol);
 
 /*!
     V.29 modem receive side descriptor. This defines the working state for a
@@ -154,7 +150,7 @@ typedef struct
     void *qam_user_data;
 
     /*! \brief The route raised cosine (RRC) pulse shaping filter buffer. */
-    float rrc_filter[2*V29RX_FILTER_STEPS];
+    float rrc_filter[2*V29_RX_FILTER_STEPS];
     /*! \brief Current offset into the RRC pulse shaping filter buffer. */
     int rrc_filter_step;
 
@@ -190,23 +186,24 @@ typedef struct
 
     float eq_delta;
     /*! \brief The adaptive equalizer coefficients */
-    complex_t eq_coeff[V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN];
-    complex_t eq_coeff_save[V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN];
-    complex_t eq_buf[V29_EQUALIZER_MASK + 1];
+    complexf_t eq_coeff[V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN];
+    complexf_t eq_coeff_save[V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN];
+    complexf_t eq_buf[V29_EQUALIZER_MASK + 1];
     /*! \brief Current offset into equalizer buffer. */
     int eq_step;
     int eq_put_step;
     int eq_skip;
 
-    /*! \brief Integration variable for damping the Gardner algorithm tests. */
-    int gardner_integrate;
-    /*! \brief Current step size of Gardner algorithm integration. */
-    int gardner_step;
-    /*! \brief The total gardner timing correction, since the carrier came up.
+    /*! \brief The current half of the baud. */
+    int baud_half;
+    /*! \brief Band edge symbol sync. filter state. */
+    float symbol_sync_low[2];
+    float symbol_sync_high[2];
+    float symbol_sync_dc_filter[2];
+    float baud_phase;
+    /*! \brief The total symbol timing correction since the carrier came up.
                This is only for performance analysis purposes. */
-    int gardner_total_correction;
-    /*! \brief The current fractional phase of the baud timing. */
-    int baud_phase;
+    int total_baud_timing_correction;
 
     /*! \brief Starting phase angles for the coarse carrier aquisition step. */
     int32_t start_angles[2];
@@ -256,14 +253,14 @@ void v29_rx_set_put_bit(v29_rx_state_t *s, put_bit_func_t put_bit, void *user_da
     \param amp The audio sample buffer.
     \param len The number of samples in the buffer.
     \return The number of samples unprocessed. */
-int v29_rx(v29_rx_state_t *s, const int16_t *amp, int len);
+int v29_rx(v29_rx_state_t *s, const int16_t amp[], int len);
 
 /*! Get a snapshot of the current equalizer coefficients.
     \brief Get a snapshot of the current equalizer coefficients.
     \param s The modem context.
     \param coeffs The vector of complex coefficients.
     \return The number of coefficients in the vector. */
-int v29_rx_equalizer_state(v29_rx_state_t *s, complex_t **coeffs);
+int v29_rx_equalizer_state(v29_rx_state_t *s, complexf_t **coeffs);
 
 /*! Get the current received carrier frequency.
     \param s The modem context.
