@@ -11,9 +11,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t4.c,v 1.50 2006/05/08 13:22:56 steveu Exp $
+ * $Id: t4.c,v 1.63 2006/10/24 13:21:59 steveu Exp $
  */
 
 /*
@@ -80,6 +79,7 @@
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
 #include "spandsp/bit_operations.h"
+#include "spandsp/async.h"
 #include "spandsp/t4.h"
 
 /* Finite state machine state codes */
@@ -138,7 +138,7 @@ typedef struct
 */
 
 /*
- * ITU T.4 1D Huffman run_length codes and
+ * ITU T.4 1D Huffman run length codes and
  * related definitions.  Given the small sizes
  * of these tables it does not seem
  * worthwhile to make code & length 8 bits.
@@ -152,8 +152,6 @@ typedef struct
 
 #define isAligned(p,t)  ((((intptr_t)(p)) & (sizeof(t) - 1)) == 0)
 
-#define        EOL        0x001        /* EOL code value - 0000 0000 0000 1 */
-
 /* Status values returned instead of a run length */
 #define        T4CODE_EOL        -1        /* NB: ACT_EOL - ACT_WRUNT */
 #define        T4CODE_INVALID    -2        /* NB: ACT_INVALID - ACT_WRUNT */
@@ -401,8 +399,7 @@ static __inline__ int flush_bits_to_image_buffer(t4_state_t *s)
     s->bit = 8;
     if (s->image_size >= s->image_buffer_size)
     {
-        t = realloc(s->image_buffer, s->image_buffer_size + 10000);
-        if (t == NULL)
+        if ((t = realloc(s->image_buffer, s->image_buffer_size + 10000)) == NULL)
             return -1;
         s->image_buffer_size += 10000;
         s->image_buffer = t;
@@ -443,7 +440,7 @@ static __inline__ void put_eol(t4_state_t *s)
         put_run(s, 0);
     if (s->row_len != s->image_width)
     {
-        STATE_TRACE("%d Bad row - %d %d\n", s->rows, s->row_len, s->row_is_2d);
+        STATE_TRACE("%d Bad row - %d %d\n", s->row, s->row_len, s->row_is_2d);
         /* Clean up the bad runs */
         while (s->a0 > s->image_width  &&  s->pa > s->cur_runs)
             s->a0 -= *--s->pa;
@@ -463,20 +460,22 @@ static __inline__ void put_eol(t4_state_t *s)
             s->run_length = 0;
             put_run(s, 0);
         }
-        /* Copy the previous row over this one */
-        if (s->row_starts_at + s->bytes_per_row >= s->image_buffer_size)
+        if (s->row_starts_at != s->last_row_starts_at)
         {
-            t = realloc(s->image_buffer, s->image_buffer_size + 10000);
-            if (t == NULL)
+            /* Copy the previous row over this one */
+            if (s->row_starts_at + s->bytes_per_row >= s->image_buffer_size)
             {
-                /* TODO: take some action to report the allocation failure */
-                return;
+                if ((t = realloc(s->image_buffer, s->image_buffer_size + 10000)) == NULL)
+                {
+                    /* TODO: take some action to report the allocation failure */
+                    return;
+                }
+                s->image_buffer_size += 10000;
+                s->image_buffer = t;
             }
-            s->image_buffer_size += 10000;
-            s->image_buffer = t;
+            memcpy(s->image_buffer + s->row_starts_at, s->image_buffer + s->last_row_starts_at, s->bytes_per_row);
+            s->image_size = s->row_starts_at + s->bytes_per_row;
         }
-        memcpy(s->image_buffer + s->row_starts_at, s->image_buffer + s->last_row_starts_at, s->bytes_per_row);
-        s->image_size = s->row_starts_at + s->bytes_per_row;
         s->bad_rows++;
         s->curr_bad_row_run++;
     }
@@ -488,27 +487,31 @@ static __inline__ void put_eol(t4_state_t *s)
                 s->longest_bad_row_run = s->curr_bad_row_run;
             s->curr_bad_row_run = 0;
         }
-        STATE_TRACE("%d Good row - %d %d\n", s->rows, s->row_len, s->row_is_2d);
+        STATE_TRACE("%d Good row - %d %d\n", s->row, s->row_len, s->row_is_2d);
     }
     
 #if 0
     /* Dump the runs of black and white for analysis */
-    span_log(&s->logging, SPAN_LOG_DEBUG_2, "Ref ");
-    total = 0;
-    for (x = s->ref_runs;  x < s->pb;  x++)
     {
-        total += *x;
-        span_log(&s->logging, SPAN_LOG_DEBUG_2, "%d ", *x);
+        int total;
+
+        span_log(&s->logging, SPAN_LOG_DEBUG_2, "Ref ");
+        total = 0;
+        for (x = s->ref_runs;  x < s->pb;  x++)
+        {
+            total += *x;
+            span_log(&s->logging, SPAN_LOG_DEBUG_2, "%d ", *x);
+        }
+        span_log(&s->logging, SPAN_LOG_DEBUG_2, " total = %d\n", total);
+        span_log(&s->logging, SPAN_LOG_DEBUG_2, "Cur ");
+        total = 0;
+        for (x = s->cur_runs;  x < s->pa;  x++)
+        {
+            total += *x;
+            span_log(&s->logging, SPAN_LOG_DEBUG_2, "%d ", *x);
+        }
+        span_log(&s->logging, SPAN_LOG_DEBUG_2, "total = %d\n", total);
     }
-    span_log(&s->logging, SPAN_LOG_DEBUG_2, " total = %d\n", total);
-    span_log(&s->logging, SPAN_LOG_DEBUG_2, "Cur ");
-    total = 0;
-    for (x = s->cur_runs;  x < s->pa;  x++)
-    {
-        total += *x;
-        span_log(&s->logging, SPAN_LOG_DEBUG_2, "%d ", *x);
-    }
-    span_log(&s->logging, SPAN_LOG_DEBUG_2, "total = %d\n", total);
 #endif
 
     /* Prepare the buffers for the next row. */
@@ -530,6 +533,7 @@ static __inline__ void put_eol(t4_state_t *s)
 int t4_rx_end_page(t4_state_t *s)
 {
     int row;
+    int i;
     time_t now;
     struct tm *tm;
     char buf[256 + 1];
@@ -537,6 +541,12 @@ int t4_rx_end_page(t4_state_t *s)
     float x_resolution;
     float y_resolution;
 
+    if (s->line_encoding == T4_COMPRESSION_ITU_T6)
+    {
+        /* Push enough zeros through the decoder to flush out any remaining codes */
+        for (i = 0;  i < 13;  i++)
+            t4_rx_put_bit(s, 0);
+    }
     if (s->curr_bad_row_run)
     {
         if (s->curr_bad_row_run > s->longest_bad_row_run)
@@ -584,8 +594,8 @@ int t4_rx_end_page(t4_state_t *s)
     resunit = RESUNIT_CENTIMETER;
     TIFFSetField(s->tiff_file, TIFFTAG_RESOLUTIONUNIT, resunit);
 #else
-    TIFFSetField(s->tiff_file, TIFFTAG_XRESOLUTION, floorf(x_resolution*2.54 + 0.5));
-    TIFFSetField(s->tiff_file, TIFFTAG_YRESOLUTION, floorf(y_resolution*2.54 + 0.5));
+    TIFFSetField(s->tiff_file, TIFFTAG_XRESOLUTION, floorf(x_resolution*2.54f + 0.5f));
+    TIFFSetField(s->tiff_file, TIFFTAG_YRESOLUTION, floorf(y_resolution*2.54f + 0.5f));
     resunit = RESUNIT_INCH;
     TIFFSetField(s->tiff_file, TIFFTAG_RESOLUTIONUNIT, resunit);
 #endif
@@ -648,7 +658,7 @@ int t4_rx_end_page(t4_state_t *s)
             break;
         }
     }
-    /* ....the the directory entry, and libtiff is happy. */
+    /* ....then the directory entry, and libtiff is happy. */
     TIFFWriteDirectory(s->tiff_file);
 
     s->bits = 0;
@@ -660,7 +670,7 @@ int t4_rx_end_page(t4_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-int t4_rx_putbit(t4_state_t *s, int bit)
+int t4_rx_put_bit(t4_state_t *s, int bit)
 {
     int bits;
 
@@ -678,6 +688,9 @@ int t4_rx_putbit(t4_state_t *s, int bit)
         s->first_eol_seen = TRUE;
         return FALSE;
     }
+    /* Check if the image has already terminated */
+    if (s->consecutive_eols >= 5)
+        return TRUE;
     if (s->row_is_2d  &&  s->black_white == 0)
     {
         switch (T4_black_table[s->bits_to_date & 0x1FFF].state)
@@ -686,8 +699,7 @@ int t4_rx_putbit(t4_state_t *s, int bit)
             STATE_TRACE("EOL\n");
             put_eol(s);
             s->row_is_2d = !(s->bits_to_date & 0x1000);
-            s->bits -= T4_black_table[s->bits_to_date & 0x1FFF].width;
-            s->bits--;
+            s->bits -= (T4_black_table[s->bits_to_date & 0x1FFF].width + 1);
             s->its_black = FALSE;
             s->row_len = 0;
             break;
@@ -794,10 +806,7 @@ int t4_rx_putbit(t4_state_t *s, int bit)
                             ((s->bits_to_date >> T4_common_table[bits].width) & 0x7),
                             s->bits_to_date);
                 if (s->row_len < s->image_width)
-                {
                     *s->pa++ = s->image_width - s->a0;
-                    //extension(s->a0);
-                }
                 break;
             case S_Null:
                 break;
@@ -854,10 +863,7 @@ int t4_rx_putbit(t4_state_t *s, int bit)
                 if (s->row_len == 0)
                 {
                     if (++s->consecutive_eols >= 5)
-                    {
-                        t4_rx_end_page(s);
                         return TRUE;
-                    }
                 }
                 else
                 {
@@ -922,10 +928,7 @@ int t4_rx_putbit(t4_state_t *s, int bit)
                 if (s->row_len == 0)
                 {
                     if (++s->consecutive_eols >= 5)
-                    {
-                        t4_rx_end_page(s);
                         return TRUE;
-                    }
                 }
                 else
                 {
@@ -948,6 +951,15 @@ int t4_rx_putbit(t4_state_t *s, int bit)
             s->bits -= T4_white_table[bits].width;
         }
     }
+
+    if (s->line_encoding == T4_COMPRESSION_ITU_T6  &&  s->row_len >= s->image_width)
+    {
+        /* T.6 has no EOL markers. We sense the end of a line by its length alone. */
+        STATE_TRACE("EOL T.6\n");
+        put_eol(s);
+        s->its_black = FALSE;
+        s->row_len = 0;
+    }
     return FALSE;
 }
 /*- End of function --------------------------------------------------------*/
@@ -956,18 +968,25 @@ t4_state_t *t4_rx_create(const char *file, int output_encoding)
 {
     t4_state_t *s;
     
-    s = (t4_state_t *) malloc(sizeof(t4_state_t *));
-    if (!s  ||  t4_rx_init(s, file, output_encoding))
-        return NULL;
+    if ((s = (t4_state_t *) malloc(sizeof(t4_state_t *))))
+    {
+        if (t4_rx_init(s, file, output_encoding))
+        {
+            free(s);
+            return NULL;
+        }
+    }
     return s;
 }
 /*- End of function --------------------------------------------------------*/
 
 int t4_rx_init(t4_state_t *s, const char *file, int output_encoding)
 {
-    span_log(&s->logging, SPAN_LOG_FLOW, "Start rx document\n");
-
     memset(s, 0, sizeof(*s));
+    span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
+    span_log_set_protocol(&s->logging, "T.4");
+
+    span_log(&s->logging, SPAN_LOG_FLOW, "Start rx document\n");
 
     if ((s->tiff_file = TIFFOpen(file, "w")) == NULL)
         return -1;
@@ -1028,12 +1047,10 @@ int t4_rx_start_page(t4_state_t *s)
     {
         /* Allocate the space required for decoding the new row length. */
         s->bytes_per_row = bytes_per_row;
-        bufptr = (uint32_t *) realloc(s->cur_runs, run_space);
-        if (bufptr == NULL)
+        if ((bufptr = (uint32_t *) realloc(s->cur_runs, run_space)) == NULL)
             return -1;
         s->cur_runs = bufptr;
-        bufptr = (uint32_t *) realloc(s->ref_runs, run_space);
-        if (bufptr == NULL)
+        if ((bufptr = (uint32_t *) realloc(s->ref_runs, run_space)) == NULL)
             return -1;
         s->ref_runs = bufptr;
     }
@@ -1044,19 +1061,23 @@ int t4_rx_start_page(t4_state_t *s)
     s->bits_to_date = 0;
 
     s->row_is_2d = (s->line_encoding == T4_COMPRESSION_ITU_T6);
-    s->its_black = FALSE;
+    s->first_eol_seen = (s->line_encoding == T4_COMPRESSION_ITU_T6);
+
     s->bad_rows = 0;
     s->longest_bad_row_run = 0;
     s->curr_bad_row_run = 0;
     s->image_length = 0;
-    s->first_eol_seen = FALSE;
     s->consecutive_eols = 0;
     s->data = 0;
     s->bit = 8;
     s->image_size = 0;
     s->row_starts_at = 0;
     s->last_row_starts_at = 0;
-    
+
+    s->row_len = 0;
+    s->its_black = FALSE;
+    s->black_white = 0;
+
     s->pa = s->cur_runs;
     s->pb = s->ref_runs;
 
@@ -1472,7 +1493,7 @@ static void t4_encode_eol(t4_state_t *s)
 /*
  * 2D-encode a row of pixels.  Consult ITU specification T.4 for the algorithm.
  */
-static void t4_encode2Drow(t4_state_t *s, uint8_t *bp)
+static void t4_encode_2d_row(t4_state_t *s, uint8_t *bp)
 {
     int a0;
     int a1;
@@ -1546,7 +1567,7 @@ static void t4_encode2Drow(t4_state_t *s, uint8_t *bp)
  * a sequence of all-white or all-black spans
  * of pixels encoded with Huffman codes.
  */
-static void t4_encode1Drow(t4_state_t *s, uint8_t *bp)
+static void t4_encode_1d_row(t4_state_t *s, uint8_t *bp)
 {
     int span;
     int bs;
@@ -1570,41 +1591,44 @@ static void t4_encode1Drow(t4_state_t *s, uint8_t *bp)
 
 static int t4_encode_row(t4_state_t *s, uint8_t *bp)
 {
-    t4_encode_eol(s);
-    if (s->line_encoding == T4_COMPRESSION_ITU_T6)
+    switch (s->line_encoding)
     {
+    case T4_COMPRESSION_ITU_T6:
         /* T.6 compression is a trivial step up from T.4 2D, so we just
            throw it in here. T.6 is only used with error correction,
            so it does not need independantly compressed (i.e. 1D) lines
-           to recover from data errors. */
-        t4_encode2Drow(s, bp);
+           to recover from data errors. It doesn't need EOLs, either. */
+        t4_encode_2d_row(s, bp);
         memcpy(s->ref_row_buf, bp, s->bytes_per_row);
-    }
-    else if (s->line_encoding == T4_COMPRESSION_ITU_T4_1D)
-    {
-        t4_encode1Drow(s, bp);
-    }
-    else
-    {
+        break;
+    case T4_COMPRESSION_ITU_T4_2D:
+        t4_encode_eol(s);
         if (s->row_is_2d)
         {
-            t4_encode2Drow(s, bp);
-            s->k--;
+            t4_encode_2d_row(s, bp);
+            s->rows_to_next_1d_row--;
         }
         else
         {
-            t4_encode1Drow(s, bp);
+            t4_encode_1d_row(s, bp);
             s->row_is_2d = TRUE;
         }
-        if (s->k == 0)
+        if (s->rows_to_next_1d_row <= 0)
         {
+            /* Insert a row of 1D encoding */
             s->row_is_2d = FALSE;
-            s->k = s->maxk - 1;
+            s->rows_to_next_1d_row = s->max_rows_to_next_1d_row - 1;
         }
         else
         {
             memcpy(s->ref_row_buf, bp, s->bytes_per_row);
         }
+        break;
+    default:
+    case T4_COMPRESSION_ITU_T4_1D:
+        t4_encode_eol(s);
+        t4_encode_1d_row(s, bp);
+        break;
     }
     bp += s->bytes_per_row;
     s->row++;
@@ -1616,9 +1640,14 @@ t4_state_t *t4_tx_create(const char *file, int start_page, int stop_page)
 {
     t4_state_t *s;
     
-    s = (t4_state_t *) malloc(sizeof(t4_state_t *));
-    if (!s  ||  t4_tx_init(s, file, start_page, stop_page))
-        return NULL;
+    if ((s = (t4_state_t *) malloc(sizeof(t4_state_t *))))
+    {
+        if (t4_tx_init(s, file, start_page, stop_page))
+        {
+            free(s);
+            return NULL;
+        }
+    }
     return s;
 }
 /*- End of function --------------------------------------------------------*/
@@ -1630,9 +1659,13 @@ int t4_tx_init(t4_state_t *s, const char *file, int start_page, int stop_page)
     uint16_t res_unit;
     uint32_t parm;
 
+    memset(s, 0, sizeof(*s));
+    span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
+    span_log_set_protocol(&s->logging, "T.4");
+
     span_log(&s->logging, SPAN_LOG_FLOW, "Start tx document\n");
-    s->tiff_file = TIFFOpen(file, "r");
-    if (s->tiff_file == NULL)
+
+    if ((s->tiff_file = TIFFOpen(file, "r")) == NULL)
         return -1;
 
     s->file = strdup(file);
@@ -1641,7 +1674,7 @@ int t4_tx_init(t4_state_t *s, const char *file, int start_page, int stop_page)
     TIFFGetField(s->tiff_file, TIFFTAG_IMAGEWIDTH, &parm);
     s->image_width = parm;
     s->bytes_per_row = (s->image_width + 7)/8;
-    TIFFGetField(s->tiff_file, TIFFTAG_YRESOLUTION, &x_resolution);
+    TIFFGetField(s->tiff_file, TIFFTAG_XRESOLUTION, &x_resolution);
     TIFFGetField(s->tiff_file, TIFFTAG_YRESOLUTION, &y_resolution);
     TIFFGetField(s->tiff_file, TIFFTAG_RESOLUTIONUNIT, &res_unit);
 
@@ -1670,30 +1703,22 @@ int t4_tx_init(t4_state_t *s, const char *file, int start_page, int stop_page)
         (res_unit == RESUNIT_INCH  &&  fabsf(y_resolution - 392.0f) < 2.0f))
     {
         s->y_resolution = T4_Y_RESOLUTION_SUPERFINE;
-        s->maxk = 8;
+        s->max_rows_to_next_1d_row = 8;
     }
     else if ((res_unit == RESUNIT_CENTIMETER  &&  fabsf(y_resolution - 77.0f) < 2.0f)
              ||
              (res_unit == RESUNIT_INCH  &&  fabsf(y_resolution - 196.0f) < 2.0f))
     {
         s->y_resolution = T4_Y_RESOLUTION_FINE;
-        s->maxk = 4;
+        s->max_rows_to_next_1d_row = 4;
     }
     else
     {
         s->y_resolution = T4_Y_RESOLUTION_STANDARD;
-        s->maxk = 2;
+        s->max_rows_to_next_1d_row = 2;
     }
+    s->rows_to_next_1d_row = s->max_rows_to_next_1d_row - 1;
 
-    if (s->line_encoding != T4_COMPRESSION_ITU_T4_2D)
-    {
-        s->maxk =
-        s->k = 0;
-    }
-    else
-    {
-        s->k = s->maxk - 1;
-    }
     s->pages_transferred = s->start_page;
     if ((s->row_buf = malloc(s->bytes_per_row)) == NULL)
         return -1;
@@ -1704,7 +1729,6 @@ int t4_tx_init(t4_state_t *s, const char *file, int start_page, int stop_page)
         return -1;
     }
     s->image_buffer_size = 0;
-    s->header_info[0] = '\0';
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
@@ -1751,17 +1775,10 @@ int t4_tx_start_page(t4_state_t *s)
     int ok;
     int i;
     int pattern;
-    int data;
-    int bit;
-    int length;
     int row_bufptr;
     uint32_t parm;
     char *t;
     char header[132 + 1];
-    static const int msbmask[9] =
-    {
-        0x00, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f, 0xff
-    };
 
     span_log(&s->logging, SPAN_LOG_FLOW, "Start tx page %d\n", s->pages_transferred);
     if (s->pages_transferred > s->stop_page)
@@ -1773,6 +1790,7 @@ int t4_tx_start_page(t4_state_t *s)
     s->image_size = 0;
     s->bit = 8;
     s->row_is_2d = (s->line_encoding == T4_COMPRESSION_ITU_T6);
+    s->rows_to_next_1d_row = s->max_rows_to_next_1d_row - 1;
 
     /* Allow for pages being of different width */
     TIFFGetField(s->tiff_file, TIFFTAG_IMAGEWIDTH, &parm);
@@ -1791,74 +1809,39 @@ int t4_tx_start_page(t4_state_t *s)
     }
     memset(s->ref_row_buf, 0, s->bytes_per_row);
 
-    if (s->header_info[0])
+    if (s->header_info  &&  s->header_info[0])
     {
         /* Modify the resulting image to include a header line, typical of hardware FAX machines */
         make_header(s, header);
         for (row = 0;  row < 16;  row++)
         {
             t = header;
-            i = 0;
-            data = 0;
-            bit = 8;
             row_bufptr = 0;
-            do
+            for (t = header;  *t  &&  row_bufptr <= s->bytes_per_row - 2;  t++)
             {
                 pattern = header_font[(uint8_t) *t][row];
-                length = 16;
-                while (length > bit)
-                {
-                    data |= (pattern >> (length - bit));
-                    length -= bit;
-                    bit = 8;
-                    s->row_buf[row_bufptr++] = (uint8_t) data;
-                    data = 0;
-                }
-                data |= ((pattern & msbmask[length]) << (bit - length));
-                bit -= length;
-                if (bit == 0)
-                {
-                    bit = 8;
-                    s->row_buf[row_bufptr++] = (uint8_t) data;
-                    data = 0;
-                }
+                s->row_buf[row_bufptr++] = (uint8_t) (pattern >> 8);
+                s->row_buf[row_bufptr++] = (uint8_t) (pattern & 0xFF);
             }
-            while (*++t  &&  row_bufptr < s->bytes_per_row - 2);
-            while (row_bufptr < s->bytes_per_row - 2)
-            {
-                pattern = 0;
-                length = 16;
-                while (length > s->bit)
-                {
-                    data |= (pattern >> (length - bit));
-                    length -= bit;
-                    bit = 8;
-                    s->row_buf[row_bufptr++] = (uint8_t) data;
-                    data = 0;
-                }
-                data |= ((pattern & msbmask[length]) << (bit - length));
-                bit -= length;
-                if (bit == 0)
-                {
-                    bit = 8;
-                    s->row_buf[row_bufptr++] = (uint8_t) data;
-                    data = 0;
-                }
-            }
-            if (s->y_resolution == T4_X_RESOLUTION_R16)
+            for (  ;  row_bufptr <= s->bytes_per_row;  )
+                s->row_buf[row_bufptr++] = 0;
+            switch (s->y_resolution)
             {
+            case T4_Y_RESOLUTION_SUPERFINE:
                 if ((ok = t4_encode_row(s, s->row_buf)) <= 0)
                     return -1;
                 if ((ok = t4_encode_row(s, s->row_buf)) <= 0)
                     return -1;
-            }
-            if (s->y_resolution == T4_X_RESOLUTION_R8)
-            {
+                /* Fall through */
+            case T4_Y_RESOLUTION_FINE:
                 if ((ok = t4_encode_row(s, s->row_buf)) <= 0)
                     return -1;
+                /* Fall through */
+            default:
+                if ((ok = t4_encode_row(s, s->row_buf)) <= 0)
+                    return -1;
+                break;
             }
-            if ((ok = t4_encode_row(s, s->row_buf)) <= 0)
-                return -1;
         }
     }
     TIFFGetField(s->tiff_file, TIFFTAG_IMAGELENGTH, &s->image_length);
@@ -1873,13 +1856,16 @@ int t4_tx_start_page(t4_state_t *s)
             return -1;
     }
 
-    /* Now attach a return to control (RTC == 6 x EOLs) to the end of the page */
-    s->row_is_2d = FALSE;
-    for (i = 0;  i < 6;  i++)
+    if (s->line_encoding != T4_COMPRESSION_ITU_T6)
     {
-        t4_encode_eol(s);
-        /* Suppress row padding between these EOLs */
-        s->row_bits = INT_MAX - 1000;
+        /* Attach a return to control (RTC == 6 x EOLs) to the end of the page */
+        s->row_is_2d = FALSE;
+        for (i = 0;  i < 6;  i++)
+        {
+            t4_encode_eol(s);
+            /* Suppress row padding between these EOLs */
+            s->row_bits = INT_MAX - 1000;
+        }
     }
     put_bits(s, 0, 7);
     s->bit_pos = 7;
@@ -1890,6 +1876,19 @@ int t4_tx_start_page(t4_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
+int t4_tx_more_pages(t4_state_t *s)
+{
+    span_log(&s->logging, SPAN_LOG_FLOW, "Checking for the existance of page %d\n", s->pages_transferred + 1);
+    if (s->pages_transferred > s->stop_page)
+        return -1;
+    if (s->tiff_file == NULL)
+        return -1;
+    if (!TIFFSetDirectory(s->tiff_file, (tdir_t) s->pages_transferred + 1))
+        return -1;
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
+
 int t4_tx_restart_page(t4_state_t *s)
 {
     s->bit_pos = 7;
@@ -1906,12 +1905,12 @@ int t4_tx_end_page(t4_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-int t4_tx_getbit(t4_state_t *s)
+int t4_tx_get_bit(t4_state_t *s)
 {
     int bit;
 
     if (s->bit_ptr >= s->image_size)
-        return 2;
+        return PUTBIT_END_OF_DATA;
     bit = (s->image_buffer[s->bit_ptr] >> s->bit_pos) & 1;
     if (--s->bit_pos < 0)
     {
@@ -1922,6 +1921,17 @@ int t4_tx_getbit(t4_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
+int t4_tx_check_bit(t4_state_t *s)
+{
+    int bit;
+
+    if (s->bit_ptr >= s->image_size)
+        return PUTBIT_END_OF_DATA;
+    bit = (s->image_buffer[s->bit_ptr] >> s->bit_pos) & 1;
+    return bit;
+}
+/*- End of function --------------------------------------------------------*/
+
 int t4_tx_delete(t4_state_t *s)
 {
     if (t4_tx_end(s))
@@ -1964,6 +1974,8 @@ int t4_tx_end(t4_state_t *s)
 void t4_tx_set_tx_encoding(t4_state_t *s, int encoding)
 {
     s->line_encoding = encoding;
+    s->rows_to_next_1d_row = s->max_rows_to_next_1d_row - 1;
+    s->row_is_2d = FALSE;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -1979,25 +1991,9 @@ void t4_tx_set_local_ident(t4_state_t *s, const char *ident)
 }
 /*- End of function --------------------------------------------------------*/
 
-int t4_tx_set_header_info(t4_state_t *s, const char *info)
+void t4_tx_set_header_info(t4_state_t *s, const char *info)
 {
-    if (info == NULL)
-    {
-        s->header_info[0] = '\0';
-        return 0;
-    }
-    if (strlen(info) < sizeof(s->header_info))
-    {
-        strcpy(s->header_info, info);
-        return 0;
-    }
-    return -1;
-}
-/*- End of function --------------------------------------------------------*/
-
-const char *t4_tx_get_header_info(t4_state_t *s)
-{
-    return s->header_info;
+    s->header_info = (info  &&  info[0])  ?  info  :  NULL;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -2048,62 +2044,3 @@ const char *t4_encoding_to_str(int encoding)
 }
 /*- End of function --------------------------------------------------------*/
 /*- End of file ------------------------------------------------------------*/
-
-#if 0
-uint32_t encode_faxrecvparams(void)
-{
-    uint32_t v;
-
-    /* Check version */
-    v = ((vertical_resolution & 7) << 0)
-      | ((bit_rate & 15) << 3)
-      | ((image_width & 7) << 9)
-      | ((image_length & 3) << 12)
-      | ((data_format & 3) << 14)
-      | ((ec_protocol & 1) << 16)
-      | ((bft_protocol & 1) << 17)
-      | ((min_scan_time & 7) << 18)
-      | (1 << 21);
-    return v;
-}
-
-void decode_faxrecvparams(uint32_t v)
-{
-    /* The format depends of the version field */
-    if ((v >> 21) == 1)
-    {
-        vertical_resolution = (v >> 0) & 7;
-        bit_rate = (v >> 3) & 15;
-        image_width = (v >> 9) & 7;
-        image_length = (v >> 12) & 3;
-        if (image_length == LN_LET)	// force protocol value
-            image_length = LN_A4;
-        data_format = (v >> 14) & 3;
-        ec_protocol = (v >> 16) & 1;
-        bft_protocol = (v >> 17) & 1;
-        min_scan_time = (v >> 18) & 7;
-    }
-    else
-    {
-        /* original version */
-        vertical_resolution = (v >> 0) & 1;
-        bit_rate = (v >> 1) & 7;
-        image_width = (v >> 4) & 7;
-        image_length = (v >> 7) & 3;
-        if (image_length == LN_LET)	// force protocol value
-            image_length = LN_A4;
-        data_format = (v >> 9) & 3;
-        ec_protocol = (v >> 11) & 1;
-        bft_protocol = (v >> 12) & 1;
-        min_scan_time = (v >> 13) & 7;
-    }
-}
-    u_int vr;		// vertical resolution (VR_*)
-    u_int br;		// bit rate (BR_*)
-    u_int wd;		// page width (WD_*)
-    u_int ln;		// page length (LN_*)
-    u_int df;		// data format (DF_*)
-    u_int ec;		// error correction protocol (EC_*)
-    u_int bf;		// binary file transfer protocol (BF_*)
-    u_int st;		// minimum scanline time (ST_*)
-#endif
