@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v22bis_rx.c,v 1.24 2006/03/10 05:01:36 steveu Exp $
+ * $Id: v22bis_rx.c,v 1.31 2006/10/24 13:45:27 steveu Exp $
  */
 
 /*! \file */
@@ -42,6 +41,7 @@
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
+#include "spandsp/vector_float.h"
 #include "spandsp/async.h"
 #include "spandsp/power_meter.h"
 #include "spandsp/arctan2.h"
@@ -103,7 +103,7 @@ enum
 //#define PULSESHAPER_GAIN        159.37711522f
 #define PULSESHAPER_GAIN        39.98830768f
 #define PULSESHAPER_COEFF_SETS  12
-static const complex_t pulseshaper_1200[PULSESHAPER_COEFF_SETS][V22BIS_RX_FILTER_STEPS] =
+static const complexf_t pulseshaper_1200[PULSESHAPER_COEFF_SETS][V22BIS_RX_FILTER_STEPS] =
 {
     {
         { 0.0079885982f, -0.0245863758f},       /* Filter 0 */
@@ -575,7 +575,7 @@ static const complex_t pulseshaper_1200[PULSESHAPER_COEFF_SETS][V22BIS_RX_FILTER
     },
 };
 
-static const complex_t pulseshaper_2400[PULSESHAPER_COEFF_SETS][V22BIS_RX_FILTER_STEPS] =
+static const complexf_t pulseshaper_2400[PULSESHAPER_COEFF_SETS][V22BIS_RX_FILTER_STEPS] =
 {
     {
         { 0.0209144205f,  0.0151952161f},       /* Filter 0 */
@@ -1065,7 +1065,7 @@ float v22bis_rx_carrier_frequency(v22bis_state_t *s)
 
 float v22bis_rx_symbol_timing_correction(v22bis_state_t *s)
 {
-    return (float) s->gardner_total_correction/((float) PULSESHAPER_COEFF_SETS*40.0f/(3.0f*2.0f));
+    return (float) s->total_baud_timing_correction/((float) PULSESHAPER_COEFF_SETS*40.0f/(3.0f*2.0f));
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -1075,7 +1075,7 @@ float v22bis_rx_signal_power(v22bis_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-int v22bis_rx_equalizer_state(v22bis_state_t *s, complex_t **coeffs)
+int v22bis_rx_equalizer_state(v22bis_state_t *s, complexf_t **coeffs)
 {
     *coeffs = s->eq_coeff;
     return 2*V22BIS_EQUALIZER_LEN + 1;
@@ -1088,10 +1088,10 @@ static void equalizer_reset(v22bis_state_t *s)
 
     /* Start with an equalizer based on everything being perfect */
     for (i = 0;  i < 2*V22BIS_EQUALIZER_LEN + 1;  i++)
-        s->eq_coeff[i] = complex_set(0.0f, 0.0f);
-    s->eq_coeff[V22BIS_EQUALIZER_LEN] = complex_set(3.0f, 0.0f);
+        s->eq_coeff[i] = complex_setf(0.0f, 0.0f);
+    s->eq_coeff[V22BIS_EQUALIZER_LEN] = complex_setf(3.0f, 0.0f);
     for (i = 0;  i <= V22BIS_EQUALIZER_MASK;  i++)
-        s->eq_buf[i] = complex_set(0.0f, 0.0f);
+        s->eq_buf[i] = complex_setf(0.0f, 0.0f);
 
     s->eq_coeff[V22BIS_EQUALIZER_LEN - 6].re = -0.02f;
     s->eq_coeff[V22BIS_EQUALIZER_LEN - 5].re =  0.035f;
@@ -1112,35 +1112,35 @@ static void equalizer_reset(v22bis_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-static complex_t equalizer_get(v22bis_state_t *s)
+static complexf_t equalizer_get(v22bis_state_t *s)
 {
     int i;
     int p;
-    complex_t z;
-    complex_t z1;
+    complexf_t z;
+    complexf_t z1;
 
     /* Get the next equalized value. */
-    z = complex_set(0.0f, 0.0f);
+    z = complex_setf(0.0f, 0.0f);
     p = s->eq_step - 1;
     for (i = 0;  i < 2*V22BIS_EQUALIZER_LEN + 1;  i++)
     {
         p = (p - 1) & V22BIS_EQUALIZER_MASK;
-        z1 = complex_mul(&s->eq_coeff[i], &s->eq_buf[p]);
-        z = complex_add(&z, &z1);
+        z1 = complex_mulf(&s->eq_coeff[i], &s->eq_buf[p]);
+        z = complex_addf(&z, &z1);
     }
     return z;
 }
 /*- End of function --------------------------------------------------------*/
 
-static void tune_equalizer(v22bis_state_t *s, const complex_t *z, const complex_t *target)
+static void tune_equalizer(v22bis_state_t *s, const complexf_t *z, const complexf_t *target)
 {
     int i;
     int p;
-    complex_t ez;
-    complex_t z1;
+    complexf_t ez;
+    complexf_t z1;
 
     /* Find the x and y mismatch from the exact constellation position. */
-    ez = complex_sub(target, z);
+    ez = complex_subf(target, z);
     ez.re *= s->eq_delta;
     ez.im *= s->eq_delta;
 
@@ -1148,9 +1148,9 @@ static void tune_equalizer(v22bis_state_t *s, const complex_t *z, const complex_
     for (i = 0;  i < 2*V22BIS_EQUALIZER_LEN + 1;  i++)
     {
         p = (p - 1) & V22BIS_EQUALIZER_MASK;
-        z1 = complex_conj(&s->eq_buf[p]);
-        z1 = complex_mul(&ez, &z1);
-        s->eq_coeff[i] = complex_add(&s->eq_coeff[i], &z1);
+        z1 = complex_conjf(&s->eq_buf[p]);
+        z1 = complex_mulf(&ez, &z1);
+        s->eq_coeff[i] = complex_addf(&s->eq_coeff[i], &z1);
         /* If we don't leak a little bit we seem to get some wandering adaption */
         s->eq_coeff[i].re *= 0.9999f;
         s->eq_coeff[i].im *= 0.9999f;
@@ -1158,7 +1158,7 @@ static void tune_equalizer(v22bis_state_t *s, const complex_t *z, const complex_
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ void track_carrier(v22bis_state_t *s, const complex_t *z, const complex_t *target)
+static __inline__ void track_carrier(v22bis_state_t *s, const complexf_t *z, const complexf_t *target)
 {
     float error;
 
@@ -1173,7 +1173,7 @@ static __inline__ void track_carrier(v22bis_state_t *s, const complex_t *z, cons
 }
 /*- End of function --------------------------------------------------------*/
 
-static inline void put_bit(v22bis_state_t *s, int bit)
+static __inline__ void put_bit(v22bis_state_t *s, int bit)
 {
     int out_bit;
 
@@ -1218,7 +1218,7 @@ static void decode_baud(v22bis_state_t *s, int nearest)
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ int find_quadrant(const complex_t *z)
+static __inline__ int find_quadrant(const complexf_t *z)
 {
     int b1;
     int b2;
@@ -1243,15 +1243,15 @@ static __inline__ int find_quadrant(const complex_t *z)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void process_half_baud(v22bis_state_t *s, const complex_t *sample)
+static void process_half_baud(v22bis_state_t *s, const complexf_t *sample)
 {
-    complex_t a;
-    complex_t b;
-    complex_t c;
+    complexf_t a;
+    complexf_t b;
+    complexf_t c;
 
-    complex_t z;
-    complex_t zz;
-    const complex_t *target;
+    complexf_t z;
+    complexf_t zz;
+    const complexf_t *target;
     float p;
     float q;
     int re;
@@ -1293,10 +1293,10 @@ static void process_half_baud(v22bis_state_t *s, const complex_t *sample)
     else
     {
         /* Rotate the points to the 45 degree positions, to maximise the effectiveness of the Gardner algorithm */
-        zz = complex_set(cosf(26.57f*3.14159f/180.0f), sinf(26.57f*3.14159f/180.0f));
-        a = complex_mul(&s->eq_buf[(s->eq_step - 3) & V22BIS_EQUALIZER_MASK], &zz);
-        b = complex_mul(&s->eq_buf[(s->eq_step - 2) & V22BIS_EQUALIZER_MASK], &zz);
-        c = complex_mul(&s->eq_buf[(s->eq_step - 1) & V22BIS_EQUALIZER_MASK], &zz);
+        zz = complex_setf(cosf(26.57f*3.14159f/180.0f), sinf(26.57f*3.14159f/180.0f));
+        a = complex_mulf(&s->eq_buf[(s->eq_step - 3) & V22BIS_EQUALIZER_MASK], &zz);
+        b = complex_mulf(&s->eq_buf[(s->eq_step - 2) & V22BIS_EQUALIZER_MASK], &zz);
+        c = complex_mulf(&s->eq_buf[(s->eq_step - 1) & V22BIS_EQUALIZER_MASK], &zz);
         p = (a.re - c.re)*b.re;
         q = (a.im - c.im)*b.im;
     }
@@ -1311,8 +1311,8 @@ static void process_half_baud(v22bis_state_t *s, const complex_t *sample)
            Rapid changes, without hysteresis, are bad. They degrade the equalizer performance
            when the true symbol boundary is close to a sample boundary. */
         s->eq_put_step += (s->gardner_integrate/16);
-        s->gardner_total_correction += (s->gardner_integrate/16);
-span_log(&s->logging, SPAN_LOG_FLOW, "Gardner kick %d [total %d]\n", s->gardner_integrate, s->gardner_total_correction);
+        s->total_baud_timing_correction += (s->gardner_integrate/16);
+span_log(&s->logging, SPAN_LOG_FLOW, "Gardner kick %d [total %d]\n", s->gardner_integrate, s->total_baud_timing_correction);
         if (s->qam_report)
             s->qam_report(s->qam_user_data, NULL, NULL, s->gardner_integrate);
         s->gardner_integrate = 0;
@@ -1338,8 +1338,8 @@ span_log(&s->logging, SPAN_LOG_FLOW, "VVV %p %d\n", s->user_data, s->rx_training
     }
     else
     {
-        zz = complex_set(3.0f/sqrtf(10.0f), -1.0f/sqrtf(10.0f));
-        zz = complex_mul(&z, &zz);
+        zz = complex_setf(3.0f/sqrtf(10.0f), -1.0f/sqrtf(10.0f));
+        zz = complex_mulf(&z, &zz);
         nearest = (find_quadrant(&zz) << 2) | 0x01;
         printf("Trackit %15.5f %15.5f     %15.5f %15.5f   %d\n", z.re, z.im, zz.re, zz.im, nearest);
     }
@@ -1499,15 +1499,15 @@ span_log(&s->logging, SPAN_LOG_FLOW, "S11 0x%02x\n", nearest);
 }
 /*- End of function --------------------------------------------------------*/
 
-int v22bis_rx(v22bis_state_t *s, const int16_t *amp, int len)
+int v22bis_rx(v22bis_state_t *s, const int16_t amp[], int len)
 {
     int i;
     int j;
     int step;
-    complex_t z;
-    complex_t zz;
+    complexf_t z;
+    complexf_t zz;
     int32_t power;
-    complex_t sample;
+    complexf_t sample;
     float ii;
     float qq;
 
@@ -1516,25 +1516,25 @@ int v22bis_rx(v22bis_state_t *s, const int16_t *amp, int len)
         /* Complex bandpass filter the signal, using a pair of FIRs, and RRC coeffs shifted
            to centre at 1200Hz or 2400Hz. The filters support 12 fractional phase shifts, to 
            permit signal extraction very close to the middle of a symbol. */
-        s->rrc_filter[s->rrc_filter_step] =
-        s->rrc_filter[s->rrc_filter_step + V22BIS_RX_FILTER_STEPS] = amp[i];
-        if (++s->rrc_filter_step >= V22BIS_RX_FILTER_STEPS)
-            s->rrc_filter_step = 0;
+        s->rx_rrc_filter[s->rx_rrc_filter_step] =
+        s->rx_rrc_filter[s->rx_rrc_filter_step + V22BIS_RX_FILTER_STEPS] = amp[i];
+        if (++s->rx_rrc_filter_step >= V22BIS_RX_FILTER_STEPS)
+            s->rx_rrc_filter_step = 0;
 
         /* Calculate the I filter, with an arbitrary phase step, just so we can calculate
            the signal power. */
         /* TODO: get rid of this */
         if (s->caller)
         {
-            ii = pulseshaper_2400[6][0].re*s->rrc_filter[s->rrc_filter_step];
+            ii = pulseshaper_2400[6][0].re*s->rx_rrc_filter[s->rx_rrc_filter_step];
             for (j = 1;  j < V22BIS_RX_FILTER_STEPS;  j++)
-                ii += pulseshaper_2400[6][j].re*s->rrc_filter[j + s->rrc_filter_step];
+                ii += pulseshaper_2400[6][j].re*s->rx_rrc_filter[j + s->rx_rrc_filter_step];
         }
         else
         {
-            ii = pulseshaper_1200[6][0].re*s->rrc_filter[s->rrc_filter_step];
+            ii = pulseshaper_1200[6][0].re*s->rx_rrc_filter[s->rx_rrc_filter_step];
             for (j = 1;  j < V22BIS_RX_FILTER_STEPS;  j++)
-                ii += pulseshaper_1200[6][j].re*s->rrc_filter[j + s->rrc_filter_step];
+                ii += pulseshaper_1200[6][j].re*s->rx_rrc_filter[j + s->rx_rrc_filter_step];
         }
         power = power_meter_update(&(s->rx_power), (int16_t) (ii/10.0f));
         if (s->carrier_present)
@@ -1579,22 +1579,22 @@ int v22bis_rx(v22bis_state_t *s, const int16_t *amp, int len)
                 s->eq_put_step += PULSESHAPER_COEFF_SETS*40/(3*2);
                 if (s->caller)
                 {
-                    ii = pulseshaper_2400[step][0].re*s->rrc_filter[s->rrc_filter_step];
-                    qq = pulseshaper_2400[step][0].im*s->rrc_filter[s->rrc_filter_step];
+                    ii = pulseshaper_2400[step][0].re*s->rx_rrc_filter[s->rx_rrc_filter_step];
+                    qq = pulseshaper_2400[step][0].im*s->rx_rrc_filter[s->rx_rrc_filter_step];
                     for (j = 1;  j < V22BIS_RX_FILTER_STEPS;  j++)
                     {
-                        ii += pulseshaper_2400[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
-                        qq += pulseshaper_2400[step][j].im*s->rrc_filter[j + s->rrc_filter_step];
+                        ii += pulseshaper_2400[step][j].re*s->rx_rrc_filter[j + s->rx_rrc_filter_step];
+                        qq += pulseshaper_2400[step][j].im*s->rx_rrc_filter[j + s->rx_rrc_filter_step];
                     }
                 }
                 else
                 {
-                    ii = pulseshaper_1200[step][0].re*s->rrc_filter[s->rrc_filter_step];
-                    qq = pulseshaper_1200[step][0].im*s->rrc_filter[s->rrc_filter_step];
+                    ii = pulseshaper_1200[step][0].re*s->rx_rrc_filter[s->rx_rrc_filter_step];
+                    qq = pulseshaper_1200[step][0].im*s->rx_rrc_filter[s->rx_rrc_filter_step];
                     for (j = 1;  j < V22BIS_RX_FILTER_STEPS;  j++)
                     {
-                        ii += pulseshaper_1200[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
-                        qq += pulseshaper_1200[step][j].im*s->rrc_filter[j + s->rrc_filter_step];
+                        ii += pulseshaper_1200[step][j].re*s->rx_rrc_filter[j + s->rx_rrc_filter_step];
+                        qq += pulseshaper_1200[step][j].im*s->rx_rrc_filter[j + s->rx_rrc_filter_step];
                     }
                 }
                 sample.re = ii*s->agc_scaling;
@@ -1617,7 +1617,8 @@ int v22bis_rx_restart(v22bis_state_t *s, int bit_rate)
     /* If bit_rate is 2400, the real bit rate is negotiated. If bit_rate
        is 1200, the real bit rate is forced to 1200. */
     s->bit_rate = bit_rate;
-
+    vec_zerof(s->rx_rrc_filter, sizeof(s->rx_rrc_filter)/sizeof(s->rx_rrc_filter[0]));
+    s->rx_rrc_filter_step = 0;
     s->rx_scramble_reg = 0;
     s->rx_scrambler_pattern_count = 0;
     s->rx_training = V22BIS_TRAINING_STAGE_SYMBOL_ACQUISITION;
