@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v22bis_tx.c,v 1.25 2006/03/27 13:35:13 steveu Exp $
+ * $Id: v22bis_tx.c,v 1.32 2006/10/24 13:45:27 steveu Exp $
  */
 
 /*! \file */
@@ -42,8 +41,10 @@
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
-#include "spandsp/async.h"
 #include "spandsp/complex.h"
+#include "spandsp/vector_float.h"
+#include "spandsp/complex_vector_float.h"
+#include "spandsp/async.h"
 #include "spandsp/dds.h"
 #include "spandsp/power_meter.h"
 
@@ -240,7 +241,7 @@ static const int phase_steps[4] =
     1, 0, 2, 3
 };
 
-const complex_t v22bis_constellation[16] =
+const complexf_t v22bis_constellation[16] =
 {
     { 1.0f,  1.0f},
     { 3.0f,  1.0f},
@@ -260,6 +261,456 @@ const complex_t v22bis_constellation[16] =
     { 3.0f, -3.0f}
 };
 
+/* Raised root cosine pulse shaping; Beta = 0.75; 4 symbols either
+   side of the centre. */
+/* Created with mkshape -r 0.0125 0.75 361 -l and then split up */
+#define PULSESHAPER_GAIN            (40.000612087f/40.0f)
+#define PULSESHAPER_COEFF_SETS      40
+
+static const float pulseshaper[PULSESHAPER_COEFF_SETS][V22BIS_TX_FILTER_STEPS] =
+{
+    {
+        -0.0047287346,          /* Filter 0 */
+        -0.0083947197,
+        -0.0087380763,
+         0.0088053673,
+         0.5108981827,
+         0.5108981827,
+         0.0088053673,
+        -0.0087380763,
+        -0.0083947197
+    },
+    {
+        -0.0044638629,          /* Filter 1 */
+        -0.0089241700,
+        -0.0111288952,
+         0.0023412184,
+         0.5623914901,
+         0.4599551720,
+         0.0144817755,
+        -0.0063186648,
+        -0.0077293609
+    },
+    {
+        -0.0041048584,          /* Filter 2 */
+        -0.0093040596,
+        -0.0134459768,
+        -0.0048558766,
+         0.6141017035,
+         0.4098822897,
+         0.0193317049,
+        -0.0039145680,
+        -0.0069438567
+    },
+    {
+        -0.0036565006,          /* Filter 3 */
+        -0.0095231635,
+        -0.0156437084,
+        -0.0127148737,
+         0.6656848457,
+         0.3609830295,
+         0.0233320755,
+        -0.0015677363,
+        -0.0060557371
+    },
+    {
+        -0.0031253709,          /* Filter 4 */
+        -0.0095729633,
+        -0.0176768181,
+        -0.0211485021,
+         0.7167894869,
+         0.3135419896,
+         0.0264748749,
+         0.0006824956,
+        -0.0050839319
+    },
+    {
+        -0.0025197700,          /* Filter 5 */
+        -0.0094478866,
+        -0.0195012095,
+        -0.0300535107,
+         0.7670600056,
+         0.2678225635,
+         0.0287663895,
+         0.0027999985,
+        -0.0040483891
+    },
+    {
+        -0.0018496023,          /* Filter 6 */
+        -0.0091454978,
+        -0.0210748106,
+        -0.0393111426,
+         0.8161399423,
+         0.2240649005,
+         0.0302262769,
+         0.0047523617,
+        -0.0029696854
+    },
+    {
+        -0.0011262266,          /* Filter 7 */
+        -0.0086666380,
+        -0.0223584207,
+        -0.0487878398,
+         0.8636754069,
+         0.1824841563,
+         0.0308864956,
+         0.0065113237,
+        -0.0018686358
+    },
+    {
+        -0.0003622774,          /* Filter 8 */
+        -0.0080155088,
+        -0.0233165437,
+        -0.0583361774,
+         0.9093185032,
+         0.1432690480,
+         0.0307901140,
+         0.0080531155,
+        -0.0007659096
+    },
+    {
+         0.0004285425,          /* Filter 9 */
+        -0.0071996967,
+        -0.0239181901,
+        -0.0677960213,
+         0.9527307304,
+         0.1065807242,
+         0.0299900191,
+         0.0093587151,
+         0.0003183408
+    },
+    {
+         0.0012316933,          /* Filter 10 */
+        -0.0062301368,
+        -0.0241376359,
+        -0.0769959031,
+         0.9935863233,
+         0.0725519600,
+         0.0285475474,
+         0.0104140102,
+         0.0013648323
+    },
+    {
+         0.0020320508,          /* Filter 11 */
+        -0.0051210137,
+        -0.0239551212,
+        -0.0857546018,
+         1.0315754934,
+         0.0412866769,
+         0.0265310587,
+         0.0112098711,
+         0.0023554782
+    },
+    {
+         0.0028141763,          /* Filter 12 */
+        -0.0038896008,
+        -0.0233574765,
+        -0.0938829156,
+         1.0664075323,
+         0.0128597894,
+         0.0240144782,
+         0.0117421344,
+         0.0032736852
+    },
+    {
+         0.0035625973,          /* Filter 13 */
+        -0.0025560369,
+        -0.0223386620,
+        -0.1011856112,
+         1.0978137424,
+        -0.0126826277,
+         0.0210758250,
+         0.0120115019,
+         0.0041046165
+    },
+    {
+         0.0042620971,          /* Filter 14 */
+        -0.0011430452,
+        -0.0209002083,
+        -0.1074635265,
+         1.1255501596,
+        -0.0353228594,
+         0.0177957527,
+         0.0120233576,
+         0.0048354157
+    },
+    {
+         0.0048980053,          /* Filter 15 */
+         0.0003244045,
+        -0.0190515462,
+        -0.1125158080,
+         1.1494000377,
+        -0.0550707703,
+         0.0142561191,
+         0.0117875105,
+         0.0054553898
+    },
+    {
+         0.0054564857,          /* Filter 16 */
+         0.0018194852,
+        -0.0168102168,
+        -0.1161422557,
+         1.1691760633,
+        -0.0719627404,
+         0.0105386068,
+         0.0113178688,
+         0.0059561483
+    },
+    {
+         0.0059248154,          /* Filter 17 */
+         0.0033139475,
+        -0.0142019526,
+        -0.1181457502,
+         1.1847222770,
+        -0.0860603350,
+         0.0067234125,
+         0.0106320540,
+         0.0063316975
+    },
+    {
+         0.0062916504,          /* Filter 18 */
+         0.0047785946,
+        -0.0112606234,
+        -0.1183347329,
+         1.1959156771,
+        -0.0974487311,
+         0.0028880206,
+         0.0097509621,
+         0.0065784888
+    },
+    {
+         0.0065472715,          /* Filter 19 */
+         0.0061837898,
+        -0.0080280420,
+        -0.1165257094,
+         1.2026674866,
+        -0.1062349247,
+        -0.0008939235,
+         0.0086982833,
+         0.0066954225
+    },
+    {
+         0.0066838062,          /* Filter 20 */
+         0.0074999881,
+        -0.0045536271,
+        -0.1125457458,
+         1.2049240699,
+        -0.1125457458,
+        -0.0045536271,
+         0.0074999881,
+         0.0066838062
+    },
+    {
+         0.0066954225,          /* Filter 21 */
+         0.0086982833,
+        -0.0008939235,
+        -0.1062349247,
+         1.2026674866,
+        -0.1165257094,
+        -0.0080280420,
+         0.0061837898,
+         0.0065472715
+    },
+    {
+         0.0065784888,          /* Filter 22 */
+         0.0097509621,
+         0.0028880206,
+        -0.0974487311,
+         1.1959156771,
+        -0.1183347329,
+        -0.0112606234,
+         0.0047785946,
+         0.0062916504
+    },
+    {
+         0.0063316975,          /* Filter 23 */
+         0.0106320540,
+         0.0067234125,
+        -0.0860603350,
+         1.1847222770,
+        -0.1181457502,
+        -0.0142019526,
+         0.0033139475,
+         0.0059248154
+    },
+    {
+         0.0059561483,          /* Filter 24 */
+         0.0113178688,
+         0.0105386068,
+        -0.0719627404,
+         1.1691760633,
+        -0.1161422557,
+        -0.0168102168,
+         0.0018194852,
+         0.0054564857
+    },
+    {
+         0.0054553898,          /* Filter 25 */
+         0.0117875105,
+         0.0142561191,
+        -0.0550707703,
+         1.1494000377,
+        -0.1125158080,
+        -0.0190515462,
+         0.0003244045,
+         0.0048980053
+    },
+    {
+         0.0048354157,          /* Filter 26 */
+         0.0120233576,
+         0.0177957527,
+        -0.0353228594,
+         1.1255501596,
+        -0.1074635265,
+        -0.0209002083,
+        -0.0011430452,
+         0.0042620971
+    },
+    {
+         0.0041046165,          /* Filter 27 */
+         0.0120115019,
+         0.0210758250,
+        -0.0126826277,
+         1.0978137424,
+        -0.1011856112,
+        -0.0223386620,
+        -0.0025560369,
+         0.0035625973
+    },
+    {
+         0.0032736852,          /* Filter 28 */
+         0.0117421344,
+         0.0240144782,
+         0.0128597894,
+         1.0664075323,
+        -0.0938829156,
+        -0.0233574765,
+        -0.0038896008,
+         0.0028141763
+    },
+    {
+         0.0023554782,          /* Filter 29 */
+         0.0112098711,
+         0.0265310587,
+         0.0412866769,
+         1.0315754934,
+        -0.0857546018,
+        -0.0239551212,
+        -0.0051210137,
+         0.0020320508
+    },
+    {
+         0.0013648323,          /* Filter 30 */
+         0.0104140102,
+         0.0285475474,
+         0.0725519600,
+         0.9935863233,
+        -0.0769959031,
+        -0.0241376359,
+        -0.0062301368,
+         0.0012316933
+    },
+    {
+         0.0003183408,          /* Filter 31 */
+         0.0093587151,
+         0.0299900191,
+         0.1065807242,
+         0.9527307304,
+        -0.0677960213,
+        -0.0239181901,
+        -0.0071996967,
+         0.0004285425
+    },
+    {
+        -0.0007659096,          /* Filter 32 */
+         0.0080531155,
+         0.0307901140,
+         0.1432690480,
+         0.9093185032,
+        -0.0583361774,
+        -0.0233165437,
+        -0.0080155088,
+        -0.0003622774
+    },
+    {
+        -0.0018686358,          /* Filter 33 */
+         0.0065113237,
+         0.0308864956,
+         0.1824841563,
+         0.8636754069,
+        -0.0487878398,
+        -0.0223584207,
+        -0.0086666380,
+        -0.0011262266
+    },
+    {
+        -0.0029696854,          /* Filter 34 */
+         0.0047523617,
+         0.0302262769,
+         0.2240649005,
+         0.8161399423,
+        -0.0393111426,
+        -0.0210748106,
+        -0.0091454978,
+        -0.0018496023
+    },
+    {
+        -0.0040483891,          /* Filter 35 */
+         0.0027999985,
+         0.0287663895,
+         0.2678225635,
+         0.7670600056,
+        -0.0300535107,
+        -0.0195012095,
+        -0.0094478866,
+        -0.0025197700
+    },
+    {
+        -0.0050839319,          /* Filter 36 */
+         0.0006824956,
+         0.0264748749,
+         0.3135419896,
+         0.7167894869,
+        -0.0211485021,
+        -0.0176768181,
+        -0.0095729633,
+        -0.0031253709
+    },
+    {
+        -0.0060557371,          /* Filter 37 */
+        -0.0015677363,
+         0.0233320755,
+         0.3609830295,
+         0.6656848457,
+        -0.0127148737,
+        -0.0156437084,
+        -0.0095231635,
+        -0.0036565006
+    },
+    {
+        -0.0069438567,          /* Filter 38 */
+        -0.0039145680,
+         0.0193317049,
+         0.4098822897,
+         0.6141017035,
+        -0.0048558766,
+        -0.0134459768,
+        -0.0093040596,
+        -0.0041048584
+    },
+    {
+        -0.0077293609,          /* Filter 39 */
+        -0.0063186648,
+         0.0144817755,
+         0.4599551720,
+         0.5623914901,
+         0.0023412184,
+        -0.0111288952,
+        -0.0089241700,
+        -0.0044638629
+    },
+};
+
 static int fake_get_bit(void *user_data)
 {
     return 1;
@@ -289,8 +740,7 @@ static __inline__ int get_scrambled_bit(v22bis_state_t *s)
 {
     int bit;
 
-    bit = s->current_get_bit(s->user_data);
-    if ((bit & 2))
+    if ((bit = s->current_get_bit(s->user_data)) == PUTBIT_END_OF_DATA)
     {
         /* Fill out this symbol with ones, and prepare to send
            the rest of the shutdown sequence. */
@@ -302,9 +752,9 @@ static __inline__ int get_scrambled_bit(v22bis_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-static complex_t training_get(v22bis_state_t *s)
+static complexf_t training_get(v22bis_state_t *s)
 {
-    complex_t z;
+    complexf_t z;
     int bits;
 
     /* V.22bis training sequence */
@@ -313,7 +763,7 @@ static complex_t training_get(v22bis_state_t *s)
     case V22BIS_TRAINING_STAGE_INITIAL_SILENCE:
         /* Segment 1: silence */
         s->tx_constellation_state = 0;
-        z = complex_set(0.0f, 0.0f);
+        z = complex_setf(0.0f, 0.0f);
         if (s->caller)
         {
             /* The caller just waits for a signal from the far end, which should be unscrambled ones */
@@ -464,14 +914,14 @@ span_log(&s->logging, SPAN_LOG_FLOW, "U0011 Tx 0x%02x\n", s->tx_constellation_st
         break;
     case V22BIS_TRAINING_STAGE_PARKED:
     default:
-        z = complex_set(0.0f, 0.0f);
+        z = complex_setf(0.0f, 0.0f);
         break;
     }
     return z;
 }
 /*- End of function --------------------------------------------------------*/
 
-static complex_t getbaud(v22bis_state_t *s)
+static complexf_t getbaud(v22bis_state_t *s)
 {
     int bits;
 
@@ -487,7 +937,7 @@ static complex_t getbaud(v22bis_state_t *s)
     if (s->shutdown)
     {
         if (++s->shutdown > 10)
-            return complex_set(0.0f, 0.0f);
+            return complex_setf(0.0f, 0.0f);
     }
     /* The first two bits define the quadrant */
     bits = get_scrambled_bit(s);
@@ -507,77 +957,38 @@ static complex_t getbaud(v22bis_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-int v22bis_tx(v22bis_state_t *s, int16_t *amp, int len)
+int v22bis_tx(v22bis_state_t *s, int16_t amp[], int len)
 {
-    complex_t x;
-    complex_t z;
+    complexf_t x;
+    complexf_t z;
     int i;
     int sample;
     float famp;
-    /* You might expect the optimum weights here to be 0.75 and 0.25. However,
-       the RRC filter warps things a bit. These values were arrived at, through
-       simulation - minimise the variance between a 3 times oversampled approach
-       and the weighted approach. */
-    static const float weights[4] = {0.0f, 0.68f, 0.32f, 0.0f};
-#define PULSESHAPER_GAIN    13.32182907f
-    static const float pulseshaper[] =
-    {
-        /* Raised root cosine pulse shaping; Beta = 0.75; 4 symbols either
-           side of the centre. Only one side of the filter is here, as the
-           other half is just a mirror image. */
-        +0.0066813026f, +0.0059389092f, +0.0040872704f, +0.0013503515f,
-        -0.0018778344f, -0.0050863959f, -0.0077248371f, -0.0092935151f,
-        -0.0094333172f, -0.0079995719f, -0.0051065519f, -0.0011325857f,
-        +0.0033186289f, +0.0074981643f, +0.0106241551f, +0.0120108745f,
-        +0.0111950748f, +0.0080386441f, +0.0027883861f, -0.0039213358f,
-        -0.0111297221f, -0.0176716757f, -0.0223483359f, -0.0241244913f,
-        -0.0223248491f, -0.0167982031f, -0.0080199318f, +0.0028908545f,
-        +0.0142532693f, +0.0240065608f, +0.0299785494f, +0.0302133859f,
-        +0.0233201208f, +0.0087965076f, -0.0127190626f, -0.0393099439f,
-        -0.0677896959f, -0.0938726444f, -0.1125034692f, -0.1183225571f,
-        -0.1062250888f, -0.0719569752f, -0.0126819092f, +0.0725475817f,
-        +0.1824755585f, +0.3135308122f, +0.4599435153f, +0.6140917443f,
-        +0.7670536014f, +0.9093168557f, +1.0315789319f, +1.1255580839f,
-        +1.1847332680f, +1.2049361492f
-    };
 
     if (s->shutdown > 10)
         return 0;
     for (sample = 0;  sample < len;  sample++)
     {
-        if ((s->tx_baud_phase += 3) > 40)
+        if ((s->tx_baud_phase += 3) >= 40)
         {
             s->tx_baud_phase -= 40;
-            x = getbaud(s);
-            /* Use a weighted value for the first sample of the baud to correct
-               for a baud not being an integral number of samples long. This is
-               almost as good as 3 times oversampling to a common multiple of the
-               baud and sampling rates, but requires less compute. */
-            s->tx_rrc_filter[s->tx_rrc_filter_step].re =
-            s->tx_rrc_filter[s->tx_rrc_filter_step + V22BIS_TX_FILTER_STEPS].re = x.re - (x.re - s->current_point.re)*weights[s->tx_baud_phase];
-            s->tx_rrc_filter[s->tx_rrc_filter_step].im =
-            s->tx_rrc_filter[s->tx_rrc_filter_step + V22BIS_TX_FILTER_STEPS].im = x.im - (x.im - s->current_point.im)*weights[s->tx_baud_phase];
-            s->current_point = x;
-        }
-        else
-        {
             s->tx_rrc_filter[s->tx_rrc_filter_step] =
-            s->tx_rrc_filter[s->tx_rrc_filter_step + V22BIS_TX_FILTER_STEPS] = s->current_point;
+            s->tx_rrc_filter[s->tx_rrc_filter_step + V22BIS_TX_FILTER_STEPS] = getbaud(s);
+            if (++s->tx_rrc_filter_step >= V22BIS_TX_FILTER_STEPS)
+                s->tx_rrc_filter_step = 0;
         }
-        if (++s->tx_rrc_filter_step >= V22BIS_TX_FILTER_STEPS)
-            s->tx_rrc_filter_step = 0;
         /* Root raised cosine pulse shaping at baseband */
-        x.re = pulseshaper[V22BIS_TX_FILTER_STEPS >> 1]*s->tx_rrc_filter[(V22BIS_TX_FILTER_STEPS >> 1) + s->tx_rrc_filter_step].re;
-        x.im = pulseshaper[V22BIS_TX_FILTER_STEPS >> 1]*s->tx_rrc_filter[(V22BIS_TX_FILTER_STEPS >> 1) + s->tx_rrc_filter_step].im;
-        for (i = 0;  i < (V22BIS_TX_FILTER_STEPS >> 1);  i++)
+        x.re = 0.0f;
+        x.im = 0.0f;
+        for (i = 0;  i < V22BIS_TX_FILTER_STEPS;  i++)
         {
-            x.re += pulseshaper[i]*(s->tx_rrc_filter[s->tx_rrc_filter_step + i].re + s->tx_rrc_filter[V22BIS_TX_FILTER_STEPS - 1 + s->tx_rrc_filter_step - i].re);
-            x.im += pulseshaper[i]*(s->tx_rrc_filter[s->tx_rrc_filter_step + i].im + s->tx_rrc_filter[V22BIS_TX_FILTER_STEPS - 1 + s->tx_rrc_filter_step - i].im);
+            x.re += pulseshaper[39 - s->tx_baud_phase][i]*s->tx_rrc_filter[i + s->tx_rrc_filter_step].re;
+            x.im += pulseshaper[39 - s->tx_baud_phase][i]*s->tx_rrc_filter[i + s->tx_rrc_filter_step].im;
         }
         /* Now create and modulate the carrier */
         z = dds_complexf(&(s->tx_carrier_phase), s->tx_carrier_phase_rate);
         famp = (x.re*z.re - x.im*z.im)*s->tx_gain;
-        if (s->guard_phase_rate  &&  (s->current_point.re != 0.0f  ||  s->current_point.im != 0.0f))
+        if (s->guard_phase_rate  &&  (s->tx_rrc_filter[s->tx_rrc_filter_step].re != 0.0f  ||  s->tx_rrc_filter[i + s->tx_rrc_filter_step].im != 0.0f))
         {
             /* Add the guard tone */
             famp += dds_modf(&(s->guard_phase), s->guard_phase_rate, s->guard_level, 0);
@@ -601,9 +1012,8 @@ void v22bis_tx_power(v22bis_state_t *s, float power)
 static int v22bis_tx_restart(v22bis_state_t *s, int bit_rate)
 {
     s->bit_rate = bit_rate;
-    memset(s->tx_rrc_filter, 0, sizeof(s->tx_rrc_filter));
+    cvec_zerof(s->tx_rrc_filter, sizeof(s->tx_rrc_filter)/sizeof(s->tx_rrc_filter[0]));
     s->tx_rrc_filter_step = 0;
-    s->current_point = complex_set(0.0f, 0.0f);
     s->tx_scramble_reg = 0;
     s->tx_scrambler_pattern_count = 0;
     s->tx_training = V22BIS_TRAINING_STAGE_INITIAL_SILENCE;
