@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_rx.c,v 1.61 2006/05/03 14:29:23 steveu Exp $
+ * $Id: v27ter_rx.c,v 1.72 2006/10/24 13:45:27 steveu Exp $
  */
 
 /*! \file */
@@ -36,10 +35,12 @@
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
+#include "spandsp/complex.h"
+#include "spandsp/vector_float.h"
+#include "spandsp/complex_vector_float.h"
 #include "spandsp/async.h"
 #include "spandsp/power_meter.h"
 #include "spandsp/arctan2.h"
-#include "spandsp/complex.h"
 #include "spandsp/dds.h"
 #include "spandsp/complex_filters.h"
 
@@ -50,9 +51,13 @@
    signal to a static constellation, even though dealing with differences is all
    that is necessary. */
 
+#define CARRIER_NOMINAL_FREQ        1800.0f
+#define EQUALIZER_DELTA             0.25f
+
 /* Segments of the training sequence */
 /* V.27ter defines a long and a short sequence. FAX doesn't use the
    short sequence, so it is not implemented here. */
+#define V27TER_TRAINING_SEG_3_LEN   50
 #define V27TER_TRAINING_SEG_5_LEN   1074
 #define V27TER_TRAINING_SEG_6_LEN   8
 
@@ -67,9 +72,7 @@ enum
     TRAINING_STAGE_PARKED
 };
 
-#define EQUALIZER_DELTA         0.25f
-
-static const complex_t v27ter_constellation[8] =
+static const complexf_t v27ter_constellation[8] =
 {
     { 1.414f,  0.0f},       /*   0deg */
     { 1.0f,    1.0f},       /*  45deg */
@@ -85,7 +88,7 @@ static const complex_t v27ter_constellation[8] =
    baud rate 1600; shifted to centre at 1800Hz; complex. */
 #define PULSESHAPER_4800_GAIN           (2.4975f*2.0f)
 #define PULSESHAPER_4800_COEFF_SETS     8
-static const complex_t pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS][V27TER_RX_4800_FILTER_STEPS] =
+static const complexf_t pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS][V27TER_RX_4800_FILTER_STEPS] =
 {
     {
         {-0.0050334423f, -0.0025646669f},   /* Filter 0 */
@@ -325,7 +328,7 @@ static const complex_t pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS][V27TER_RX_4
    baud rate 1200; shifted to centre at 1800Hz; complex. */
 #define PULSESHAPER_2400_GAIN           2.223f
 #define PULSESHAPER_2400_COEFF_SETS     12
-static const complex_t pulseshaper_2400[PULSESHAPER_2400_COEFF_SETS][V27TER_RX_2400_FILTER_STEPS] =
+static const complexf_t pulseshaper_2400[PULSESHAPER_2400_COEFF_SETS][V27TER_RX_2400_FILTER_STEPS] =
 {
     {
         { 0.0036326320f,  0.0018509185f},   /* Filter 0 */
@@ -687,9 +690,8 @@ float v27ter_rx_symbol_timing_correction(v27ter_rx_state_t *s)
 {
     int steps_per_symbol;
     
-    steps_per_symbol = (s->bit_rate == 4800)  ?  PULSESHAPER_4800_COEFF_SETS*5/2  :  PULSESHAPER_2400_COEFF_SETS*20/(3*2);
-
-    return (float) s->gardner_total_correction/(float) steps_per_symbol;
+    steps_per_symbol = (s->bit_rate == 4800)  ?  PULSESHAPER_4800_COEFF_SETS*5  :  PULSESHAPER_2400_COEFF_SETS*20/3;
+    return (float) s->total_baud_timing_correction/(float) steps_per_symbol;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -702,12 +704,12 @@ float v27ter_rx_signal_power(v27ter_rx_state_t *s)
 void v27ter_rx_signal_cutoff(v27ter_rx_state_t *s, float cutoff)
 {
     /* The 0.4 factor allows for the gain of the DC blocker */
-    s->carrier_on_power = power_meter_level_dbm0(cutoff - 2.5f)*0.4;
-    s->carrier_off_power = power_meter_level_dbm0(cutoff + 2.5f)*0.4;
+    s->carrier_on_power = power_meter_level_dbm0(cutoff + 2.5f)*0.4f;
+    s->carrier_off_power = power_meter_level_dbm0(cutoff - 2.5f)*0.4f;
 }
 /*- End of function --------------------------------------------------------*/
 
-int v27ter_rx_equalizer_state(v27ter_rx_state_t *s, complex_t **coeffs)
+int v27ter_rx_equalizer_state(v27ter_rx_state_t *s, complexf_t **coeffs)
 {
     *coeffs = s->eq_coeff;
     return V27TER_EQUALIZER_PRE_LEN + 1 + V27TER_EQUALIZER_POST_LEN;
@@ -716,21 +718,14 @@ int v27ter_rx_equalizer_state(v27ter_rx_state_t *s, complex_t **coeffs)
 
 static void equalizer_save(v27ter_rx_state_t *s)
 {
-    int i;
-
-    for (i = 0;  i < V27TER_EQUALIZER_PRE_LEN + 1 + V27TER_EQUALIZER_POST_LEN;  i++)
-        s->eq_coeff_save[i] = s->eq_coeff[i];
+    cvec_copyf(s->eq_coeff_save, s->eq_coeff, V27TER_EQUALIZER_PRE_LEN + 1 + V27TER_EQUALIZER_POST_LEN);
 }
 /*- End of function --------------------------------------------------------*/
 
 static void equalizer_restore(v27ter_rx_state_t *s)
 {
-    int i;
-
-    for (i = 0;  i < V27TER_EQUALIZER_PRE_LEN + 1 + V27TER_EQUALIZER_POST_LEN;  i++)
-        s->eq_coeff[i] = s->eq_coeff_save[i];
-    for (i = 0;  i <= V27TER_EQUALIZER_MASK;  i++)
-        s->eq_buf[i] = complex_set(0.0f, 0.0f);
+    cvec_copyf(s->eq_coeff, s->eq_coeff_save, V27TER_EQUALIZER_PRE_LEN + 1 + V27TER_EQUALIZER_POST_LEN);
+    cvec_zerof(s->eq_buf, V27TER_EQUALIZER_MASK);
 
     s->eq_put_step = (s->bit_rate == 4800)  ?  PULSESHAPER_4800_COEFF_SETS*5/2  :  PULSESHAPER_2400_COEFF_SETS*20/(3*2);
     s->eq_step = 0;
@@ -740,14 +735,10 @@ static void equalizer_restore(v27ter_rx_state_t *s)
 
 static void equalizer_reset(v27ter_rx_state_t *s)
 {
-    int i;
-
     /* Start with an equalizer based on everything being perfect */
-    for (i = 0;  i < V27TER_EQUALIZER_PRE_LEN + 1 + V27TER_EQUALIZER_POST_LEN;  i++)
-        s->eq_coeff[i] = complex_set(0.0f, 0.0f);
-    s->eq_coeff[V27TER_EQUALIZER_PRE_LEN] = complex_set(1.414f, 0.0f);
-    for (i = 0;  i <= V27TER_EQUALIZER_MASK;  i++)
-        s->eq_buf[i] = complex_set(0.0f, 0.0f);
+    cvec_zerof(s->eq_coeff, V27TER_EQUALIZER_PRE_LEN + 1 + V27TER_EQUALIZER_POST_LEN);
+    s->eq_coeff[V27TER_EQUALIZER_PRE_LEN] = complex_setf(1.414f, 0.0f);
+    cvec_zerof(s->eq_buf, V27TER_EQUALIZER_MASK);
 
     s->eq_put_step = (s->bit_rate == 4800)  ?  PULSESHAPER_4800_COEFF_SETS*5/2  :  PULSESHAPER_2400_COEFF_SETS*20/(3*2);
     s->eq_step = 0;
@@ -755,35 +746,35 @@ static void equalizer_reset(v27ter_rx_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ complex_t equalizer_get(v27ter_rx_state_t *s)
+static __inline__ complexf_t equalizer_get(v27ter_rx_state_t *s)
 {
     int i;
     int p;
-    complex_t z;
-    complex_t z1;
+    complexf_t z;
+    complexf_t z1;
 
     /* Get the next equalized value. */
-    z = complex_set(0.0, 0.0);
+    z = complex_setf(0.0, 0.0);
     p = s->eq_step - 1;
     for (i = 0;  i < V27TER_EQUALIZER_PRE_LEN + 1 + V27TER_EQUALIZER_POST_LEN;  i++)
     {
         p = (p - 1) & V27TER_EQUALIZER_MASK;
-        z1 = complex_mul(&s->eq_coeff[i], &s->eq_buf[p]);
-        z = complex_add(&z, &z1);
+        z1 = complex_mulf(&s->eq_coeff[i], &s->eq_buf[p]);
+        z = complex_addf(&z, &z1);
     }
     return z;
 }
 /*- End of function --------------------------------------------------------*/
 
-static void tune_equalizer(v27ter_rx_state_t *s, const complex_t *z, const complex_t *target)
+static void tune_equalizer(v27ter_rx_state_t *s, const complexf_t *z, const complexf_t *target)
 {
     int i;
     int p;
-    complex_t ez;
-    complex_t z1;
+    complexf_t ez;
+    complexf_t z1;
 
     /* Find the x and y mismatch from the exact constellation position. */
-    ez = complex_sub(target, z);
+    ez = complex_subf(target, z);
     ez.re *= s->eq_delta;
     ez.im *= s->eq_delta;
 
@@ -791,9 +782,9 @@ static void tune_equalizer(v27ter_rx_state_t *s, const complex_t *z, const compl
     for (i = 0;  i < V27TER_EQUALIZER_PRE_LEN + 1 + V27TER_EQUALIZER_POST_LEN;  i++)
     {
         p = (p - 1) & V27TER_EQUALIZER_MASK;
-        z1 = complex_conj(&s->eq_buf[p]);
-        z1 = complex_mul(&ez, &z1);
-        s->eq_coeff[i] = complex_add(&s->eq_coeff[i], &z1);
+        z1 = complex_conjf(&s->eq_buf[p]);
+        z1 = complex_mulf(&ez, &z1);
+        s->eq_coeff[i] = complex_addf(&s->eq_coeff[i], &z1);
         /* Leak a little to tame uncontrolled wandering */
         s->eq_coeff[i].re *= 0.9999f;
         s->eq_coeff[i].im *= 0.9999f;
@@ -801,7 +792,7 @@ static void tune_equalizer(v27ter_rx_state_t *s, const complex_t *z, const compl
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ void track_carrier(v27ter_rx_state_t *s, const complex_t *z, const complex_t *target)
+static __inline__ void track_carrier(v27ter_rx_state_t *s, const complexf_t *z, const complexf_t *target)
 {
     float error;
 
@@ -873,7 +864,7 @@ static __inline__ void put_bit(v27ter_rx_state_t *s, int bit)
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ int find_quadrant(const complex_t *z)
+static __inline__ int find_quadrant(const complexf_t *z)
 {
     int b1;
     int b2;
@@ -885,7 +876,7 @@ static __inline__ int find_quadrant(const complex_t *z)
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ int find_octant(complex_t *z)
+static __inline__ int find_octant(complexf_t *z)
 {
     float abs_re;
     float abs_im;
@@ -914,7 +905,7 @@ static __inline__ int find_octant(complex_t *z)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void decode_baud(v27ter_rx_state_t *s, complex_t *z)
+static void decode_baud(v27ter_rx_state_t *s, complexf_t *z)
 {
     static const uint8_t phase_steps_4800[8] =
     {
@@ -959,14 +950,14 @@ static void decode_baud(v27ter_rx_state_t *s, complex_t *z)
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complex_t *sample)
+static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complexf_t *sample)
 {
     static const int abab_pos[2] =
     {
         0, 4
     };
-    complex_t z;
-    complex_t zz;
+    complexf_t z;
+    complexf_t zz;
     float p;
     float q;
     int i;
@@ -1005,7 +996,7 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complex_t *
            when the true symbol boundary is close to a sample boundary. */
         //span_log(&s->logging, SPAN_LOG_FLOW, "Hop %d\n", s->gardner_integrate);
         s->eq_put_step += (s->gardner_integrate/256);
-        s->gardner_total_correction += (s->gardner_integrate/256);
+        s->total_baud_timing_correction += (s->gardner_integrate/256);
         if (s->qam_report)
             s->qam_report(s->qam_user_data, NULL, NULL, s->gardner_integrate);
         s->gardner_integrate = 0;
@@ -1068,14 +1059,25 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complex_t *
                     s->carrier_phase_rate += 3*(ang/40);
             }
             span_log(&s->logging, SPAN_LOG_FLOW, "Coarse carrier frequency %7.2f (%d)\n", dds_frequencyf(s->carrier_phase_rate), s->training_count);
+            /* Check if the carrier frequency is plausible */
+            if (s->carrier_phase_rate < dds_phase_ratef(CARRIER_NOMINAL_FREQ - 20.0f)
+                ||
+                s->carrier_phase_rate > dds_phase_ratef(CARRIER_NOMINAL_FREQ + 20.0f))
+            {
+               span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (sequence failed)\n");
+               /* Park this modem */
+               s->in_training = TRAINING_STAGE_PARKED;
+               s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
+               break;
+            }
 
             /* Make a step shift in the phase, to pull it into line. We need to rotate the equalizer
                buffer, as well as the carrier phase, for this to play out nicely. */
             angle += 0x80000000;
             p = angle*2.0f*3.14159f/(65536.0f*65536.0f);
-            zz = complex_set(cosf(p), -sinf(p));
+            zz = complex_setf(cosf(p), -sinf(p));
             for (i = 0;  i <= V27TER_EQUALIZER_MASK;  i++)
-                s->eq_buf[i] = complex_mul(&s->eq_buf[i], &zz);
+                s->eq_buf[i] = complex_mulf(&s->eq_buf[i], &zz);
             s->carrier_phase += angle;
 
             s->gardner_step = 1;
@@ -1087,9 +1089,14 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complex_t *
             s->training_count = 1;
             s->in_training = TRAINING_STAGE_TRAIN_ON_ABAB;
         }
-        else
+        else if (++s->training_count > V27TER_TRAINING_SEG_3_LEN)
         {
-            s->training_count++;
+            /* This is bogus. There are not this many bits in this section
+               of a real training sequence. */
+            span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (sequence failed)\n");
+            /* Park this modem */
+            s->in_training = TRAINING_STAGE_PARKED;
+            s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
         }
         break;
     case TRAINING_STAGE_TRAIN_ON_ABAB:
@@ -1114,10 +1121,10 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complex_t *
         decode_baud(s, &z);
         /* Measure the training error */
         if (s->bit_rate == 4800)
-            zz = complex_sub(&z, &v27ter_constellation[s->constellation_state]);
+            zz = complex_subf(&z, &v27ter_constellation[s->constellation_state]);
         else
-            zz = complex_sub(&z, &v27ter_constellation[s->constellation_state << 1]);
-        s->training_error += power(&zz);
+            zz = complex_subf(&z, &v27ter_constellation[s->constellation_state << 1]);
+        s->training_error += powerf(&zz);
         if (++s->training_count >= V27TER_TRAINING_SEG_6_LEN)
         {
             if ((s->bit_rate == 4800  &&  s->training_error < 1.0f)
@@ -1160,14 +1167,14 @@ static __inline__ void process_half_baud(v27ter_rx_state_t *s, const complex_t *
 }
 /*- End of function --------------------------------------------------------*/
 
-int v27ter_rx(v27ter_rx_state_t *s, const int16_t *amp, int len)
+int v27ter_rx(v27ter_rx_state_t *s, const int16_t amp[], int len)
 {
     int i;
     int j;
     int step;
-    complex_t z;
-    complex_t zz;
-    complex_t samplex;
+    complexf_t z;
+    complexf_t zz;
+    complexf_t samplex;
     int32_t power;
 
     if (s->bit_rate == 4800)
@@ -1196,8 +1203,8 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t *amp, int len)
                            few bits through the filters before stopping. */
                         v27ter_rx_restart(s, s->bit_rate, FALSE);
                         s->put_bit(s->user_data, PUTBIT_CARRIER_DOWN);
+                        continue;
                     }
-                    continue;
                 }
             }
             else
@@ -1241,7 +1248,7 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t *amp, int len)
                     samplex.re = zz.re*s->agc_scaling;
                     samplex.im = zz.im*s->agc_scaling;
                     /* Shift to baseband - since this is done in a full complex form, the
-                       result is clean, and requires no further filtering apart from the
+                       result is clean, and requires no further filtering, apart from the
                        equalizer. */
                     zz.re = samplex.re*z.re - samplex.im*z.im;
                     zz.im = -samplex.re*z.im - samplex.im*z.re;
@@ -1261,8 +1268,8 @@ int v27ter_rx(v27ter_rx_state_t *s, const int16_t *amp, int len)
 
             /* There should be no DC in the signal, but sometimes there is.
                We need to measure the power with the DC blocked, but not using
-               a slow to respond DC blocker. Use the most elementart HPF. */
-            power = power_meter_update(&(s->power), amp[i] - s->last_sample);
+               a slow to respond DC blocker. Use the most elementary HPF. */
+            power = power_meter_update(&(s->power), (amp[i] - s->last_sample) >> 1);
             s->last_sample = amp[i];
             //span_log(&s->logging, SPAN_LOG_FLOW, "Power = %f\n", power_meter_dbm0(&(s->power)));
             if (s->carrier_present)
@@ -1348,7 +1355,7 @@ int v27ter_rx_restart(v27ter_rx_state_t *s, int rate, int old_train)
         return -1;
     s->bit_rate = rate;
 
-    memset(s->rrc_filter, 0, sizeof(s->rrc_filter));
+    vec_zerof(s->rrc_filter, sizeof(s->rrc_filter)/sizeof(s->rrc_filter[0]));
     s->rrc_filter_step = 0;
 
     s->scramble_reg = 0x3C;
@@ -1376,7 +1383,7 @@ int v27ter_rx_restart(v27ter_rx_state_t *s, int rate, int old_train)
     }
     else
     {
-        s->carrier_phase_rate = dds_phase_ratef(1800.0f);
+        s->carrier_phase_rate = dds_phase_ratef(CARRIER_NOMINAL_FREQ);
         s->agc_scaling = 0.0005f;
         equalizer_reset(s);
     }
@@ -1384,7 +1391,7 @@ int v27ter_rx_restart(v27ter_rx_state_t *s, int rate, int old_train)
     s->last_sample = 0;
 
     s->gardner_integrate = 0;
-    s->gardner_total_correction = 0;
+    s->total_baud_timing_correction = 0;
     s->gardner_step = 512;
     s->baud_phase = 0;
 
@@ -1400,9 +1407,7 @@ v27ter_rx_state_t *v27ter_rx_init(v27ter_rx_state_t *s, int rate, put_bit_func_t
             return NULL;
     }
     memset(s, 0, sizeof(*s));
-    /* The 0.4 factor allows for the gain of the DC blocker */
-    s->carrier_on_power = power_meter_level_dbm0(-43)*0.4;
-    s->carrier_off_power = power_meter_level_dbm0(-48)*0.4;
+    v27ter_rx_signal_cutoff(s, -45.5f);
     s->put_bit = put_bit;
     s->user_data = user_data;
     span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
