@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v29rx.c,v 1.81 2006/05/05 10:14:21 steveu Exp $
+ * $Id: v29rx.c,v 1.98 2006/10/24 13:45:27 steveu Exp $
  */
 
 /*! \file */
@@ -40,22 +39,26 @@
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
+#include "spandsp/complex.h"
+#include "spandsp/vector_float.h"
+#include "spandsp/complex_vector_float.h"
 #include "spandsp/async.h"
 #include "spandsp/power_meter.h"
 #include "spandsp/arctan2.h"
-#include "spandsp/complex.h"
 #include "spandsp/dds.h"
 #include "spandsp/complex_filters.h"
 
 #include "spandsp/v29rx.h"
 
+#define CARRIER_NOMINAL_FREQ        1700.0f
+#define BAUD_RATE                   2400
+#define EQUALIZER_DELTA             0.21f
+
 /* Segments of the training sequence */
 #define V29_TRAINING_SEG_2_LEN      128
 #define V29_TRAINING_SEG_3_LEN      384
 #define V29_TRAINING_SEG_4_LEN      48
 
-#define EQUALIZER_DELTA             (0.25f*2.0f)
-
 enum
 {
     TRAINING_STAGE_NORMAL_OPERATION = 0,
@@ -68,7 +71,7 @@ enum
     TRAINING_STAGE_PARKED
 };
 
-static const complex_t v29_constellation[16] =
+static const complexf_t v29_constellation[16] =
 {
     { 3.0f,  0.0f},     /*   0deg low  */
     { 1.0f,  1.0f},     /*  45deg low  */
@@ -118,7 +121,7 @@ static const uint8_t space_map_9600[20][20] =
    sample. Only one is symmetric. */
 #define PULSESHAPER_GAIN        10.00736638f
 #define PULSESHAPER_COEFF_SETS  48
-static const complex_t pulseshaper[PULSESHAPER_COEFF_SETS][V29RX_FILTER_STEPS] =
+static const complexf_t pulseshaper[PULSESHAPER_COEFF_SETS][V29_RX_FILTER_STEPS] =
 {
     {
         {-0.0006054906f, -0.0076934875f},       /* Filter 0 */
@@ -1514,6 +1517,15 @@ static const complex_t pulseshaper[PULSESHAPER_COEFF_SETS][V29RX_FILTER_STEPS] =
     },
 };
 
+/* Coefficients for the band edge symbol timing synchroniser */
+#define SYNC_LOW_BAND_EDGE_COEFF_0       1.829281f    /* 2*alpha*cos(low_edge) */
+#define SYNC_LOW_BAND_EDGE_COEFF_1      -0.980100f    /* -alpha^2 */
+#define SYNC_HIGH_BAND_EDGE_COEFF_0     -1.285907f    /* 2*alpha*cos(high_edge) */
+#define SYNC_HIGH_BAND_EDGE_COEFF_1     -0.980100f    /* -alpha^2 */
+#define SYNC_CROSS_CORR_COEFF_A         -0.932131f    /* -alpha^2*sin(freq_diff) */
+#define SYNC_CROSS_CORR_COEFF_B          0.752802f    /* alpha*sin(high_edge) */
+#define SYNC_CROSS_CORR_COEFF_C         -0.378857f    /* -alpha*sin(low_edge) */
+
 float v29_rx_carrier_frequency(v29_rx_state_t *s)
 {
     return dds_frequencyf(s->carrier_phase_rate);
@@ -1522,15 +1534,15 @@ float v29_rx_carrier_frequency(v29_rx_state_t *s)
 
 float v29_rx_symbol_timing_correction(v29_rx_state_t *s)
 {
-    return (float) s->gardner_total_correction/((float) PULSESHAPER_COEFF_SETS*10.0f/(3.0f*2.0f));
+    return (float) s->total_baud_timing_correction/((float) PULSESHAPER_COEFF_SETS*10.0f/3.0f);
 }
 /*- End of function --------------------------------------------------------*/
 
 void v29_rx_signal_cutoff(v29_rx_state_t *s, float cutoff)
 {
     /* The 0.4 factor allows for the gain of the DC blocker */
-    s->carrier_on_power = power_meter_level_dbm0(cutoff - 2.5f)*0.4;
-    s->carrier_off_power = power_meter_level_dbm0(cutoff + 2.5f)*0.4;
+    s->carrier_on_power = power_meter_level_dbm0(cutoff + 2.5f)*0.4f;
+    s->carrier_off_power = power_meter_level_dbm0(cutoff - 2.5f)*0.4f;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -1540,7 +1552,7 @@ float v29_rx_signal_power(v29_rx_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-int v29_rx_equalizer_state(v29_rx_state_t *s, complex_t **coeffs)
+int v29_rx_equalizer_state(v29_rx_state_t *s, complexf_t **coeffs)
 {
     *coeffs = s->eq_coeff;
     return V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN;
@@ -1549,21 +1561,14 @@ int v29_rx_equalizer_state(v29_rx_state_t *s, complex_t **coeffs)
 
 static void equalizer_save(v29_rx_state_t *s)
 {
-    int i;
-
-    for (i = 0;  i < V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN;  i++)
-        s->eq_coeff_save[i] = s->eq_coeff[i];
+    cvec_copyf(s->eq_coeff_save, s->eq_coeff, V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN);
 }
 /*- End of function --------------------------------------------------------*/
 
 static void equalizer_restore(v29_rx_state_t *s)
 {
-    int i;
-
-    for (i = 0;  i < V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN;  i++)
-        s->eq_coeff[i] = s->eq_coeff_save[i];
-    for (i = 0;  i <= V29_EQUALIZER_MASK;  i++)
-        s->eq_buf[i] = complex_set(0.0f, 0.0f);
+    cvec_copyf(s->eq_coeff, s->eq_coeff_save, V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN);
+    cvec_zerof(s->eq_buf, V29_EQUALIZER_MASK);
 
     s->eq_put_step = PULSESHAPER_COEFF_SETS*10/(3*2) - 1;
     s->eq_step = 0;
@@ -1573,14 +1578,10 @@ static void equalizer_restore(v29_rx_state_t *s)
 
 static void equalizer_reset(v29_rx_state_t *s)
 {
-    int i;
-
     /* Start with an equalizer based on everything being perfect */
-    for (i = 0;  i < V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN;  i++)
-        s->eq_coeff[i] = complex_set(0.0f, 0.0f);
-    s->eq_coeff[V29_EQUALIZER_PRE_LEN] = complex_set(3.0f, 0.0f);
-    for (i = 0;  i <= V29_EQUALIZER_MASK;  i++)
-        s->eq_buf[i] = complex_set(0.0f, 0.0f);
+    cvec_zerof(s->eq_coeff, V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN);
+    s->eq_coeff[V29_EQUALIZER_PRE_LEN] = complex_setf(3.0f, 0.0f);
+    cvec_zerof(s->eq_buf, V29_EQUALIZER_MASK);
 
     s->eq_put_step = PULSESHAPER_COEFF_SETS*10/(3*2) - 1;
     s->eq_step = 0;
@@ -1588,35 +1589,35 @@ static void equalizer_reset(v29_rx_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ complex_t equalizer_get(v29_rx_state_t *s)
+static __inline__ complexf_t equalizer_get(v29_rx_state_t *s)
 {
     int i;
     int p;
-    complex_t z;
-    complex_t z1;
+    complexf_t z;
+    complexf_t z1;
 
     /* Get the next equalized value. */
-    z = complex_set(0.0f, 0.0f);
+    z = complex_setf(0.0f, 0.0f);
     p = s->eq_step - 1;
     for (i = 0;  i < V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN;  i++)
     {
         p = (p - 1) & V29_EQUALIZER_MASK;
-        z1 = complex_mul(&s->eq_coeff[i], &s->eq_buf[p]);
-        z = complex_add(&z, &z1);
+        z1 = complex_mulf(&s->eq_coeff[i], &s->eq_buf[p]);
+        z = complex_addf(&z, &z1);
     }
     return z;
 }
 /*- End of function --------------------------------------------------------*/
 
-static void tune_equalizer(v29_rx_state_t *s, const complex_t *z, const complex_t *target)
+static void tune_equalizer(v29_rx_state_t *s, const complexf_t *z, const complexf_t *target)
 {
     int i;
     int p;
-    complex_t ez;
-    complex_t z1;
+    complexf_t ez;
+    complexf_t z1;
 
     /* Find the x and y mismatch from the exact constellation position. */
-    ez = complex_sub(target, z);
+    ez = complex_subf(target, z);
     ez.re *= s->eq_delta;
     ez.im *= s->eq_delta;
 
@@ -1624,9 +1625,9 @@ static void tune_equalizer(v29_rx_state_t *s, const complex_t *z, const complex_
     for (i = 0;  i < V29_EQUALIZER_PRE_LEN + 1 + V29_EQUALIZER_POST_LEN;  i++)
     {
         p = (p - 1) & V29_EQUALIZER_MASK;
-        z1 = complex_conj(&s->eq_buf[p]);
-        z1 = complex_mul(&ez, &z1);
-        s->eq_coeff[i] = complex_add(&s->eq_coeff[i], &z1);
+        z1 = complex_conjf(&s->eq_buf[p]);
+        z1 = complex_mulf(&ez, &z1);
+        s->eq_coeff[i] = complex_addf(&s->eq_coeff[i], &z1);
         /* Leak a little to tame uncontrolled wandering */
         s->eq_coeff[i].re *= 0.9999f;
         s->eq_coeff[i].im *= 0.9999f;
@@ -1648,7 +1649,7 @@ static int scrambled_training_bit(v29_rx_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ int find_quadrant(const complex_t *z)
+static __inline__ int find_quadrant(const complexf_t *z)
 {
     int b1;
     int b2;
@@ -1660,7 +1661,7 @@ static __inline__ int find_quadrant(const complex_t *z)
 }
 /*- End of function --------------------------------------------------------*/
 
-static __inline__ void track_carrier(v29_rx_state_t *s, const complex_t *z, const complex_t *target)
+static __inline__ void track_carrier(v29_rx_state_t *s, const complexf_t *z, const complexf_t *target)
 {
     float error;
 
@@ -1714,7 +1715,7 @@ static __inline__ void put_bit(v29_rx_state_t *s, int bit)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void decode_baud(v29_rx_state_t *s, complex_t *z)
+static void decode_baud(v29_rx_state_t *s, complexf_t *z)
 {
     static const uint8_t phase_steps_9600[8] =
     {
@@ -1794,7 +1795,7 @@ static void decode_baud(v29_rx_state_t *s, complex_t *z)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void process_half_baud(v29_rx_state_t *s, complex_t *sample)
+static void process_half_baud(v29_rx_state_t *s, complexf_t *sample)
 {
     static const int cdcd_pos[6] =
     {
@@ -1802,18 +1803,18 @@ static void process_half_baud(v29_rx_state_t *s, complex_t *sample)
         0,  3,
         0,  2
     };
-    complex_t z;
-    complex_t zz;
-    const complex_t *target;
+    complexf_t z;
+    complexf_t zz;
+    const complexf_t *target;
+    float v;
     float p;
-    float q;
     int bit;
     int i;
     int j;
     int32_t angle;
     int32_t ang;
 
-    /* This routines processes every half a baud, as we put things into the equalizer at the T/2 rate.
+    /* This routine processes every half a baud, as we put things into the equalizer at the T/2 rate.
        This routine adapts the position of the half baud samples, which the caller takes. */
 
     /* Add a sample to the equalizer's circular buffer, but don't calculate anything
@@ -1822,30 +1823,38 @@ static void process_half_baud(v29_rx_state_t *s, complex_t *sample)
     s->eq_step = (s->eq_step + 1) & V29_EQUALIZER_MASK;
 
     /* On alternate insertions we have a whole baud, and must process it. */
-    if ((s->baud_phase ^= 1))
+    if ((s->baud_half ^= 1))
         return;
 
-    /* Perform a Gardner test for baud alignment on the three most recent samples. */
-    p = s->eq_buf[(s->eq_step - 3) & V29_EQUALIZER_MASK].re
-      - s->eq_buf[(s->eq_step - 1) & V29_EQUALIZER_MASK].re;
-    p *= s->eq_buf[(s->eq_step - 2) & V29_EQUALIZER_MASK].re;
+    /* Symbol timing synchronisation */
+    /* Cross correlate */
+    v = s->symbol_sync_low[1]*s->symbol_sync_high[1]*SYNC_CROSS_CORR_COEFF_A
+      + s->symbol_sync_low[0]*s->symbol_sync_high[1]*SYNC_CROSS_CORR_COEFF_B
+      + s->symbol_sync_low[1]*s->symbol_sync_high[0]*SYNC_CROSS_CORR_COEFF_C;
 
-    q = s->eq_buf[(s->eq_step - 3) & V29_EQUALIZER_MASK].im
-      - s->eq_buf[(s->eq_step - 1) & V29_EQUALIZER_MASK].im;
-    q *= s->eq_buf[(s->eq_step - 2) & V29_EQUALIZER_MASK].im;
+    /* Filter away any DC component  */
+    p = v - s->symbol_sync_dc_filter[1];
+    s->symbol_sync_dc_filter[1] = s->symbol_sync_dc_filter[0];
+    s->symbol_sync_dc_filter[0] = v;
+    /* A little integration will now filter away much of the noise */
+    s->baud_phase -= p;
 
-    s->gardner_integrate += (p + q > 0.0f)  ?  s->gardner_step  :  -s->gardner_step;
+    i = 0;
+    if (s->baud_phase > 1000.0f)
+        i = 5;
+    else if (s->baud_phase < -1000.0f)
+        i = -5;
+    else if (s->baud_phase > 100.0f)
+        i = 1;
+    else if (s->baud_phase < -100.0f)
+        i = -1;
 
-    if (abs(s->gardner_integrate) >= 8)
+    //printf("v = %10.5f %5d - %f %f %d %d\n", v, i, p, s->baud_phase, s->total_baud_timing_correction);
+
+    if (i)
     {
-        /* This integrate and dump approach avoids rapid changes of the equalizer put step.
-           Rapid changes, without hysteresis, are bad. They degrade the equalizer performance
-           when the true symbol boundary is close to a sample boundary. */
-        s->eq_put_step += (s->gardner_integrate/8);
-        s->gardner_total_correction += (s->gardner_integrate/8);
-        if (s->qam_report)
-            s->qam_report(s->qam_user_data, NULL, NULL, s->gardner_integrate);
-        s->gardner_integrate = 0;
+        s->eq_put_step += i;
+        s->total_baud_timing_correction += i;
     }
 
     z = equalizer_get(s);
@@ -1858,7 +1867,7 @@ static void process_half_baud(v29_rx_state_t *s, complex_t *sample)
         target = &v29_constellation[s->constellation_state];
         break;
     case TRAINING_STAGE_SYMBOL_ACQUISITION:
-        /* Allow time for the Gardner algorithm to settle the symbol timing. */
+        /* Allow time for symbol synchronisation to settle the symbol timing. */
         target = &z;
         if (++s->training_count >= 60)
         {
@@ -1901,21 +1910,24 @@ static void process_half_baud(v29_rx_state_t *s, complex_t *sample)
                 s->carrier_phase_rate += 3*(ang/20);
             }
             span_log(&s->logging, SPAN_LOG_FLOW, "Coarse carrier frequency %7.2f\n", dds_frequencyf(s->carrier_phase_rate));
-
+            /* Check if the carrier frequency is plausible */
+            if (s->carrier_phase_rate < dds_phase_ratef(CARRIER_NOMINAL_FREQ - 20.0f)
+                ||
+                s->carrier_phase_rate > dds_phase_ratef(CARRIER_NOMINAL_FREQ + 20.0f))
+            {
+               span_log(&s->logging, SPAN_LOG_FLOW, "Training failed (sequence failed)\n");
+               /* Park this modem */
+               s->in_training = TRAINING_STAGE_PARKED;
+               s->put_bit(s->user_data, PUTBIT_TRAINING_FAILED);
+               break;
+            }
             /* Make a step shift in the phase, to pull it into line. We need to rotate the equalizer
                buffer, as well as the carrier phase, for this to play out nicely. */
             p = angle*2.0f*3.14159f/(65536.0f*65536.0f);
-            zz = complex_set(cosf(p), -sinf(p));
+            zz = complex_setf(cosf(p), -sinf(p));
             for (i = 0;  i <= V29_EQUALIZER_MASK;  i++)
-                s->eq_buf[i] = complex_mul(&s->eq_buf[i], &zz);
+                s->eq_buf[i] = complex_mulf(&s->eq_buf[i], &zz);
             s->carrier_phase += angle;
-
-            /* QAM and Gardner only play nicely with heavy damping, so we need to change to
-               a slow rate of symbol timing adaption. However, it must not be so slow that it
-               cannot track the worst case timing error specified in V.29. This should be 0.01%,
-               but since we might be off in the opposite direction from the source, the total
-               error could be higher. */
-            s->gardner_step = 2;
             /* We have just seen the first bit of the scrambled sequence, so skip it. */
             bit = scrambled_training_bit(s);
             s->training_count = 1;
@@ -1957,8 +1969,8 @@ static void process_half_baud(v29_rx_state_t *s, complex_t *sample)
         track_carrier(s, &z, target);
         tune_equalizer(s, &z, target);
         /* Measure the training error */
-        zz = complex_sub(&z, target);
-        s->training_error += power(&zz);
+        zz = complex_subf(&z, target);
+        s->training_error += powerf(&zz);
         if (++s->training_count >= V29_TRAINING_SEG_3_LEN)
         {
             span_log(&s->logging, SPAN_LOG_FLOW, "Constellation mismatch %f\n", s->training_error);
@@ -1986,8 +1998,8 @@ static void process_half_baud(v29_rx_state_t *s, complex_t *sample)
         decode_baud(s, &z);
         target = &v29_constellation[s->constellation_state];
         /* Measure the training error */
-        zz = complex_sub(&z, target);
-        s->training_error += power(&zz);
+        zz = complex_subf(&z, target);
+        s->training_error += powerf(&zz);
         if (++s->training_count >= V29_TRAINING_SEG_4_LEN)
         {
             if (s->training_error < 50.0f)
@@ -2025,21 +2037,22 @@ static void process_half_baud(v29_rx_state_t *s, complex_t *sample)
 }
 /*- End of function --------------------------------------------------------*/
 
-int v29_rx(v29_rx_state_t *s, const int16_t *amp, int len)
+int v29_rx(v29_rx_state_t *s, const int16_t amp[], int len)
 {
     int i;
     int j;
     int step;
-    complex_t z;
-    complex_t zz;
-    complex_t sample;
+    complexf_t z;
+    complexf_t zz;
+    complexf_t sample;
     int32_t power;
+    float v;
 
     for (i = 0;  i < len;  i++)
     {
         s->rrc_filter[s->rrc_filter_step] =
-        s->rrc_filter[s->rrc_filter_step + V29RX_FILTER_STEPS] = amp[i];
-        if (++s->rrc_filter_step >= V29RX_FILTER_STEPS)
+        s->rrc_filter[s->rrc_filter_step + V29_RX_FILTER_STEPS] = amp[i];
+        if (++s->rrc_filter_step >= V29_RX_FILTER_STEPS)
             s->rrc_filter_step = 0;
 
         /* There should be no DC in the signal, but sometimes there is.
@@ -2074,11 +2087,32 @@ int v29_rx(v29_rx_state_t *s, const int16_t *amp, int len)
         {
             /* Only spend effort processing this data if the modem is not
                parked, after training failure. */
+            s->eq_put_step -= PULSESHAPER_COEFF_SETS;
+            step = -s->eq_put_step;
+            if (step > PULSESHAPER_COEFF_SETS - 1)
+                step = PULSESHAPER_COEFF_SETS - 1;
+            if (step < 0)
+                step += PULSESHAPER_COEFF_SETS;
+            zz.re = pulseshaper[step][0].re*s->rrc_filter[s->rrc_filter_step];
+            for (j = 1;  j < V29_RX_FILTER_STEPS;  j++)
+                zz.re += pulseshaper[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
+            sample.re = zz.re*s->agc_scaling;
+
+            /* Symbol timing synchronisation band edge filters */
+            /* Low Nyquist band edge filter */
+            v = s->symbol_sync_low[0]*SYNC_LOW_BAND_EDGE_COEFF_0 + s->symbol_sync_low[1]*SYNC_LOW_BAND_EDGE_COEFF_1 + sample.re;
+            s->symbol_sync_low[1] = s->symbol_sync_low[0];
+            s->symbol_sync_low[0] = v;
+            /* High Nyquist band edge filter */
+            v = s->symbol_sync_high[0]*SYNC_HIGH_BAND_EDGE_COEFF_0 + s->symbol_sync_high[1]*SYNC_HIGH_BAND_EDGE_COEFF_1 + sample.re;
+            s->symbol_sync_high[1] = s->symbol_sync_high[0];
+            s->symbol_sync_high[0] = v;
+
             z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
 
-            /* Put things into the equalization buffer at T/2 rate. The Gardner algorithm
+            /* Put things into the equalization buffer at T/2 rate. The symbol synchronisation
                will fiddle the step to align this with the symbols. */
-            if ((s->eq_put_step -= PULSESHAPER_COEFF_SETS) <= 0)
+            if (s->eq_put_step <= 0)
             {
                 if (s->in_training == TRAINING_STAGE_SYMBOL_ACQUISITION)
                 {
@@ -2092,18 +2126,14 @@ int v29_rx(v29_rx_state_t *s, const int16_t *amp, int len)
                 step = -s->eq_put_step;
                 if (step > PULSESHAPER_COEFF_SETS - 1)
                     step = PULSESHAPER_COEFF_SETS - 1;
-                s->eq_put_step += PULSESHAPER_COEFF_SETS*10/(3*2);
-                zz.re = pulseshaper[step][0].re*s->rrc_filter[s->rrc_filter_step];
                 zz.im = pulseshaper[step][0].im*s->rrc_filter[s->rrc_filter_step];
-                for (j = 1;  j < V29RX_FILTER_STEPS;  j++)
-                {
-                    zz.re += pulseshaper[step][j].re*s->rrc_filter[j + s->rrc_filter_step];
+                for (j = 1;  j < V29_RX_FILTER_STEPS;  j++)
                     zz.im += pulseshaper[step][j].im*s->rrc_filter[j + s->rrc_filter_step];
-                }
                 sample.re = zz.re*s->agc_scaling;
                 sample.im = zz.im*s->agc_scaling;
+                s->eq_put_step += PULSESHAPER_COEFF_SETS*10/(3*2);
                 /* Shift to baseband - since this is done in a full complex form, the
-                   result is clean, and requires no further filtering apart from the
+                   result is clean, and requires no further filtering, apart from the
                    equalizer. */
                 zz.re = sample.re*z.re - sample.im*z.im;
                 zz.im = -sample.re*z.im - sample.im*z.re;
@@ -2124,8 +2154,6 @@ void v29_rx_set_put_bit(v29_rx_state_t *s, put_bit_func_t put_bit, void *user_da
 
 int v29_rx_restart(v29_rx_state_t *s, int rate, int old_train)
 {
-    int i;
-
     switch (rate)
     {
     case 9600:
@@ -2142,8 +2170,7 @@ int v29_rx_restart(v29_rx_state_t *s, int rate, int old_train)
     }
     s->bit_rate = rate;
 
-    for (i = 0;  i < sizeof(s->rrc_filter)/sizeof(s->rrc_filter[0]);  i++)
-        s->rrc_filter[i] = 0.0f;
+    vec_zerof(s->rrc_filter, sizeof(s->rrc_filter)/sizeof(s->rrc_filter[0]));
     s->rrc_filter_step = 0;
 
     s->scramble_reg = 0;
@@ -2154,7 +2181,7 @@ int v29_rx_restart(v29_rx_state_t *s, int rate, int old_train)
     s->old_train = old_train;
 
     s->carrier_phase = 0;
-    s->carrier_track_i = 2000.0f;
+    s->carrier_track_i = 8000.0f;
     s->carrier_track_p = 8000000.0f;
 
     power_meter_init(&(s->power), 4);
@@ -2169,18 +2196,25 @@ int v29_rx_restart(v29_rx_state_t *s, int rate, int old_train)
     }
     else
     {
-        s->carrier_phase_rate = dds_phase_ratef(1700.0f);
+        s->carrier_phase_rate = dds_phase_ratef(CARRIER_NOMINAL_FREQ);
         s->agc_scaling = 0.0005f;
         equalizer_reset(s);
     }
     s->eq_skip = 0;
     s->last_sample = 0;
 
-    s->gardner_integrate = 0;
-    s->gardner_total_correction = 0;
-    /* Start with very big Gardner steps, until we start to pull in the symbol timing. */
-    s->gardner_step = 64;
-    s->baud_phase = 0;
+    /* Initialise the working data for symbol timing synchronisation */
+    s->symbol_sync_low[0] = 0.0f;
+    s->symbol_sync_low[1] = 0.0f;
+    s->symbol_sync_high[0] = 0.0f;
+    s->symbol_sync_high[1] = 0.0f;
+    s->symbol_sync_dc_filter[0] = 0.0f;
+    s->symbol_sync_dc_filter[1] = 0.0f;
+    s->baud_phase = 0.0f;
+    s->baud_half = 0;
+
+    s->total_baud_timing_correction = 0;
+
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
@@ -2197,17 +2231,11 @@ v29_rx_state_t *v29_rx_init(v29_rx_state_t *s, int rate, put_bit_func_t put_bit,
     s->put_bit = put_bit;
     s->user_data = user_data;
     /* The V.29 spec says the thresholds should be -31dBm and -26dBm, but that makes little
-       sense. V.17 uses -48dBm and -43dBm, and there seems no goo reason to cut off at a
+       sense. V.17 uses -48dBm and -43dBm, and there seems no good reason to cut off at a
        higher level (though at 9600bps and 7200bps, TCM should put V.17 sensitivity several
        dB ahead of V.29). */
-    /* The 0.4 factor allows for the gain of the DC blocker */
-#if 0
-    s->carrier_on_power = power_meter_level_dbm0(-26)*0.4;
-    s->carrier_off_power = power_meter_level_dbm0(-31)*0.4;
-#else
-    s->carrier_on_power = power_meter_level_dbm0(-43)*0.4;
-    s->carrier_off_power = power_meter_level_dbm0(-48)*0.4;
-#endif
+    /* The thresholds should be on at -26dBm0 and off at -31dBm0 */
+    v29_rx_signal_cutoff(s, -28.5f);
     span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
     span_log_set_protocol(&s->logging, "V.29");
 
