@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v42.c,v 1.25 2006/05/19 12:23:32 steveu Exp $
+ * $Id: v42.c,v 1.29 2006/10/24 13:45:27 steveu Exp $
  */
 
 /* THIS IS A WORK IN PROGRESS. IT IS NOT FINISHED. */
@@ -82,8 +81,8 @@
 #define LAPM_DLCI_DTE_TO_DTE        0
 #define LAPM_DLCI_LAYER2_MANAGEMENT 63
 
-static void t401_expired(sp_sched_state_t *s, void *user_data);
-static void t403_expired(sp_sched_state_t *s, void *user_data);
+static void t401_expired(span_sched_state_t *s, void *user_data);
+static void t403_expired(span_sched_state_t *s, void *user_data);
 
 void lapm_reset(lapm_state_t *s);
 void lapm_restart(lapm_state_t *s);
@@ -109,7 +108,7 @@ static int lapm_tx_frame(lapm_state_t *s, uint8_t *frame, int len)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void t400_expired(sp_sched_state_t *ss, void *user_data)
+static void t400_expired(span_sched_state_t *ss, void *user_data)
 {
     v42_state_t *s;
     
@@ -135,7 +134,7 @@ static void lapm_send_ua(lapm_state_t *s, int pfbit)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void lapm_send_sabme(sp_sched_state_t *ss, void *user_data)
+static void lapm_send_sabme(span_sched_state_t *ss, void *user_data)
 {
     lapm_state_t *s;
     uint8_t frame[3];
@@ -144,7 +143,7 @@ static void lapm_send_sabme(sp_sched_state_t *ss, void *user_data)
     if (s->t401_timer >= 0)
     {
 fprintf(stderr, "Deleting T401 q [%p]\n", s);
-        sp_schedule_del(&s->sched, s->t401_timer);
+        span_schedule_del(&s->sched, s->t401_timer);
         s->t401_timer = -1;
     }
     /*endif*/
@@ -159,7 +158,7 @@ fprintf(stderr, "Deleting T401 q [%p]\n", s);
     }
     /*endif*/
 fprintf(stderr, "Setting T401 a [%p]\n", s);
-    s->t401_timer = sp_schedule_event(&s->sched, T_401, lapm_send_sabme, s);
+    s->t401_timer = span_schedule_event(&s->sched, T_401, lapm_send_sabme, s);
     lapm_init_header(frame, s->we_are_originator);
     frame[1] = 0x7F;
     frame[2] = 0;
@@ -239,7 +238,7 @@ fprintf(stderr, "T401 a is %d [%p]\n", s->t401_timer, s);
         if (s->t401_timer >= 0)
         {
 fprintf(stderr, "Deleting T401 a [%p]\n", s);
-            sp_schedule_del(&s->sched, s->t401_timer);
+            span_schedule_del(&s->sched, s->t401_timer);
             s->t401_timer = -1;
         }
         /*endif*/
@@ -251,7 +250,7 @@ fprintf(stderr, "Deleting T401 a [%p]\n", s);
         if (s->t403_timer >= 0)
         {
 fprintf(stderr, "Deleting T403 b\n");
-            sp_schedule_del(&s->sched, s->t403_timer);
+            span_schedule_del(&s->sched, s->t403_timer);
             s->t403_timer = -1;
         }
         /*endif*/
@@ -267,7 +266,7 @@ fprintf(stderr, "Deleting T403 b\n");
         if (s->t401_timer < 0)
         {
 fprintf(stderr, "Setting T401 b [%p]\n", s);
-            s->t401_timer = sp_schedule_event(&s->sched, T_401, t401_expired, s);
+            s->t401_timer = span_schedule_event(&s->sched, T_401, t401_expired, s);
         }
         /*endif*/
     }
@@ -276,7 +275,7 @@ fprintf(stderr, "Setting T401 b [%p]\n", s);
         span_log(&s->logging, SPAN_LOG_FLOW, "-- Nothing left, starting timer T_403\n");
         /* Nothing to transmit. Start timer T_403. */
 fprintf(stderr, "Setting T403 c\n");
-        s->t403_timer = sp_schedule_event(&s->sched, T_403, t403_expired, s);
+        s->t403_timer = span_schedule_event(&s->sched, T_403, t403_expired, s);
     }
     /*endif*/
 }
@@ -309,7 +308,7 @@ static void lapm_rr(lapm_state_t *s, int pfbit)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void t401_expired(sp_sched_state_t *ss, void *user_data)
+static void t401_expired(span_sched_state_t *ss, void *user_data)
 {
     lapm_state_t *s;
     
@@ -331,7 +330,7 @@ fprintf(stderr, "Expiring T401 a [%p]\n", s);
             lapm_tx_frame(s, s->txqueue->frame, s->txqueue->len);
             span_log(&s->logging, SPAN_LOG_FLOW, "-- Scheduling retransmission (%d)\n", s->retransmissions);
 fprintf(stderr, "Setting T401 d [%p]\n", s);
-            s->t401_timer = sp_schedule_event(&s->sched, T_401, t401_expired, s);
+            s->t401_timer = span_schedule_event(&s->sched, T_401, t401_expired, s);
         }
         else
         {
@@ -444,14 +443,14 @@ int lapm_tx_iframe(lapm_state_t *s, const void *buf, int len, int cr)
     {
         span_log(&s->logging, SPAN_LOG_FLOW, "Stopping T_403 timer\n");
 fprintf(stderr, "Deleting T403 c\n");
-        sp_schedule_del(&s->sched, s->t403_timer);
+        span_schedule_del(&s->sched, s->t403_timer);
         s->t403_timer = -1;
     }
     /*endif*/
     if (s->t401_timer < 0)
     {
         span_log(&s->logging, SPAN_LOG_FLOW, "Starting timer T_401\n");
-        s->t401_timer = sp_schedule_event(&s->sched, T_401, t401_expired, s);
+        s->t401_timer = span_schedule_event(&s->sched, T_401, t401_expired, s);
 fprintf(stderr, "Setting T401 e %d [%p]\n", s->t401_timer, s);
     }
     else
@@ -463,7 +462,7 @@ fprintf(stderr, "Setting T401 e %d [%p]\n", s->t401_timer, s);
 }
 /*- End of function --------------------------------------------------------*/
 
-static void t403_expired(sp_sched_state_t *ss, void *user_data)
+static void t403_expired(span_sched_state_t *ss, void *user_data)
 {
     lapm_state_t *s;
 
@@ -476,7 +475,7 @@ static void t403_expired(sp_sched_state_t *ss, void *user_data)
     lapm_rr(s, 1);
     /* Restart ourselves */
 fprintf(stderr, "Setting T403 f\n");
-    s->t401_timer = sp_schedule_event(&s->sched, T_401, t401_expired, s);
+    s->t401_timer = span_schedule_event(&s->sched, T_401, t401_expired, s);
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -647,13 +646,13 @@ static void lapm_link_up(lapm_state_t *s)
     if (s->t401_timer >= 0)
     {
 fprintf(stderr, "Deleting T401 x [%p]\n", s);
-        sp_schedule_del(&s->sched, s->t401_timer);
+        span_schedule_del(&s->sched, s->t401_timer);
         s->t401_timer = -1;
     }
     /*endif*/
     /* Start the T403 timer */
 fprintf(stderr, "Setting T403 g\n");
-    s->t403_timer = sp_schedule_event(&s->sched, T_403, t403_expired, s);
+    s->t403_timer = span_schedule_event(&s->sched, T_403, t403_expired, s);
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -682,14 +681,14 @@ void lapm_reset(lapm_state_t *s)
     if (s->t401_timer >= 0)
     {
 fprintf(stderr, "Deleting T401 d [%p]\n", s);
-        sp_schedule_del(&s->sched, s->t401_timer);
+        span_schedule_del(&s->sched, s->t401_timer);
         s->t401_timer = -1;
     }
     /*endif*/
     if (s->t403_timer >= 0)
     {
 fprintf(stderr, "Deleting T403 e\n");
-        sp_schedule_del(&s->sched, s->t403_timer);
+        span_schedule_del(&s->sched, s->t403_timer);
         s->t403_timer = -1;
     }
     /*endif*/
@@ -915,7 +914,7 @@ fprintf(stderr, "LAPM receive %d %d\n", ok, len);
                     if (s->t401_timer >= 0)
                     {
 fprintf(stderr, "Deleting T401 f [%p]\n", s);
-                        sp_schedule_del(&s->sched, s->t401_timer);
+                        span_schedule_del(&s->sched, s->t401_timer);
                         s->t401_timer = -1;
                     }
                     /*endif*/
@@ -923,12 +922,12 @@ fprintf(stderr, "Deleting T401 f [%p]\n", s);
                     if (s->t403_timer >= 0)
                     {
 fprintf(stderr, "Deleting T403 g\n");
-                        sp_schedule_del(&s->sched, s->t403_timer);
+                        span_schedule_del(&s->sched, s->t403_timer);
                         s->t403_timer = -1;
                     }
                     /*endif*/
 fprintf(stderr, "Setting T403 h\n");
-                    s->t403_timer = sp_schedule_event(&s->sched, T_403, t403_expired, s);
+                    s->t403_timer = span_schedule_event(&s->sched, T_403, t403_expired, s);
                 }
                 /*endif*/
             }
@@ -1104,7 +1103,7 @@ void lapm_restart(lapm_state_t *s)
 #endif
     span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
     span_log_set_protocol(&s->logging, "LAPM");
-    hdlc_tx_init(&s->hdlc_tx, FALSE, lapm_hdlc_underflow, s);
+    hdlc_tx_init(&s->hdlc_tx, FALSE, 1, TRUE, lapm_hdlc_underflow, s);
     hdlc_rx_init(&s->hdlc_rx, FALSE, FALSE, 1, lapm_receive, s);
     /* TODO: This is a bodge! */
 fprintf(stderr, "LAPM restart T401 a [%p]\n", s);
@@ -1246,7 +1245,7 @@ static void negotiation_rx_bit(v42_state_t *s, int new_bit)
                     if (s->t400_timer >= 0)
                     {
 fprintf(stderr, "Deleting T400 h\n");
-                        sp_schedule_del(&s->lapm.sched, s->t400_timer);
+                        span_schedule_del(&s->lapm.sched, s->t400_timer);
                         s->t400_timer = -1;
                     }
                     /*endif*/
@@ -1314,7 +1313,7 @@ static int v42_support_negotiation_tx_bit(v42_state_t *s)
                     if (s->t400_timer >= 0)
                     {
 fprintf(stderr, "Deleting T400 i\n");
-                        sp_schedule_del(&s->lapm.sched, s->t400_timer);
+                        span_schedule_del(&s->lapm.sched, s->t400_timer);
                         s->t400_timer = -1;
                     }
                     /*endif*/
@@ -1359,7 +1358,7 @@ void v42_rx_bit(void *user_data, int bit)
     if (s->lapm.state == LAPM_DETECT)
         negotiation_rx_bit(s, bit);
     else
-        hdlc_rx_bit(&s->lapm.hdlc_rx, bit);
+        hdlc_rx_put_bit(&s->lapm.hdlc_rx, bit);
     /*endif*/
 }
 /*- End of function --------------------------------------------------------*/
@@ -1373,7 +1372,7 @@ int v42_tx_bit(void *user_data)
     if (s->lapm.state == LAPM_DETECT)
         bit = v42_support_negotiation_tx_bit(s);
     else
-        bit = hdlc_tx_getbit(&s->lapm.hdlc_tx);
+        bit = hdlc_tx_get_bit(&s->lapm.hdlc_tx);
     /*endif*/
     return bit;
 }
@@ -1388,7 +1387,7 @@ void v42_set_status_callback(v42_state_t *s, v42_status_func_t callback, void *u
 
 void v42_restart(v42_state_t *s)
 {
-    sp_schedule_init(&s->lapm.sched);
+    span_schedule_init(&s->lapm.sched);
 
     s->lapm.we_are_originator = s->caller;
     lapm_restart(&s->lapm);
@@ -1403,7 +1402,7 @@ void v42_restart(v42_state_t *s)
         s->rx_negotiation_step = 0;
         s->odp_seen = FALSE;
 fprintf(stderr, "Setting T400 i\n");
-        s->t400_timer = sp_schedule_event(&s->lapm.sched, T_400, t400_expired, s);
+        s->t400_timer = span_schedule_event(&s->lapm.sched, T_400, t400_expired, s);
         s->lapm.state = LAPM_DETECT;
     }
     else
