@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v42bis.c,v 1.21 2006/05/08 13:22:57 steveu Exp $
+ * $Id: v42bis.c,v 1.26 2006/11/03 13:37:21 steveu Exp $
  */
 
 /* THIS IS A WORK IN PROGRESS. IT IS NOT FINISHED. 
@@ -98,8 +97,6 @@ static __inline__ void push_compressed_code(v42bis_compress_state_t *ss, int cod
 
 int v42bis_compress(v42bis_state_t *s, const uint8_t *buf, int len)
 {
-    int index;
-    int offset;
     int ptr;
     int i;
     uint32_t octet;
@@ -128,24 +125,29 @@ int v42bis_compress(v42bis_state_t *s, const uint8_t *buf, int len)
     while (ptr < len)
     {
         octet = buf[ptr++];
-        /* Hash to find a match */
-        index = (octet << (V42BIS_MAX_BITS - V42BIS_N3)) ^ ss->string_code;
-        offset = (index == 0)  ?  1  :  V42BIS_TABLE_SIZE - index;
-        for (;;)
+        if ((ss->dict[ss->string_code].children[octet >> 5] & (1 << (octet & 0x1F))))
         {
-            if (ss->code[index] == 0xFFFF)
-                break;
-            code = ss->code[index];
-            if (ss->prior_code[code] == ss->string_code  &&  ss->node_octet[code] == octet)
-                break;
-            if ((index -= offset) < 0)
-                index += V42BIS_TABLE_SIZE;
+            /* The leaf exists. Now find it in the table. */
+            /* TODO: This is a brute force scan for a match. We need something better. */
+            for (code = 0;  code < s->v42bis_parm_n2;  code++)
+            {
+                if (ss->dict[code].parent_code == ss->string_code  &&  ss->dict[code].node_octet == octet)
+                    break;
+            }
         }
-        ss->compressibility_filter += (((8 << 20) - ss->compressibility_filter) >> 10);
-        if (ss->code[index] != 0xFFFF)
+        else
+        {
+            /* The leaf does not exist. */
+            code = s->v42bis_parm_n2;
+        }
+        /* 6.3(b) If the string matches a dictionary entry, and the entry is not that entry
+                  created by the last invocation of the string matching procedure, then the
+                  next character shall be read and appended to the string and this step
+                  repeated. */
+        if (code < s->v42bis_parm_n2  &&  code != ss->latest_code)
         {
             /* The string was found */
-            ss->string_code = ss->code[index];
+            ss->string_code = code;
             ss->string_length++;
         }
         else
@@ -171,16 +173,17 @@ int v42bis_compress(v42bis_state_t *s, const uint8_t *buf, int len)
             /* 7.6    Dictionary updating */
             /* 6.4    Add the string to the dictionary */
             /* 6.4(b) The string is not in the table. */
-            if (ss->string_length < s->v42bis_parm_n7)
+            if (code != ss->latest_code  &&  ss->string_length < s->v42bis_parm_n7)
             {
+                ss->latest_code = ss->v42bis_parm_c1;
                 /* 6.4(a) The length of the string is in range for adding to the dictionary */
-                ss->code[index] = (uint16_t) ss->v42bis_parm_c1;
                 /* If the last code was a leaf, it no longer is */
-                ss->leaves[ss->string_code]++;
+                ss->dict[ss->string_code].leaves++;
+                ss->dict[ss->string_code].children[octet >> 5] |= (1 << (octet & 0x1F));
                 /* The new one is definitely a leaf */
-                ss->prior_code[ss->v42bis_parm_c1] = (uint8_t) ss->string_code;
-                ss->leaves[ss->v42bis_parm_c1] = 0;
-                ss->node_octet[ss->v42bis_parm_c1] = (uint8_t) octet;
+                ss->dict[ss->v42bis_parm_c1].parent_code = (uint16_t) ss->string_code;
+                ss->dict[ss->v42bis_parm_c1].leaves = 0;
+                ss->dict[ss->v42bis_parm_c1].node_octet = (uint8_t) octet;
                 /* 7.7    Node recovery */
                 /* 6.5    Recovering a dictionary entry to use next */
                 for (;;)
@@ -189,38 +192,36 @@ int v42bis_compress(v42bis_state_t *s, const uint8_t *buf, int len)
                     if ((int) (++ss->v42bis_parm_c1) >= s->v42bis_parm_n2)
                         ss->v42bis_parm_c1 = V42BIS_N5;
                     /* 6.5(c) We need to reuse a leaf node */
-                    if (ss->leaves[ss->v42bis_parm_c1])
+                    if (ss->dict[ss->v42bis_parm_c1].leaves)
                         continue;
-                    if (ss->prior_code[ss->v42bis_parm_c1] == 0xFFFF)
+                    if (ss->dict[ss->v42bis_parm_c1].parent_code == 0xFFFF)
                         break;
                     /* 6.5(d) Detach the leaf node from its parent, and re-use it */
-                    /* Clear the entry from the hash table */
-                    index = (ss->node_octet[ss->v42bis_parm_c1] << (V42BIS_MAX_BITS - V42BIS_N3)) ^ ss->prior_code[ss->v42bis_parm_c1];
-                    offset = (index == 0)  ?  1  :  V42BIS_TABLE_SIZE - index;
-                    for (;;)
-                    {
-                        if (ss->code[index] == ss->v42bis_parm_c1)
-                        {
-                            ss->code[index] = 0xFFFF;
-                            break;
-                        }
-                        if ((index -= offset) < 0)
-                            index += V42BIS_TABLE_SIZE;
-                    }
                     /* Possibly make the parent a leaf node again */
-                    ss->leaves[ss->prior_code[ss->v42bis_parm_c1]]--;
-                    ss->prior_code[ss->v42bis_parm_c1] = 0xFFFF;
+                    ss->dict[ss->dict[ss->v42bis_parm_c1].parent_code].leaves--;
+                    ss->dict[ss->dict[ss->v42bis_parm_c1].parent_code].children[ss->dict[ss->v42bis_parm_c1].node_octet >> 5] &= ~(1 << (ss->dict[ss->v42bis_parm_c1].node_octet & 0x1F));
+                    ss->dict[ss->v42bis_parm_c1].parent_code = 0xFFFF;
                     break;
                 }
             }
+            else
+            {
+                ss->latest_code = 0xFFFFFFFF;
+            }
             /* 7.8 Data compressibility test */
-            ss->compressibility_filter += (((-ss->v42bis_parm_c2 << 20) - ss->compressibility_filter) >> 10);
-#if 0
+            /* Filter on the balance of what went into the compressor, and what came out */
+            ss->compressibility_filter += ((((8*ss->string_length - ss->v42bis_parm_c2) << 20) - ss->compressibility_filter) >> 10);
             if (ss->transparent)
             {
+                if (ss->compressibility_filter > 0)
+                    ss->compressibility_persistence++;
+                else
+                    ss->compressibility_persistence = 0;
                 /* 7.8.1 Transition to compressed mode - TODO */
-                if (ss->compressibility_filter > 0x1000)
+                if (ss->compressibility_persistence > 1000)
                 {
+//printf("Compress\n");
+#if 0
                     /* Switch out of transparent now, between codes. We need to send the octet which did not
                        match, just before switching. */
                     if (octet == ss->escape_code)
@@ -235,21 +236,29 @@ int v42bis_compress(v42bis_state_t *s, const uint8_t *buf, int len)
                     push_compressed_octet(ss, ss->escape_code++);
                     push_compressed_octet(ss, V42BIS_ECM);
                     ss->transparent = FALSE;
+#endif
+                    ss->compressibility_persistence = 0;
                 }
             }
             else
             {
-                /* 7.8.2 Transition to transparent mode - TODO */
                 if (ss->compressibility_filter < 0)
+                    ss->compressibility_persistence++;
+                else
+                    ss->compressibility_persistence = 0;
+                /* 7.8.2 Transition to transparent mode - TODO */
+                if (ss->compressibility_persistence > 1000)
                 {
+//printf("Transparent\n");
+#if 0
                     /* Switch into transparent now, between codes, and the unmatched octet should
                        go out in transparent mode, just below */
                     push_compressed_code(ss, V42BIS_ETM);
                     ss->transparent = TRUE;
+#endif
+                    ss->compressibility_persistence = 0;
                 }
             }
-printf("Compress %x\n", ss->compressibility_filter);
-#endif
             /* 7.8.3 Reset function - TODO */
             ss->string_code = octet + V42BIS_N6;
             ss->string_length = 1;
@@ -307,9 +316,9 @@ int v42bis_compress_dump(v42bis_state_t *s)
     
     for (i = 0;  i < V42BIS_MAX_CODEWORDS;  i++)
     {
-        if (s->compress.prior_code[i] != 0xFFFF)
+        if (s->compress.dict[i].parent_code != 0xFFFF)
         {
-            printf("Entry %4x, prior %4x, leaves %d, octet %2x\n", i, s->compress.prior_code[i], s->compress.leaves[i], s->compress.node_octet[i]);
+            printf("Entry %4x, prior %4x, leaves %d, octet %2x\n", i, s->compress.dict[i].parent_code, s->compress.dict[i].leaves, s->compress.dict[i].node_octet);
         }
     }
     return 0;
@@ -326,6 +335,7 @@ int v42bis_decompress(v42bis_state_t *s, const uint8_t *buf, int len)
     uint32_t code;
     uint32_t new_code;
     v42bis_decompress_state_t *ss;
+    uint8_t decode_buf[V42BIS_MAX_STRING_SIZE];
 
     ss = &s->decompress;
     if ((s->v42bis_parm_p0 & 1) == 0)
@@ -425,38 +435,23 @@ int v42bis_decompress(v42bis_state_t *s, const uint8_t *buf, int len)
             continue;
         }
         /* Start at the end of the buffer, and decode backwards */
-        string = &ss->decode_buf[V42BIS_MAX_STRING_SIZE - 1];
-        if ((int) new_code == ss->v42bis_parm_c1)
-        {
-            /*
-             * This code deals with special situations like "XXXXXXX". The first X goes out
-             * as 0x5B. A code is then added to the encoder's dictionary for "XX". Immediately,
-             * the second and third X's match this code, even though it has not yet gone into
-             * the decoder's directory. We handle this by decoding the last code, and adding a
-             * single character to the end of the decoded string.
-             */
-            *string-- = (uint8_t) ss->octet;
-            code = ss->old_code;
-        }
-        else
-        {
-            /* Check the received code is valid. It can't be too big, as we pulled only the expected number
-               of bits from the input stream. It could, however, be unknown. */
-            if (ss->prior_code[new_code] == 0xFFFF)
-                return -1;
-            /* Otherwise we do a straight decode of the new code. */
-            code = new_code;
-        }
+        string = &decode_buf[V42BIS_MAX_STRING_SIZE - 1];
+        /* Check the received code is valid. It can't be too big, as we pulled only the expected number
+           of bits from the input stream. It could, however, be unknown. */
+        if (ss->dict[new_code].parent_code == 0xFFFF)
+            return -1;
+        /* Otherwise we do a straight decode of the new code. */
+        code = new_code;
         /* Trace back through the octets which form the string, and output them. */
         while (code >= V42BIS_N5)
         {
-            *string-- = ss->node_octet[code];
-            code = ss->prior_code[code];
+            *string-- = ss->dict[code].node_octet;
+            code = ss->dict[code].parent_code;
         }
         *string = (uint8_t) (code - V42BIS_N6);
         ss->octet = code - V42BIS_N6;
         /* Output the decoded string. */
-        this_length = V42BIS_MAX_STRING_SIZE - (int) (string - ss->decode_buf);
+        this_length = V42BIS_MAX_STRING_SIZE - (int) (string - decode_buf);
         memcpy(ss->output_buf + ss->output_octet_count, string, this_length);
         ss->output_octet_count += this_length;
         if (ss->output_octet_count >= ss->max_len - s->v42bis_parm_n7)
@@ -465,32 +460,42 @@ int v42bis_decompress(v42bis_state_t *s, const uint8_t *buf, int len)
             ss->output_octet_count = 0;
         }
         /* 6.4 Add the string to the dictionary */
-        /* 6.4(b) The string is not in the table. */
         if (ss->last_length < s->v42bis_parm_n7)
         {
-            /* 6.4(a) The length of the string is in range for adding to the dictionary */
-            ss->leaves[ss->old_code]++;
-            /* The new one is definitely a leaf */
-            ss->prior_code[ss->v42bis_parm_c1] = (uint8_t) ss->old_code;
-            ss->leaves[ss->v42bis_parm_c1] = 0;
-            ss->node_octet[ss->v42bis_parm_c1] = (uint8_t) ss->octet;
-            /* 6.5 Recovering a dictionary entry to use next */
-            for (;;)
+            /* 6.4(a) The string does not exceed N7 in length */
+            if (ss->last_old_code != ss->old_code
+                ||
+                ss->last_extra_octet != *string)
             {
-                /* 6.5(a) and (b) */
-                if (++ss->v42bis_parm_c1 >= s->v42bis_parm_n2)
-                    ss->v42bis_parm_c1 = V42BIS_N5;
-                /* 6.5(c) We need to reuse a leaf node */
-                if (ss->leaves[ss->v42bis_parm_c1])
-                    continue;
-                /* 6.5(d) This is a leaf node, so re-use it */
-                /* Possibly make the parent a leaf node again */
-                if (ss->prior_code[ss->v42bis_parm_c1] != 0xFFFF)
-                    ss->leaves[ss->prior_code[ss->v42bis_parm_c1]]--;
-                ss->prior_code[ss->v42bis_parm_c1] = 0xFFFF;
-                break;
+                /* 6.4(b) The string is not in the table. */
+                ss->dict[ss->old_code].leaves++;
+                /* The new one is definitely a leaf */
+                ss->dict[ss->v42bis_parm_c1].parent_code = (uint16_t) ss->old_code;
+                ss->dict[ss->v42bis_parm_c1].leaves = 0;
+                ss->dict[ss->v42bis_parm_c1].node_octet = (uint8_t) ss->octet;
+                /* 6.5 Recovering a dictionary entry to use next */
+                for (;;)
+                {
+                    /* 6.5(a) and (b) */
+                    if (++ss->v42bis_parm_c1 >= s->v42bis_parm_n2)
+                        ss->v42bis_parm_c1 = V42BIS_N5;
+                    /* 6.5(c) We need to reuse a leaf node */
+                    if (ss->dict[ss->v42bis_parm_c1].leaves)
+                        continue;
+                    /* 6.5(d) This is a leaf node, so re-use it */
+                    /* Possibly make the parent a leaf node again */
+                    if (ss->dict[ss->v42bis_parm_c1].parent_code != 0xFFFF)
+                        ss->dict[ss->dict[ss->v42bis_parm_c1].parent_code].leaves--;
+                    ss->dict[ss->v42bis_parm_c1].parent_code = 0xFFFF;
+                    break;
+                }
             }
         }
+        /* Record the addition to the dictionary, so we can check for repeat attempts
+           at the next code - see II.4.3 */
+        ss->last_old_code = ss->old_code;
+        ss->last_extra_octet = *string;
+
         ss->old_code = new_code;
         ss->last_length = this_length;
     }
@@ -520,9 +525,9 @@ int v42bis_decompress_dump(v42bis_state_t *s)
     
     for (i = 0;  i < V42BIS_MAX_CODEWORDS;  i++)
     {
-        if (s->decompress.prior_code[i] != 0xFFFF)
+        if (s->decompress.dict[i].parent_code != 0xFFFF)
         {
-            printf("Entry %4x, prior %4x, leaves %d, octet %2x\n", i, s->decompress.prior_code[i], s->decompress.leaves[i], s->decompress.node_octet[i]);
+            printf("Entry %4x, prior %4x, leaves %d, octet %2x\n", i, s->decompress.dict[i].parent_code, s->decompress.dict[i].leaves, s->decompress.dict[i].node_octet);
         }
     }
     return 0;
@@ -580,19 +585,23 @@ v42bis_state_t *v42bis_init(v42bis_state_t *s,
 
     s->compress.first =
     s->decompress.first = TRUE;
-    for (i = 0;  i < V42BIS_TABLE_SIZE;  i++)
-        s->compress.code[i] = 0xFFFF;
     for (i = 0;  i < V42BIS_MAX_CODEWORDS;  i++)
     {
-        s->compress.prior_code[i] =
-        s->decompress.prior_code[i] = 0xFFFF;
-        s->compress.leaves[i] =
-        s->decompress.leaves[i] = 0;
+        s->compress.dict[i].parent_code =
+        s->decompress.dict[i].parent_code = 0xFFFF;
+        s->compress.dict[i].leaves =
+        s->decompress.dict[i].leaves = 0;
     }
     /* Point the root nodes for decompression to themselves. It doesn't matter much what
        they are set to, as long as they are considered "known" codes. */
     for (i = 0;  i < V42BIS_N5;  i++)
-        s->decompress.prior_code[i] = (uint16_t) i;
+        s->decompress.dict[i].parent_code = (uint16_t) i;
+    s->compress.string_code = 0xFFFFFFFF;
+    s->compress.latest_code = 0xFFFFFFFF;
+    
+    s->decompress.last_old_code = 0xFFFFFFFF;
+    s->decompress.last_extra_octet = -1;
+
     return s;
 }
 /*- End of function --------------------------------------------------------*/
