@@ -1,7 +1,7 @@
 /*
  * SpanDSP - a series of DSP components for telephony
  *
- * v8.h - V.8 modem negotiation processing.
+ * v8.c - V.8 modem negotiation processing.
  *
  * Written by Steve Underwood <steveu@coppice.org>
  *
@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v8.c,v 1.14 2006/04/04 15:59:22 steveu Exp $
+ * $Id: v8.c,v 1.18 2006/10/24 13:22:01 steveu Exp $
  */
  
 /*! \file */
@@ -42,8 +41,11 @@
 #include "spandsp/logging.h"
 #include "spandsp/queue.h"
 #include "spandsp/async.h"
-#include "spandsp/biquad.h"
-#include "spandsp/ec_disable_tone.h"
+#include "spandsp/dds.h"
+#include "spandsp/tone_detect.h"
+#include "spandsp/tone_generate.h"
+#include "spandsp/super_tone_rx.h"
+#include "spandsp/modem_connect_tones.h"
 #include "spandsp/power_meter.h"
 #include "spandsp/fsk.h"
 #include "spandsp/v8.h"
@@ -481,9 +483,9 @@ static void put_bit(void *user_data, int bit)
 static void v8_decode_init(v8_state_t *s)
 {
     if (s->caller)
-        fsk_rx_init(&s->v21rx, &preset_fsk_specs[FSK_V21CH2], FALSE, put_bit, s);
+        fsk_rx_init(&s->v21rx, &preset_fsk_specs[FSK_V21CH2], FALSE, 0, put_bit, s);
     else
-        fsk_rx_init(&s->v21rx, &preset_fsk_specs[FSK_V21CH1], FALSE, put_bit, s);
+        fsk_rx_init(&s->v21rx, &preset_fsk_specs[FSK_V21CH1], FALSE, 0, put_bit, s);
     s->preamble_type = 0;
     s->bit_stream = 0;
     s->cm_jm_count = 0;
@@ -613,7 +615,7 @@ int v8_tx(v8_state_t *s, int16_t *amp, int max_len)
         break;
     case V8_CM_WAIT:
         /* Send the ANSam tone */
-        len = echo_can_disable_tone_tx(&s->v8_tx, amp, max_len);
+        len = modem_connect_tones_tx(&s->ec_dis_tx, amp, max_len);
         break;
     }
     return len;
@@ -641,11 +643,11 @@ int v8_rx(v8_state_t *s, const int16_t *amp, int len)
             break;
         s->state = V8_CI;
         s->ci_count = 0;
-        echo_can_disable_tone_rx_init(&s->v8_rx);
+        modem_connect_tones_rx_init(&s->ec_dis_rx, MODEM_CONNECT_TONES_EC_DISABLE, NULL, NULL);
         fsk_tx_init(&s->v21tx, &preset_fsk_specs[FSK_V21CH1], get_bit, s);
         /* Fall through to the next state */
     case V8_CI:
-        residual_samples = echo_can_disable_tone_rx(&s->v8_rx, amp, len);
+        residual_samples = modem_connect_tones_rx(&s->ec_dis_rx, amp, len);
         /* Send 4 CI packets in a burst (the spec says at least 3) */
         for (i = 0;  i < 4;  i++)
         {
@@ -656,7 +658,7 @@ int v8_rx(v8_state_t *s, const int16_t *amp, int len)
         s->state = V8_CI_ON;
         break;
     case V8_CI_ON:
-        residual_samples = echo_can_disable_tone_rx(&s->v8_rx, amp, len);
+        residual_samples = modem_connect_tones_rx(&s->ec_dis_rx, amp, len);
         if (queue_empty(&s->tx_queue))
         {
             s->state = V8_CI_OFF;
@@ -664,9 +666,9 @@ int v8_rx(v8_state_t *s, const int16_t *amp, int len)
         }
         break;
     case V8_CI_OFF:
-        residual_samples = echo_can_disable_tone_rx(&s->v8_rx, amp, len);
+        residual_samples = modem_connect_tones_rx(&s->ec_dis_rx, amp, len);
         /* Check if an ANSam tone has been detected */
-        if (s->v8_rx.hit)
+        if (modem_connect_tones_rx_get(&s->ec_dis_rx))
         {
             /* Set the Te interval. The spec. says 500ms is the minimum,
                but gives reasons why 1 second is a better value. */
@@ -759,7 +761,7 @@ int v8_rx(v8_state_t *s, const int16_t *amp, int len)
         if ((s->negotiation_timer -= len) <= 0)
         {
             /* Send the ANSam tone */
-            echo_can_disable_tone_tx_init(&s->v8_tx, TRUE);
+            modem_connect_tones_tx_init(&s->ec_dis_tx, MODEM_CONNECT_TONES_EC_DISABLE_MOD);
                 
             v8_decode_init(s);
             s->state = V8_CM_WAIT;
