@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: adsi_tests.c,v 1.22 2006/03/27 13:35:14 steveu Exp $
+ * $Id: adsi_tests.c,v 1.27 2006/10/24 13:45:28 steveu Exp $
  */
 
 /*! \page adsi_tests_page ADSI tests
@@ -520,7 +519,9 @@ int main(int argc, char *argv[])
     int i;
     int j;
     int push;
+    int log_audio;
 
+    log_audio = FALSE;
     decode_test_file = NULL;
     current_standard = ADSI_STANDARD_CLASS;
     for (i = 1;  i < argc;  i++)
@@ -531,7 +532,7 @@ int main(int argc, char *argv[])
             decode_test_file = argv[i];
             continue;
         }
-        else if (strcmp(argv[i], "-s") == 0)
+        if (strcmp(argv[i], "-s") == 0)
         {
             i++;
             if (strcasecmp("CLASS", argv[i]) == 0)
@@ -550,8 +551,15 @@ int main(int argc, char *argv[])
                 current_standard = atoi(argv[i]);
             continue;
         }
+        if (strcmp(argv[i], "-l") == 0)
+        {
+            log_audio = TRUE;
+            continue;
+        }
     }
-
+    filesetup = AF_NULL_FILESETUP;
+    outhandle = AF_NULL_FILEHANDLE;
+    
 #if 0
     /* This part tests internal static routines in the ADSI module. It can
        only be run with a modified version of the ADSI module, which makes
@@ -580,16 +588,15 @@ int main(int argc, char *argv[])
     if (decode_test_file)
     {
         /* We will decode the audio from a wave file. */
-        inhandle = afOpenFile(decode_test_file, "r", NULL);
-        if (inhandle == AF_NULL_FILEHANDLE)
+        if ((inhandle = afOpenFile(decode_test_file, "r", NULL)) == AF_NULL_FILEHANDLE)
         {
             fprintf(stderr, "    Cannot open wave file '%s'\n", decode_test_file);
             exit(2);
         }
 
         adsi_rx_init(&rx_adsi, current_standard, put_adsi_msg, NULL);
-        rx_adsi.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
-        rx_adsi.logging.tag = "ADSI";
+        span_log_set_level(&rx_adsi.logging, SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW);
+        span_log_set_tag(&rx_adsi.logging, "ADSI");
         for (;;)
         {
             len = afReadFrames(inhandle,
@@ -608,24 +615,24 @@ int main(int argc, char *argv[])
     }
     else
     {
-        filesetup = afNewFileSetup();
-        if (filesetup == AF_NULL_FILESETUP)
+        if (log_audio)
         {
-            fprintf(stderr, "    Failed to create file setup\n");
-            exit(2);
+            if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
+            {
+                fprintf(stderr, "    Failed to create file setup\n");
+                exit(2);
+            }
+            afInitSampleFormat(filesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
+            afInitRate(filesetup, AF_DEFAULT_TRACK, (float) SAMPLE_RATE);
+            afInitFileFormat(filesetup, AF_FILE_WAVE);
+            afInitChannels(filesetup, AF_DEFAULT_TRACK, 1);
+        
+            if ((outhandle = afOpenFile(OUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
+            {
+                fprintf(stderr, "    Cannot create wave file '%s'\n", OUT_FILE_NAME);
+                exit(2);
+            }
         }
-        afInitSampleFormat(filesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
-        afInitRate(filesetup, AF_DEFAULT_TRACK, (float) SAMPLE_RATE);
-        afInitFileFormat(filesetup, AF_FILE_WAVE);
-        afInitChannels(filesetup, AF_DEFAULT_TRACK, 1);
-    
-        outhandle = afOpenFile(OUT_FILE_NAME, "w", filesetup);
-        if (outhandle == AF_NULL_FILEHANDLE)
-        {
-            fprintf(stderr, "    Cannot create wave file '%s'\n", OUT_FILE_NAME);
-            exit(2);
-        }
-
         /* Go through all the standards */
         /* This assumes standard 0 is NULL, and TDD is the last in the list */
         for (j = 1;  j <= ADSI_STANDARD_TDD;  j++)
@@ -666,24 +673,30 @@ int main(int argc, char *argv[])
                     memset(&amp[len], 0, sizeof(int16_t)*(BLOCK_LEN - len));
                     len = BLOCK_LEN;
                 }
-                outframes = afWriteFrames(outhandle,
-                                          AF_DEFAULT_TRACK,
-                                          amp,
-                                          len);
-                if (outframes != len)
+                if (log_audio)
                 {
-                    fprintf(stderr, "    Error writing wave file\n");
-                    exit(2);
+                    outframes = afWriteFrames(outhandle,
+                                              AF_DEFAULT_TRACK,
+                                              amp,
+                                              len);
+                    if (outframes != len)
+                    {
+                        fprintf(stderr, "    Error writing wave file\n");
+                        exit(2);
+                    }
                 }
                 adsi_rx(&rx_adsi, amp, len);
             }
         }
-        if (afCloseFile(outhandle) != 0)
+        if (log_audio)
         {
-            fprintf(stderr, "    Cannot close wave file '%s'\n", OUT_FILE_NAME);
-            exit(2);
+            if (afCloseFile(outhandle) != 0)
+            {
+                fprintf(stderr, "    Cannot close wave file '%s'\n", OUT_FILE_NAME);
+                exit(2);
+            }
+            afFreeFileSetup(filesetup);
         }
-        afFreeFileSetup(filesetup);
     }
     
     printf("Tests passed.\n");
