@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: at_interpreter_tests.c,v 1.5 2006/05/24 09:19:11 steveu Exp $
+ * $Id: at_interpreter_tests.c,v 1.8 2006/10/24 13:45:28 steveu Exp $
  */
 
 /*! \file */
@@ -241,7 +240,7 @@ static const struct command_response_s general_test_seq[] =
     {"AT+FCLASS?\r", "\r\n1\r\n\r\nOK\r\n"},
     {"AT+FDD=?\r", "\r\n(0,1)\r\n\r\nOK\r\n"},                      /* T.31 8.5.3 - Double escape character replacement */ 
     {"AT+FDD?\r", "\r\n0\r\n\r\nOK\r\n"},
-    {"AT+FIT=?\r", "\r\n+FIT:(0-255),(0-255)\r\n\r\nOK\r\n"},       /* T.31 8.5.4 - DTE inactivity timeout */ 
+    {"AT+FIT=?\r", "\r\n+FIT:(0-255),(0-1)\r\n\r\nOK\r\n"},         /* T.31 8.5.4 - DTE inactivity timeout */ 
     {"AT+FIT?\r", "\r\n+FIT:0,0\r\n\r\nOK\r\n"},
     {"AT+FLO\r", "\r\nOK\r\n"},                                     /* T.31 says to implement something similar to +IFC */ 
     {"AT+FMI?\r", "\r\n" MANUFACTURER "\r\n\r\nOK\r\n"},            /* T.31 says to duplicate +GMI */ 
@@ -353,7 +352,7 @@ static const struct command_response_s general_test_seq[] =
     {"AT+VRX\r", "\r\nOK\r\n"},                                     /* V.253 10.1.3 - Voice receive state */
     {"AT+VSD\r", "\r\nOK\r\n"},                                     /* V.253 10.2.7 - Silence detection (QUIET and SILENCE) */
     {"AT+VSID=12345\r", "\r\nOK\r\n"},                              /* Extension - Set the originating number */
-    {"AT+VSID?\r", "\r\n12345\r\n\r\nOK\r\n"},                      /* Extension - Set the originating number */
+    {"AT+vsid?\r", "\r\n12345\r\n\r\nOK\r\n"},                      /* Extension - Set the originating number */
     {"AT+VSM\r", "\r\nOK\r\n"},                                     /* V.253 10.2.8 - Compression method selection */
     {"AT+VSP\r", "\r\nOK\r\n"},                                     /* V.253 10.5.1 - Voice speakerphone state */
     {"AT+VTA\r", "\r\nOK\r\n"},                                     /* V.253 10.5.4 - Train acoustic echo-canceller */ 
@@ -481,19 +480,22 @@ static int modem_call_control(at_state_t *s, void *user_data, int op, const char
 {
     switch (op)
     {
+    case AT_MODEM_CONTROL_CALL:
+        printf("\nModem control - Dialing '%s'\n", num);
+        break;
     case AT_MODEM_CONTROL_ANSWER:
         printf("\nModem control - Answering\n");
         /* Force an error response, so we get something well defined for the test. */
         return -1;
-    case AT_MODEM_CONTROL_CALL:
-        printf("\nModem control - Dialing '%s'\n", num);
-        break;
     case AT_MODEM_CONTROL_HANGUP:
         printf("\nModem control - Hanging up\n");
         break;
     case AT_MODEM_CONTROL_OFFHOOK:
         printf("\nModem control - Going off hook\n");
         break;
+    case AT_MODEM_CONTROL_ONHOOK:
+        printf("\nModem control - Going on hook\n");
+        break;
     case AT_MODEM_CONTROL_DTR:
         printf("\nModem control - DTR %d\n", (int) (intptr_t) num);
         break;
@@ -512,6 +514,15 @@ static int modem_call_control(at_state_t *s, void *user_data, int op, const char
     case AT_MODEM_CONTROL_DSR:
         printf("\nModem control - DSR %d\n", (int) (intptr_t) num);
         break;
+    case AT_MODEM_CONTROL_SETID:
+        printf("\nModem control - Set ID '%s'\n", num);
+        break;
+    case AT_MODEM_CONTROL_RESTART:
+        printf("\nModem control - Restart %d\n", (int) (intptr_t) num);
+        break;
+    case AT_MODEM_CONTROL_DTE_TIMEOUT:
+        printf("\nModem control - Set DTE timeout %d\n", (int) (intptr_t) num);
+        break;
     default:
         printf("\nModem control - operation %d\n", op);
         break;
