@@ -13,9 +13,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +25,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: echo_tests.c,v 1.23 2006/05/14 08:34:18 steveu Exp $
+ * $Id: echo_tests.c,v 1.26 2006/10/24 13:45:29 steveu Exp $
  */
 
 /*! \page echo_can_tests_page Line echo cancellation for voice tests
@@ -210,7 +209,7 @@ static int16_t signal_amp(signal_source_t *sig)
 }
 /*- End of function --------------------------------------------------------*/
 
-static inline int16_t alaw_munge(int16_t amp)
+static inline int16_t codec_munge(int16_t amp)
 {
     return alaw_to_linear(linear_to_alaw(amp));
 }
@@ -260,17 +259,17 @@ static int16_t channel_model(int16_t *new_local, int16_t *new_far, int16_t local
 
     /* The local tx signal will usually have gone through an A-law munging before
        it reached the line's analogue area, where the echo occurs. */
-    local = alaw_munge(local);
+    local = codec_munge(local);
     /* Now we need to model the echo. We only model a single analogue segment, as per
        the G.168 spec. However, there will generally be near end and far end analogue/echoey
        segments in the real world, unless an end is purely digital. */
     echo = fir32(&line_model, local/8);
     /* The far end signal will have been through an A-law munging, although
        this should not affect things. */
-    rx = echo + alaw_munge(far);
+    rx = echo + codec_munge(far);
     /* This mixed echo and far end signal will have been through an A-law munging
        when it came back into the digital network. */
-    rx = alaw_munge(rx);
+    rx = codec_munge(rx);
     if (new_far)
         *new_far = rx;
     if (new_local)
@@ -933,7 +932,7 @@ int main(int argc, char *argv[])
         {
             tx = signal_amp(&local_css);
             rx = 0;
-            tx = alaw_munge(tx);
+            tx = codec_munge(tx);
             clean = echo_can_update(ctx, tx, rx);
             put_residue(clean);
         }
