@@ -11,9 +11,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: fax_decode.c,v 1.17 2006/05/19 12:23:32 steveu Exp $
+ * $Id: fax_decode.c,v 1.21 2006/10/24 13:22:03 steveu Exp $
  */
 
 /*! \page fax_decode_page FAX decoder
@@ -224,7 +223,7 @@ static void v29_put_bit(void *user_data, int bit)
         return;
     }
 
-    end_of_page = t4_rx_putbit(&t4_state, bit);
+    end_of_page = t4_rx_put_bit(&t4_state, bit);
     if (end_of_page)
     {
         t4_end();
@@ -295,7 +294,7 @@ int main(int argc, char *argv[])
     span_log_set_protocol(&t30_dummy.logging, "T.30");
 
     hdlc_rx_init(&hdlcrx, FALSE, FALSE, 1, hdlc_accept, NULL);
-    fsk_rx_init(&fsk, &preset_fsk_specs[FSK_V21CH2], TRUE, (put_bit_func_t) hdlc_rx_bit, &hdlcrx);
+    fsk_rx_init(&fsk, &preset_fsk_specs[FSK_V21CH2], TRUE, 0, (put_bit_func_t) hdlc_rx_put_bit, &hdlcrx);
 #if defined(ENABLE_V17)
     v17_rx_init(&v17, 14400, v17_put_bit, NULL);
 #endif
@@ -308,6 +307,10 @@ int main(int argc, char *argv[])
     v29_rx_signal_cutoff(&v29, -45.0);
     v27ter_rx_signal_cutoff(&v27ter, -40.0);
 
+    //span_log_init(&v29.logging, SPAN_LOG_FLOW, NULL);
+    //span_log_set_protocol(&v29.logging, "V.29");
+    //span_log_set_level(&v29.logging, SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_SHOW_TAG | SPAN_LOG_FLOW);
+
     if (t4_rx_init(&t4_state, "fax_decode.tif", T4_COMPRESSION_ITU_T4_2D))
     {
         printf("Failed to init\n");
