@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: fsk_tests.c,v 1.23 2006/05/05 10:51:53 steveu Exp $
+ * $Id: fsk_tests.c,v 1.29 2006/10/24 13:45:29 steveu Exp $
  */
 
 /*! \page fsk_tests_page FSK modem tests
@@ -57,6 +56,7 @@ These tests allow either:
 #include <tiffio.h>
 
 #include "spandsp.h"
+#include "test_utils.h"
 #include "line_model.h"
 
 #define BLOCK_LEN           160
@@ -170,14 +170,23 @@ int main(int argc, char *argv[])
     int line_model_no;
     int modem_under_test_1;
     int modem_under_test_2;
+    int log_audio;
+    int channel_codec;
 
+    channel_codec = MUNGE_CODEC_NONE;
     line_model_no = 0;
     decode_test_file = NULL;
     noise_sweep = FALSE;
     modem_under_test_1 = FSK_V21CH1;
     modem_under_test_2 = FSK_V21CH2;
+    log_audio = FALSE;
     for (i = 1;  i < argc;  i++)
     {
+        if (strcmp(argv[i], "-c") == 0)
+        {
+            channel_codec = atoi(argv[++i]);
+            continue;
+        }
         if (strcmp(argv[i], "-d") == 0)
         {
             i++;
@@ -203,17 +212,28 @@ int main(int argc, char *argv[])
             noise_sweep = TRUE;
             continue;
         }
+        if (strcmp(argv[i], "-l") == 0)
+        {
+            log_audio = TRUE;
+            continue;
+        }
     }
-    if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
-    {
-        fprintf(stderr, "    Failed to create file setup\n");
-        exit(2);
-    }
-    afInitSampleFormat(filesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
-    afInitRate(filesetup, AF_DEFAULT_TRACK, (float) SAMPLE_RATE);
-    afInitFileFormat(filesetup, AF_FILE_WAVE);
-    afInitChannels(filesetup, AF_DEFAULT_TRACK, 2);
 
+    filesetup = AF_NULL_FILESETUP;
+    outhandle = AF_NULL_FILEHANDLE;
+
+    if (log_audio)
+    {
+        if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
+        {
+            fprintf(stderr, "    Failed to create file setup\n");
+            exit(2);
+        }
+        afInitSampleFormat(filesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
+        afInitRate(filesetup, AF_DEFAULT_TRACK, (float) SAMPLE_RATE);
+        afInitFileFormat(filesetup, AF_FILE_WAVE);
+        afInitChannels(filesetup, AF_DEFAULT_TRACK, 2);
+    }
     noise_level = -200;
     bits_per_test = 0;
     inhandle = NULL;
@@ -224,7 +244,7 @@ int main(int argc, char *argv[])
             fprintf(stderr, "    Cannot open wave file '%s'\n", decode_test_file);
             exit(2);
         }
-        fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_1], TRUE, put_bit, NULL);
+        fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_1], TRUE, 0, put_bit, NULL);
         test_bps = preset_fsk_specs[modem_under_test_1].baud_rate;
     }
     else
@@ -233,12 +253,12 @@ int main(int argc, char *argv[])
         if (modem_under_test_1 >= 0)
         {
             fsk_tx_init(&caller_tx, &preset_fsk_specs[modem_under_test_1], (get_bit_func_t) bert_get_bit, &caller_bert);
-            fsk_rx_init(&answerer_rx, &preset_fsk_specs[modem_under_test_1], TRUE, (put_bit_func_t) bert_put_bit, &answerer_bert);
+            fsk_rx_init(&answerer_rx, &preset_fsk_specs[modem_under_test_1], TRUE, 0, (put_bit_func_t) bert_put_bit, &answerer_bert);
         }
         if (modem_under_test_2 >= 0)
         {
             fsk_tx_init(&answerer_tx, &preset_fsk_specs[modem_under_test_2], (get_bit_func_t) bert_get_bit, &answerer_bert);
-            fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_2], TRUE, (put_bit_func_t) bert_put_bit, &caller_bert);
+            fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_2], TRUE, 0, (put_bit_func_t) bert_put_bit, &caller_bert);
         }
         test_bps = preset_fsk_specs[modem_under_test_1].baud_rate;
 
@@ -249,16 +269,19 @@ int main(int argc, char *argv[])
         bert_set_report(&caller_bert, 100000, reporter, (void *) (intptr_t) 1);
         bert_init(&answerer_bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
         bert_set_report(&answerer_bert, 100000, reporter, (void *) (intptr_t) 2);
-        if ((model = both_ways_line_model_init(line_model_no, noise_level, line_model_no, noise_level)) == NULL)
+        if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, noise_level, channel_codec)) == NULL)
         {
             fprintf(stderr, "    Failed to create line model\n");
             exit(2);
         }
     }
-    if ((outhandle = afOpenFile(OUTPUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
+    if (log_audio)
     {
-        fprintf(stderr, "    Cannot create wave file '%s'\n", OUTPUT_FILE_NAME);
-        exit(2);
+        if ((outhandle = afOpenFile(OUTPUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
+        {
+            fprintf(stderr, "    Cannot create wave file '%s'\n", OUTPUT_FILE_NAME);
+            exit(2);
+        }
     }
 
     if (modem_under_test_1 >= 0)
@@ -312,15 +335,18 @@ int main(int argc, char *argv[])
             out_amp[2*i + 1] = answerer_model_amp[i];
         for (  ;  i < BLOCK_LEN;  i++)
             out_amp[2*i + 1] = 0;
-
-        outframes = afWriteFrames(outhandle,
-                                  AF_DEFAULT_TRACK,
-                                  out_amp,
-                                  BLOCK_LEN);
-        if (outframes != BLOCK_LEN)
+        
+        if (log_audio)
         {
-            fprintf(stderr, "    Error writing wave file\n");
-            exit(2);
+            outframes = afWriteFrames(outhandle,
+                                      AF_DEFAULT_TRACK,
+                                      out_amp,
+                                      BLOCK_LEN);
+            if (outframes != BLOCK_LEN)
+            {
+                fprintf(stderr, "    Error writing wave file\n");
+                exit(2);
+            }
         }
 
         if (samples < BLOCK_LEN)
@@ -357,25 +383,28 @@ int main(int argc, char *argv[])
 
             /* Put a little silence between the chunks in the file. */
             memset(out_amp, 0, sizeof(out_amp));
-            for (i = 0;  i < 200;  i++)
+            if (log_audio)
             {
-                outframes = afWriteFrames(outhandle,
-                                          AF_DEFAULT_TRACK,
-                                          out_amp,
-                                          BLOCK_LEN);
+                for (i = 0;  i < 200;  i++)
+                {
+                    outframes = afWriteFrames(outhandle,
+                                              AF_DEFAULT_TRACK,
+                                              out_amp,
+                                              BLOCK_LEN);
+                }
             }
             if (modem_under_test_1 >= 0)
             {
                 fsk_tx_init(&caller_tx, &preset_fsk_specs[modem_under_test_1], (get_bit_func_t) bert_get_bit, &caller_bert);
-                fsk_rx_init(&answerer_rx, &preset_fsk_specs[modem_under_test_1], TRUE, (put_bit_func_t) bert_put_bit, &answerer_bert);
+                fsk_rx_init(&answerer_rx, &preset_fsk_specs[modem_under_test_1], TRUE, 0, (put_bit_func_t) bert_put_bit, &answerer_bert);
             }
             if (modem_under_test_2 >= 0)
             {
                 fsk_tx_init(&answerer_tx, &preset_fsk_specs[modem_under_test_2], (get_bit_func_t) bert_get_bit, &answerer_bert);
-                fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_2], TRUE, (put_bit_func_t) bert_put_bit, &caller_bert);
+                fsk_rx_init(&caller_rx, &preset_fsk_specs[modem_under_test_2], TRUE, 0, (put_bit_func_t) bert_put_bit, &caller_bert);
             }
             noise_level++;
-            if ((model = both_ways_line_model_init(line_model_no, noise_level, line_model_no, noise_level)) == NULL)
+            if ((model = both_ways_line_model_init(line_model_no, (float) noise_level, line_model_no, noise_level, channel_codec)) == NULL)
             {
                 fprintf(stderr, "    Failed to create line model\n");
                 exit(2);
@@ -387,10 +416,14 @@ int main(int argc, char *argv[])
         }
     }
 
-    if (afCloseFile(outhandle) != 0)
+    if (log_audio)
     {
-        fprintf(stderr, "    Cannot close wave file '%s'\n", OUTPUT_FILE_NAME);
-        exit(2);
+        if (afCloseFile(outhandle) != 0)
+        {
+            fprintf(stderr, "    Cannot close wave file '%s'\n", OUTPUT_FILE_NAME);
+            exit(2);
+        }
+        afFreeFileSetup(filesetup);
     }
     if (decode_test_file)
     {
@@ -404,7 +437,6 @@ int main(int argc, char *argv[])
     {
         printf("Tests passed.\n");
     }
-    afFreeFileSetup(filesetup);
     return  0;
 }
 /*- End of function --------------------------------------------------------*/
