@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_tests.c,v 1.55 2006/05/14 08:34:19 steveu Exp $
+ * $Id: v27ter_tests.c,v 1.69 2006/10/24 13:22:04 steveu Exp $
  */
 
 /*! \page v27ter_tests_page V.27ter modem tests
@@ -59,15 +58,17 @@ display of modem status is maintained.
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
-#include <math.h>
+#include <tgmath.h>
 #include <assert.h>
 #include <audiofile.h>
 #include <tiffio.h>
 
 #include "spandsp.h"
+#include "test_utils.h"
 #include "line_model.h"
 #if defined(ENABLE_GUI)
 #include "modem_monitor.h"
+#include "line_model_monitor.h"
 #endif
 
 #define BLOCK_LEN       160
@@ -170,11 +171,11 @@ static int v27tergetbit(void *user_data)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void qam_report(void *user_data, const complex_t *constel, const complex_t *target, int symbol)
+static void qam_report(void *user_data, const complexf_t *constel, const complexf_t *target, int symbol)
 {
     int i;
     int len;
-    complex_t *coeffs;
+    complexf_t *coeffs;
     float fpower;
     float error;
     v27ter_rx_state_t *rx;
@@ -212,8 +213,8 @@ static void qam_report(void *user_data, const complex_t *constel, const complex_
         len = v27ter_rx_equalizer_state(rx, &coeffs);
         printf("Equalizer B:\n");
         for (i = 0;  i < len;  i++)
-            printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, power(&coeffs[i]));
-        printf("Gardtest %d %d %d\n", symbol_no, rx->gardner_total_correction, rx->gardner_integrate);
+            printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, powerf(&coeffs[i]));
+        printf("Gardtest %d %f %d\n", symbol_no, v27ter_rx_symbol_timing_correction(rx), rx->gardner_integrate);
         printf("Carcar %d %f\n", symbol_no, v27ter_rx_carrier_frequency(rx));
 #if defined(ENABLE_GUI)
         if (use_gui)
@@ -233,7 +234,7 @@ static void qam_report(void *user_data, const complex_t *constel, const complex_
         len = v27ter_rx_equalizer_state(rx, &coeffs);
         printf("Equalizer A:\n");
         for (i = 0;  i < len;  i++)
-            printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, power(&coeffs[i]));
+            printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, powerf(&coeffs[i]));
 #if defined(ENABLE_GUI)
         if (use_gui)
             qam_monitor_update_equalizer(qam_monitor, coeffs, len);
@@ -255,21 +256,26 @@ int main(int argc, char *argv[])
     int outframes;
     int samples;
     int i;
-    int test_bps;
     int tep;
+    int test_bps;
     int noise_level;
     int signal_level;
     int bits_per_test;
     int line_model_no;
     int block;
+    int log_audio;
+    int channel_codec;
 
+    channel_codec = MUNGE_CODEC_NONE;
     test_bps = 4800;
     tep = FALSE;
     line_model_no = 0;
     decode_test_file = NULL;
+    use_gui = FALSE;
     noise_level = -70;
     signal_level = -13;
     bits_per_test = 50000;
+    log_audio = FALSE;
     for (i = 1;  i < argc;  i++)
     {
         if (strcmp(argv[i], "-b") == 0)
@@ -277,6 +283,11 @@ int main(int argc, char *argv[])
             bits_per_test = atoi(argv[++i]);
             continue;
         }
+        if (strcmp(argv[i], "-c") == 0)
+        {
+            channel_codec = atoi(argv[++i]);
+            continue;
+        }
         if (strcmp(argv[i], "-d") == 0)
         {
             decode_test_file = argv[++i];
@@ -292,6 +303,11 @@ int main(int argc, char *argv[])
             use_gui = TRUE;
             continue;
         }
+        if (strcmp(argv[i], "-l") == 0)
+        {
+            log_audio = TRUE;
+            continue;
+        }
         if (strcmp(argv[i], "-m") == 0)
         {
             line_model_no = atoi(argv[++i]);
@@ -317,18 +333,28 @@ int main(int argc, char *argv[])
             exit(2);
         }
     }
-    if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
-    {
-        fprintf(stderr, "    Failed to create file setup\n");
-        exit(2);
-    }
-    afInitSampleFormat(filesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
-    afInitRate(filesetup, AF_DEFAULT_TRACK, (float) SAMPLE_RATE);
-    afInitFileFormat(filesetup, AF_FILE_WAVE);
-    afInitChannels(filesetup, AF_DEFAULT_TRACK, 1);
-
     inhandle = NULL;
     outhandle = NULL;
+
+    filesetup = AF_NULL_FILESETUP;
+    if (log_audio)
+    {
+        if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
+        {
+            fprintf(stderr, "    Failed to create file setup\n");
+            exit(2);
+        }
+        afInitSampleFormat(filesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
+        afInitRate(filesetup, AF_DEFAULT_TRACK, (float) SAMPLE_RATE);
+        afInitFileFormat(filesetup, AF_FILE_WAVE);
+        afInitChannels(filesetup, AF_DEFAULT_TRACK, 1);
+        if ((outhandle = afOpenFile(OUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
+        {
+            fprintf(stderr, "    Cannot create wave file '%s'\n", OUT_FILE_NAME);
+            exit(2);
+        }
+    }
+
     if (decode_test_file)
     {
         /* We will decode the audio from a wave file. */
@@ -341,12 +367,6 @@ int main(int argc, char *argv[])
     else
     {
         /* We will generate V.27ter audio, and add some noise to it. */
-        if ((outhandle = afOpenFile(OUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
-        {
-            fprintf(stderr, "    Cannot create wave file '%s'\n", OUT_FILE_NAME);
-            exit(2);
-        }
-
         v27ter_tx_init(&tx, test_bps, tep, v27tergetbit, NULL);
         v27ter_tx_power(&tx, signal_level);
         /* Move the carrier off a bit */
@@ -355,7 +375,7 @@ int main(int argc, char *argv[])
         bert_init(&bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
         bert_set_report(&bert, 10000, reporter, NULL);
 
-        if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level)) == NULL)
+        if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec)) == NULL)
         {
             fprintf(stderr, "    Failed to create line model\n");
             exit(2);
@@ -364,16 +384,23 @@ int main(int argc, char *argv[])
 
     v27ter_rx_init(&rx, test_bps, v27terputbit, NULL);
     v27ter_rx_set_qam_report_handler(&rx, qam_report, (void *) &rx);
-    rx.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
-    rx.logging.tag = "V.27ter rx";
+    span_log_set_level(&rx.logging, SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW);
+    span_log_set_tag(&rx.logging, "V.27ter-rx");
 
 #if defined(ENABLE_GUI)
     if (use_gui)
+    {
         qam_monitor = qam_monitor_init(2.0, NULL);
+        if (!decode_test_file)
+        {
+            start_line_model_monitor(129);
+            line_model_monitor_line_model_update(line_model->near_filter, line_model->near_filter_len);
+        }
+    }
 #endif
 
     memset(&latest_results, 0, sizeof(latest_results));
-    for (block = 0; ;  block++)
+    for (block = 0;  ;  block++)
     {
         if (decode_test_file)
         {
@@ -430,24 +457,31 @@ int main(int argc, char *argv[])
                 bert_init(&bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
                 bert_set_report(&bert, 10000, reporter, NULL);
                 one_way_line_model_release(line_model);
-                if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level)) == NULL)
+                if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec)) == NULL)
                 {
                     fprintf(stderr, "    Failed to create line model\n");
                     exit(2);
                 }
             }
 
-            outframes = afWriteFrames(outhandle,
-                                      AF_DEFAULT_TRACK,
-                                      gen_amp,
-                                      samples);
-            if (outframes != samples)
+            if (log_audio)
             {
-                fprintf(stderr, "    Error writing wave file\n");
-                exit(2);
+                outframes = afWriteFrames(outhandle,
+                                          AF_DEFAULT_TRACK,
+                                          gen_amp,
+                                          samples);
+                if (outframes != samples)
+                {
+                    fprintf(stderr, "    Error writing wave file\n");
+                    exit(2);
+                }
             }
             one_way_line_model(line_model, amp, gen_amp, samples);
         }
+#if defined(ENABLE_GUI)
+        if (use_gui  &&  !decode_test_file)
+            line_model_monitor_line_spectrum_update(amp, samples);
+#endif
         v27ter_rx(&rx, amp, samples);
         if (decode_test_file == NULL  &&  block%500 == 0)
             printf("Noise level is %d\n", noise_level);
@@ -465,13 +499,8 @@ int main(int argc, char *argv[])
         fprintf(stderr, "At completion:\n");
         fprintf(stderr, "Final result %ddBm0, %d bits, %d bad bits, %d resyncs\n", signal_level, bert_results.total_bits, bert_results.bad_bits, bert_results.resyncs);
         fprintf(stderr, "Last report  %ddBm0, %d bits, %d bad bits, %d resyncs\n", signal_level, latest_results.total_bits, latest_results.bad_bits, latest_results.resyncs);
-        if (afCloseFile(outhandle))
-        {
-            fprintf(stderr, "    Cannot close wave file '%s'\n", OUT_FILE_NAME);
-            exit(2);
-        }
         one_way_line_model_release(line_model);
-        if (signal_level > -38)
+        if (signal_level > -43)
         {
             printf("Tests failed.\n");
             exit(2);
@@ -479,7 +508,15 @@ int main(int argc, char *argv[])
 
         printf("Tests passed.\n");
     }
-    afFreeFileSetup(filesetup);
+    if (log_audio)
+    {
+        if (afCloseFile(outhandle))
+        {
+            fprintf(stderr, "    Cannot close wave file '%s'\n", OUT_FILE_NAME);
+            exit(2);
+        }
+        afFreeFileSetup(filesetup);
+    }
     return  0;
 }
 /*- End of function --------------------------------------------------------*/
