@@ -10,9 +10,8 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU General Public License version 2, as
+ * published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v29_tests.c,v 1.63 2006/05/14 08:34:19 steveu Exp $
+ * $Id: v29_tests.c,v 1.81 2006/10/24 13:45:29 steveu Exp $
  */
 
 /*! \page v29_tests_page V.29 modem tests
@@ -65,9 +64,11 @@ display of modem status is maintained.
 #include <tiffio.h>
 
 #include "spandsp.h"
+#include "test_utils.h"
 #include "line_model.h"
 #if defined(ENABLE_GUI)
 #include "modem_monitor.h"
+#include "line_model_monitor.h"
 #endif
 
 #define BLOCK_LEN       160
@@ -136,7 +137,7 @@ static void v29putbit(void *user_data, int bit)
     v29_rx_state_t *rx;
     int i;
     int len;
-    complex_t *coeffs;
+    complexf_t *coeffs;
     
     rx = (v29_rx_state_t *) user_data;
     if (bit < 0)
@@ -152,7 +153,7 @@ static void v29putbit(void *user_data, int bit)
             len = v29_rx_equalizer_state(rx, &coeffs);
             printf("Equalizer:\n");
             for (i = 0;  i < len;  i++)
-                printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, power(&coeffs[i]));
+                printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, powerf(&coeffs[i]));
             break;
         case PUTBIT_CARRIER_UP:
             printf("Carrier up\n");
@@ -180,14 +181,15 @@ static int v29getbit(void *user_data)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void qam_report(void *user_data, const complex_t *constel, const complex_t *target, int symbol)
+static void qam_report(void *user_data, const complexf_t *constel, const complexf_t *target, int symbol)
 {
     int i;
     int len;
-    complex_t *coeffs;
+    complexf_t *coeffs;
     float fpower;
     v29_rx_state_t *rx;
     static float smooth_power = 0.0;
+    static int update_interval = 100;
 
     rx = (v29_rx_state_t *) user_data;
     if (constel)
@@ -216,18 +218,18 @@ static void qam_report(void *user_data, const complex_t *constel, const complex_
                v29_rx_signal_power(rx),
                v29_rx_symbol_timing_correction(rx));
         symbol_no++;
-    }
-    else
-    {
-        //printf("Gardner step %d\n", symbol);
-        len = v29_rx_equalizer_state(rx, &coeffs);
-        printf("Equalizer A:\n");
-        for (i = 0;  i < len;  i++)
-            printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, power(&coeffs[i]));
+        if (--update_interval <= 0)
+        {
+            len = v29_rx_equalizer_state(rx, &coeffs);
+            printf("Equalizer A:\n");
+            for (i = 0;  i < len;  i++)
+                printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, powerf(&coeffs[i]));
 #if defined(ENABLE_GUI)
-        if (use_gui)
-            qam_monitor_update_equalizer(qam_monitor, coeffs, len);
+            if (use_gui)
+                qam_monitor_update_equalizer(qam_monitor, coeffs, len);
 #endif
+            update_interval = 100;
+        }
     }
 }
 /*- End of function --------------------------------------------------------*/
@@ -252,7 +254,10 @@ int main(int argc, char *argv[])
     int bits_per_test;
     int line_model_no;
     int block;
+    int log_audio;
+    int channel_codec;
     
+    channel_codec = MUNGE_CODEC_NONE;
     test_bps = 9600;
     tep = FALSE;
     line_model_no = 0;
@@ -261,6 +266,7 @@ int main(int argc, char *argv[])
     noise_level = -70;
     signal_level = -13;
     bits_per_test = 50000;
+    log_audio = FALSE;
     for (i = 1;  i < argc;  i++)
     {
         if (strcmp(argv[i], "-b") == 0)
@@ -268,6 +274,11 @@ int main(int argc, char *argv[])
             bits_per_test = atoi(argv[++i]);
             continue;
         }
+        if (strcmp(argv[i], "-c") == 0)
+        {
+            channel_codec = atoi(argv[++i]);
+            continue;
+        }
         if (strcmp(argv[i], "-d") == 0)
         {
             decode_test_file = argv[++i];
@@ -283,6 +294,11 @@ int main(int argc, char *argv[])
             use_gui = TRUE;
             continue;
         }
+        if (strcmp(argv[i], "-l") == 0)
+        {
+            log_audio = TRUE;
+            continue;
+        }
         if (strcmp(argv[i], "-m") == 0)
         {
             line_model_no = atoi(argv[++i]);
@@ -310,18 +326,28 @@ int main(int argc, char *argv[])
             exit(2);
         }
     }
-    if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
-    {
-        fprintf(stderr, "    Failed to create file setup\n");
-        exit(2);
-    }
-    afInitSampleFormat(filesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
-    afInitRate(filesetup, AF_DEFAULT_TRACK, (float) SAMPLE_RATE);
-    afInitFileFormat(filesetup, AF_FILE_WAVE);
-    afInitChannels(filesetup, AF_DEFAULT_TRACK, 1);
-
     inhandle = NULL;
     outhandle = NULL;
+
+    filesetup = AF_NULL_FILESETUP;
+    if (log_audio)
+    {
+        if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
+        {
+            fprintf(stderr, "    Failed to create file setup\n");
+            exit(2);
+        }
+        afInitSampleFormat(filesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
+        afInitRate(filesetup, AF_DEFAULT_TRACK, (float) SAMPLE_RATE);
+        afInitFileFormat(filesetup, AF_FILE_WAVE);
+        afInitChannels(filesetup, AF_DEFAULT_TRACK, 1);
+        if ((outhandle = afOpenFile(OUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
+        {
+            fprintf(stderr, "    Cannot create wave file '%s'\n", OUT_FILE_NAME);
+            exit(2);
+        }
+    }
+
     if (decode_test_file)
     {
         /* We will decode the audio from a wave file. */
@@ -334,11 +360,6 @@ int main(int argc, char *argv[])
     else
     {
         /* We will generate V.29 audio, and add some noise to it. */
-        if ((outhandle = afOpenFile(OUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
-        {
-            fprintf(stderr, "    Cannot create wave file '%s'\n", OUT_FILE_NAME);
-            exit(2);
-        }
         v29_tx_init(&tx, test_bps, tep, v29getbit, NULL);
         v29_tx_power(&tx, signal_level);
         /* Move the carrier off a bit */
@@ -348,7 +369,7 @@ int main(int argc, char *argv[])
         bert_init(&bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
         bert_set_report(&bert, 10000, reporter, NULL);
 
-        if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level)) == NULL)
+        if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec)) == NULL)
         {
             fprintf(stderr, "    Failed to create line model\n");
             exit(2);
@@ -356,15 +377,23 @@ int main(int argc, char *argv[])
     }
 
     v29_rx_init(&rx, test_bps, v29putbit, &rx);
+    v29_rx_signal_cutoff(&rx, -45.5);
     v29_rx_set_qam_report_handler(&rx, qam_report, (void *) &rx);
     /* Rotate the starting phase */
     rx.carrier_phase = 0x80000000;
-    rx.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
-    rx.logging.tag = "V.29-rx";
+    span_log_set_level(&rx.logging, SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW);
+    span_log_set_tag(&rx.logging, "V.29-rx");
 
 #if defined(ENABLE_GUI)
     if (use_gui)
+    {
         qam_monitor = qam_monitor_init(6.0, NULL);
+        if (!decode_test_file)
+        {
+            start_line_model_monitor(129);
+            line_model_monitor_line_model_update(line_model->near_filter, line_model->near_filter_len);
+        }
+    }
 #endif
 
     memset(&latest_results, 0, sizeof(latest_results));
@@ -418,26 +447,34 @@ int main(int argc, char *argv[])
                 v29_tx_restart(&tx, test_bps, tep);
                 v29_tx_power(&tx, signal_level);
                 v29_rx_restart(&rx, test_bps, FALSE);
+                rx.eq_put_step = rand()%(48*10/3);
                 bert_init(&bert, bits_per_test, BERT_PATTERN_ITU_O152_11, test_bps, 20);
                 bert_set_report(&bert, 10000, reporter, NULL);
                 one_way_line_model_release(line_model);
-                if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level)) == NULL)
+                if ((line_model = one_way_line_model_init(line_model_no, (float) noise_level, channel_codec)) == NULL)
                 {
                     fprintf(stderr, "    Failed to create line model\n");
                     exit(2);
                 }
             }
-            one_way_line_model(line_model, amp, gen_amp, samples);
-            outframes = afWriteFrames(outhandle,
-                                      AF_DEFAULT_TRACK,
-                                      amp,
-                                      samples);
-            if (outframes != samples)
+            if (log_audio)
             {
-                fprintf(stderr, "    Error writing wave file\n");
-                exit(2);
+                outframes = afWriteFrames(outhandle,
+                                          AF_DEFAULT_TRACK,
+                                          gen_amp,
+                                          samples);
+                if (outframes != samples)
+                {
+                    fprintf(stderr, "    Error writing wave file\n");
+                    exit(2);
+                }
             }
+            one_way_line_model(line_model, amp, gen_amp, samples);
         }
+#if defined(ENABLE_GUI)
+        if (use_gui  &&  !decode_test_file)
+            line_model_monitor_line_spectrum_update(amp, samples);
+#endif
         v29_rx(&rx, amp, samples);
         if (decode_test_file == NULL  &&  block%500 == 0)
             printf("Noise level is %d\n", noise_level);
@@ -455,14 +492,9 @@ int main(int argc, char *argv[])
         fprintf(stderr, "At completion:\n");
         fprintf(stderr, "Final result %ddBm0, %d bits, %d bad bits, %d resyncs\n", signal_level, bert_results.total_bits, bert_results.bad_bits, bert_results.resyncs);
         fprintf(stderr, "Last report  %ddBm0, %d bits, %d bad bits, %d resyncs\n", signal_level, latest_results.total_bits, latest_results.bad_bits, latest_results.resyncs);
-        if (afCloseFile(outhandle))
-        {
-            fprintf(stderr, "    Cannot close wave file '%s'\n", OUT_FILE_NAME);
-            exit(2);
-        }
         one_way_line_model_release(line_model);
         
-        if (signal_level > -39)
+        if (signal_level > -43)
         {
             printf("Tests failed.\n");
             exit(2);
@@ -470,7 +502,15 @@ int main(int argc, char *argv[])
 
         printf("Tests passed.\n");
     }
-    afFreeFileSetup(filesetup);
+    if (log_audio)
+    {
+        if (afCloseFile(outhandle))
+        {
+            fprintf(stderr, "    Cannot close wave file '%s'\n", OUT_FILE_NAME);
+            exit(2);
+        }
+        afFreeFileSetup(filesetup);
+    }
     return  0;
 }
 /*- End of function --------------------------------------------------------*/
